/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.faces.cms.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import javax.faces.application.FacesMessage;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Repository;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.ValueFormatException;

import org.exoplatform.Constants;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.cms.CMSUtils;
import org.exoplatform.faces.core.Selector;
import org.exoplatform.faces.core.SelectorHandler;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.services.jcr.RepositoryService ;

/**
 * Wed, Dec 22, 2003 @ 23:14
 * @author: Tuan Nguyen
 * @email: tuan08@users.sourceforge.net
 * @version: $Id: UISelectBox.java,v 1.6 2004/06/26 03:47:31 tuan08 Exp $
 */
public class JcrBrowser extends UIExoCommand implements Selector{
  
  static String CHANGE_NODE = "changeNode";
  static String SELECT_NODE = "selectNode";

  static Parameter[] CHANGE_NODE_PARAMS = { new Parameter(ACTION, CHANGE_NODE) };
  static Parameter[] SELECT_NODE_PARAMS = { new Parameter(ACTION, SELECT_NODE) };
  static Parameter[] CANCEL_PARAMS = { new Parameter(ACTION, CANCEL_ACTION) };
  
  static private Map smallNodeTypeIconMapping_;

  private Node cmsRootNode_;  
  private Node selectedNode_;
  private Node parentNode_;  
  private SelectorHandler handler_;
  private String info_;
  private Session session;
  private String[] parameters_;  
  private List selectFilter_ ; 

  public JcrBrowser() throws Exception {
    setRendererType("VelocityRenderer");
    setId("JcrBrowser");
    addActionListener(ChangeNodeActionListener.class, CHANGE_NODE);
    addActionListener(SelectNodeActionListener.class, SELECT_NODE);
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
  }
  
  public void setSession(Session session) throws RepositoryException{
    this.session = session;
    selectedNode_ = session.getRootNode();
    parentNode_ = null;
  }
  
  public Session getSession(String workspace){
    PortalContainer pC = PortalContainer.getInstance();
    RepositoryService rS = (RepositoryService) pC.getComponentInstanceOfType(RepositoryService.class);
    Repository repository;
    Session session = null;
    try {
      repository = rS.getRepository();
      if(workspace == null)
        session = repository.login();
      else 
        session = rS.getRepository().getSystemSession(workspace);
      selectedNode_ = session.getRootNode();
      parentNode_ = null;
    } catch (Exception e) {
      e.printStackTrace();
    } 
    return session;
  }
  
  public Session getSession(){
    if(session == null){
      session = getSession(null);
    }
    return session;
  }

  public void setHandler(SelectorHandler handler) {
    handler_ = handler;
  }
  
  public void setParameters(String[] args){
    parameters_ = args;
    String workspace = args[0];
    session = getSession(workspace);
    String path = null ;
    Node selectedNode ;
    if (args.length > 1)  path = args[1] ;    	
    else path = "/" ;
    try {
      if (path.startsWith("/")) selectedNode = (Node) session.getItem(path) ;
      else selectedNode = (Node) session.getItem("/" + path) ;
    } catch (RepositoryException e) {
      return ;
    }
    cmsRootNode_ = selectedNode ;
    selectedNode_ = selectedNode ;
  }
  
  public String[] getParameters() { return parameters_; }

  public String getInfo() { return info_; }
  public void setInfo(String info) { info_ = info; }

  public Parameter[] getChangeNodeParams() {return CHANGE_NODE_PARAMS; }
  public Parameter[] getSelectNodeParams() { return SELECT_NODE_PARAMS; }
  public Parameter[] getCancelParams() { return CANCEL_PARAMS; }

  public Node getSelectedNode() { return selectedNode_; }

  public Node getParentNode() { return parentNode_; }

  public String getInputValue(){
    String rootNodePath = null;
    try {
      rootNodePath = cmsRootNode_.getPath();
      if(selectedNode_ == null) 
        return rootNodePath;
      return selectedNode_.getPath();
    } catch (Exception e){ 
      e.printStackTrace() ;
      return rootNodePath ;
    }   
  }
  
  public List getFilter() {return selectFilter_ ; }
  public void setFilter(List filter ) { selectFilter_ = filter ; }
  
  public String getSelectedNodePath() throws Exception {
    return getInputValue();
  }

  public String getParentNodePath() throws Exception{
    if(parentNode_ == null)
      return "";
    return parentNode_.getPath();
  }
  
  public Collection getSelectedChildrenNodes() throws Exception{
    if(selectedNode_ == null)
      return Constants.EMPTY_LIST;
    NodeIterator iter = selectedNode_.getNodes();
    List nodesPaths = new ArrayList();
    while (iter.hasNext()) {
      Node node = iter.nextNode();
      nodesPaths.add(node);
    }
    return nodesPaths;
  }

  public Collection getParentChildrenNodes() throws Exception {
    if (parentNode_ == null) { 
      return Constants.EMPTY_LIST;
    }    
    NodeIterator iter = parentNode_.getNodes();
    List nodesPaths = new ArrayList();
    while (iter.hasNext()) {
      Node node = iter.nextNode();
      nodesPaths.add(node);
    }
    return nodesPaths;
  }
  
  public void changeNode(String nodePath) throws Exception {    
    if(nodePath.length() >= cmsRootNode_.getPath().length() && 
        nodePath.indexOf(cmsRootNode_.getPath()) > -1){
      selectedNode_ = (Node) getSession().getItem(nodePath);
      if (!nodePath.equals("/")) {
        String parentPath = selectedNode_.getParent().getPath() ;
        if(parentPath.indexOf(cmsRootNode_.getPath()) > -1) 
          parentNode_ = selectedNode_.getParent();
        else 
          parentNode_ = null;
      } else {
        parentNode_ = null;               
      }
    }else {
      selectedNode_ = cmsRootNode_;
      parentNode_ = null;
    }
  }
     
  public String getSmallNodeTypeIcon(Node node) throws Exception {   
    return CMSUtils.getSmallNodeTypeIcon(node) ;
  }
      
  static public class ChangeNodeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      JcrBrowser uiComp = (JcrBrowser) event.getSource();
      String nodePath = event.getParameter(OBJECTID);
      try {
        uiComp.changeNode(nodePath); 
      } catch (Exception e) {
        e.printStackTrace() ;
      }
    }
  }

  static public class SelectNodeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
    	JcrBrowser uiComp = (JcrBrowser) event.getSource();
      Information iprovider = findInformationProvider(uiComp);
    	String selectedPath = event.getParameter(OBJECTID);
      Node currentNode = uiComp.selectedNode_ ; 
    	uiComp.selectedNode_ = (Node)uiComp.getSession().getItem(selectedPath);            
    	try{
        uiComp.handler_.select(uiComp);
      }catch (ValueFormatException vfe) {
        vfe.printStackTrace() ;
        uiComp.selectedNode_ = currentNode ;
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR,"Can not add mixin because this node already have property " +
            "same name with mixin's property but diffrences definition of property's value type." +
            "Please check definition of this nodetype and definition of mixin nodetype for more detail.")) ;
        return ;
      }catch(Exception e) {
        e.printStackTrace() ;
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR,e.getMessage())) ;
        return ;
      }                  
    }
  }

  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      JcrBrowser uiComp = (JcrBrowser) event.getSource();
      uiComp.handler_.cancel(uiComp);
    }
  }
}