package org.exoplatform.portlets.content.actions.component;

import java.security.AccessControlException;
import java.util.Map;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.faces.event.ActionListener;
import javax.jcr.Node;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.cms.CMSDialogComponent;
import org.exoplatform.faces.cms.CMSUtils;
import org.exoplatform.faces.core.component.UIExoComponent;
import org.exoplatform.faces.core.component.UISimpleMulltipartForm;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.component.UIJCRExplorer;
import org.exoplatform.services.cms.JcrInputProperty;
import org.exoplatform.services.cms.actions.ActionServiceContainer;
import org.exoplatform.services.cms.templates.TemplateService;

public class UIActionForm extends UISimpleMulltipartForm implements
CMSDialogComponent {

  protected TemplateService templateService_;
  protected ActionServiceContainer actionsServiceContainer_;
  protected Node parentNode_;
  protected Node actionNode_;
  protected String nodeType_;

  public UIActionForm(TemplateService templateService,
      ActionServiceContainer actionsServiceContainer) {
    super("actionForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");

    templateService_ = templateService;
    actionsServiceContainer_ = actionsServiceContainer;

    addActionListener(CancelActionListener.class, CANCEL_ACTION);
    addActionListener(SaveActionListener.class, SAVE_ACTION);
  }

  public void createNewAction(Node parentNode, String actionType) throws Exception {
    clean();
    parentNode_ = parentNode;
    actionNode_ = null;
    nodeType_ = actionType;
  }

  public Node getNode() throws Exception {
    return actionNode_;
  }

  public boolean isNodeTypeSupported() throws Exception {
    return templateService_.isManagedNodeType(nodeType_);
  }

  public String getDialogPath() throws Exception{
    return templateService_.getTemplatePath(true, nodeType_);
  }

  public void clean() {
    clearInputs();
    getChildren().clear();
    ActionListener[] listeners = getActionListeners();
    System.out.println("\n##### listeners.length "+listeners.length);
    for (int i = 0; i < listeners.length; i++) {
      ExoActionListener listener = (ExoActionListener) listeners[i];
      String actionName = listener.getActionName();
      if (!CANCEL_ACTION.equals(actionName) && !SAVE_ACTION.equals(actionName))
        removeActionListener(listener);
    }
  }

  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIActionForm uiForm = (UIActionForm) event.getSource();
      UIJCRExplorer uiExplorer = (UIJCRExplorer)uiForm.getAncestorOfType(UIJCRExplorer.class) ;
      Information iprovider = findInformationProvider(uiForm);
      ResourceBundle res = getApplicationResourceBundle() ;
      Map sortedInputs = CMSUtils.prepareMap(uiForm);
      JcrInputProperty rootProp = (JcrInputProperty) sortedInputs.get("/node");      
      rootProp.setValue(((JcrInputProperty)sortedInputs.get("/node/exo:name")).getValue());
      String actionName = (String)((JcrInputProperty)sortedInputs.get("/node/exo:name")).getValue() ;
      if(uiForm.parentNode_.hasNode(actionName)) { 
        Object[] args = {actionName} ;
        iprovider.addMessage(
            new ExoFacesMessage(FacesMessage.SEVERITY_WARN,res.getString("UIActionForm.msg.existed-action"),args));
        return ;
      }
      if(uiForm.parentNode_.isNew()) {
        String[] args = {uiForm.parentNode_.getPath()} ;
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN,res.getString("UIActionForm.msg.unable-add-action"),args)) ;
        return ;
      }      
      try {
        uiForm.parentNode_.getSession().checkPermission(uiForm.parentNode_.getPath(), "add_node,set_property");
        uiForm.actionsServiceContainer_.addAction(uiForm.parentNode_, uiForm.nodeType_, sortedInputs);        
      } catch (AccessControlException ae) {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR,ae.getLocalizedMessage())) ;
        return ;
      }catch (Exception e) {
        //e.printStackTrace() ;
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR,e.getLocalizedMessage())) ;
        return ;
      }      
      if(!uiExplorer.getPreference().isJcrEnable()) uiExplorer.getSession().save() ;
      UIActionList uiActionList = 
        (UIActionList)((UIExoComponent) uiForm.getParent()).getSibling(UIActionList.class) ;
      uiActionList.update() ;
      ((UIExoComponent) uiForm.getParent()).setRenderedSibling(UIActionList.class);
      uiForm.clean();
    }
  }

  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIActionForm uiForm = (UIActionForm) event.getSource();
      ((UIExoComponent) uiForm.getParent()).setRenderedSibling(UIActionList.class);
    }
  }

}
