package org.exoplatform.portlets.content.contentbrowser.component;

import java.util.List;

import javax.jcr.Node;
import javax.jcr.Session;
import javax.jcr.query.Query;
import javax.portlet.PortletPreferences;

import org.exoplatform.faces.FacesUtil;
import org.exoplatform.faces.core.component.UIPortlet;
import org.exoplatform.services.cms.BasePath;
import org.exoplatform.services.cms.CmsConfigurationService;
import org.exoplatform.services.cms.queries.QueryService;
import org.exoplatform.services.cms.scripts.CmsScript;
import org.exoplatform.services.cms.scripts.DataTransfer;
import org.exoplatform.services.cms.scripts.ScriptService;
import org.exoplatform.services.cms.views.ManageViewService;
import org.exoplatform.services.jcr.RepositoryService;

public class UIContentBrowserPortlet extends UIPortlet {

  private String rootPath_ ;
  private boolean flag = true ;
  private Session session_ ;  
  private String usecase_ ;
  private String workspace_ ;  
  private String queryLanguage_ ;
  private String queryStatement_ ;
  private String queryStore_ ;
  private String scriptName_ ;

  private ScriptService scriptService_ ;
  private RepositoryService repositoryService_ ;
  private CmsConfigurationService cmsConfService_ ;
  private ManageViewService viewService_ ;

  public UIContentBrowserPortlet(RepositoryService repositoryService, ManageViewService viewService,
      CmsConfigurationService  cmsConfService, ScriptService scriptService, QueryService qservice) throws Exception {    
    repositoryService_ = repositoryService ;
    cmsConfService_ = cmsConfService ;
    scriptService_ = scriptService ;    
    viewService_ = viewService ;
    PortletPreferences prefs = FacesUtil.getPortletPreferences() ;
    usecase_ = prefs.getValue(CBUtils.USECASE,"") ;
    workspace_ = prefs.getValue(CBUtils.WORKSPACE,"") ;
    rootPath_ = prefs.getValue(CBUtils.JCR_PATH,"") ;
    queryLanguage_ = prefs.getValue(CBUtils.QUERY_LANGUAGE,"") ;
    queryStatement_ = prefs.getValue(CBUtils.QUERY_STATEMENT,"") ;
    queryStore_ = prefs.getValue(CBUtils.QUERY_STORE,"") ;
    if(queryStore_ == null) queryStore_ = "" ;
    scriptName_ = prefs.getValue(CBUtils.SCRIPT_NAME, "") ;
    String boxTemplate = prefs.getValue(CBUtils.BOX_TEMPLATE , "") ;    
    String templateName = prefs.getValue(CBUtils.TEMPLATE , "") ;    
    Node detailHome = viewService_.getTemplateHome(BasePath.CB_DETAIL_VIEW_TEMPLATES) ; 
    String documentTemplatePath = detailHome.getNode(boxTemplate).getPath() ;   
    if(usecase_.equals(CBUtils.USE_DOCUMENT)) {      
      if(!checkPath()) {
        setRendererType("VelocityRenderer");
        return ;
      }
      try {
        Node parent = (Node)session_.getItem(rootPath_) ;
        String documentName = prefs.getValue(CBUtils.DOCUMENT_NAME,"") ;
        Node document = parent.getNode(documentName) ;
        if(!CBUtils.isDocument(document.getPrimaryNodeType())) {
          throw new Exception() ;
        }        
        setRendererType("ChildrenRenderer") ;
        UIDetailDocument uiDetailDocument = (UIDetailDocument)addChild(UIDetailDocument.class) ;
        String uri = workspace_+ CBUtils.COLON + document.getPath() ;
        uiDetailDocument.setNodeByUri(uri) ;
        uiDetailDocument.setRendered(true) ;
        uiDetailDocument.setTemplatePath(documentTemplatePath) ;
      }catch (Throwable e) {
        setRendererType("VelocityRenderer");        
      }      
    } else if(usecase_.equals(CBUtils.FORM_PATH) || usecase_.equals(CBUtils.USE_JCR_QUERY)){
      if(usecase_.equals(CBUtils.FORM_PATH)) {
        flag = checkPath() ;
      }else {
        if(queryStatement_.trim().equals("") && queryStore_.equals("")){
          flag = false;
        }        
      }
      if(!flag) {
        setRendererType("VelocityRenderer");
        return ;
      }          
      setRendererType("ChildrenRenderer") ;
      UIContentExplorer contentExplorer = (UIContentExplorer)addChild(UIContentExplorer.class) ;      
      if(rootPath_.startsWith("/jcr:system")) {          
        session_ = repositoryService_.getRepository().getSystemSession(cmsConfService_.getWorkspace()) ;          
      } else {        
        session_ = repositoryService_.getRepository().login(workspace_) ;
      }    
      String contentListTemplatePath = "" ;
      if(usecase_.equals(CBUtils.FORM_PATH)) {
        contentListTemplatePath = 
          viewService_.getTemplateHome(BasePath.CB_PATH_TEMPLATES).getNode(templateName).getPath() ;
      }else if(usecase_.equals(CBUtils.USE_JCR_QUERY)) {
        contentListTemplatePath = 
          viewService_.getTemplateHome(BasePath.CB_QUERY_TEMPLATES).getNode(templateName).getPath() ;
      }      
      contentExplorer.initContenExplorerData(session_,rootPath_,contentListTemplatePath,documentTemplatePath) ;
      contentExplorer.setRendered(true) ;      
    }else {      
      try {
        List<Node> contentList = getNodesByScript(scriptName_) ;
        UIContentExplorer contentExplorer = (UIContentExplorer)addChild(UIContentExplorer.class) ;                        
        String contentListTemplatePath = 
        viewService_.getTemplateHome(BasePath.CB_SCRIPT_TEMPLATES).getNode(templateName).getPath() ;        
        session_ = repositoryService_.getRepository().login(workspace_) ;
        contentExplorer.initScriptContenExplorerData(session_, contentList, workspace_, 
            contentListTemplatePath, documentTemplatePath) ;
        setRendererType("ChildrenRenderer") ;
        contentExplorer.setRendered(true) ;
      }catch (Exception e) {
        e.printStackTrace() ;
        setRendererType("VelocityRenderer");        
      }                      
    }                       
  }

  private boolean checkPath() {
    boolean isValid = true ;
    try {     
      if(rootPath_.startsWith("/jcr:system")) {          
        session_ = repositoryService_.getRepository().getSystemSession(cmsConfService_.getWorkspace()) ;          
      }
      else {        
        session_ = repositoryService_.getRepository().login(workspace_) ;
      }           
      session_.getItem(rootPath_) ;        
    }catch (Throwable e) {      
      isValid = false ;
    }    
    return isValid ;
  }

  public List<Node> getNodesByScript(String scriptName) throws Exception {
    String[] array = scriptName.split(CBUtils.SEMI_COLON) ;
    DataTransfer data = new DataTransfer() ;
    data.setWorkspace(array[0].trim()) ;
    CmsScript cmsScript = scriptService_.getScript(array[1].trim());
    cmsScript.execute(data);                
    return data.getContentList() ;
  }
}
