/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.contentbrowser.component;

import java.util.ArrayList;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Session;
import javax.jcr.query.Query;
import javax.jcr.query.QueryManager;
import javax.jcr.query.QueryResult;

import org.apache.commons.lang.StringUtils;
import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.services.cms.queries.QueryService;
import org.exoplatform.services.cms.templates.TemplateService;


/**
 * @author Tuan Nguyen (tuan08@users.sourceforge.net)
 * @since Aug 27, 2004
 * @version $Id$
 */
public class UISearchForm extends UISimpleForm {  
  public static final String SEARCH_ACTION = "searchAction";
  public static final String CHANGE_TYPE_ACTION = "changeType" ;
  public static final String SEARCH_TYPE = "type" ;
  public static final String CATEGORY_SEARCH = "Category" ;
  public static final String DOCUMENT_SEARCH = "Content" ;	

  public static final String CATEGORY_QUERY = "select * from $0 where jcr:path like '%/$1[%]' "  ;
  public static final String DOCUMENT_QUERY = "select * from $0 where contains(*, '$1') AND jcr:path like '$2[%]/%' ";	

  private Node node_;
  protected UIStringInput inputText_;
  protected QueryService queryService_;	
  protected UICheckBox referenceBox_ ;
  protected UICheckBox relationBox_ ;
  protected UISelectBox selectTypeBox_ ;
  public boolean isDocumentType = true ;
  protected long duration_ ;  
  protected TemplateService templateService_ ;

  public UISearchForm(QueryService queryService,TemplateService templateService) throws Exception {
    super("UISearchForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");

    queryService_ = queryService;
    templateService_ = templateService  ;

    List<SelectItem> browsingOptions = new ArrayList<SelectItem>() ;    		
    browsingOptions.add(new SelectItem(CATEGORY_SEARCH,CATEGORY_SEARCH)) ;
    browsingOptions.add(new SelectItem(DOCUMENT_SEARCH,DOCUMENT_SEARCH)) ;

    selectTypeBox_ = new  UISelectBox(SEARCH_TYPE,DOCUMENT_SEARCH, browsingOptions) ;    
    selectTypeBox_.setUpdateOnChangeAction(CHANGE_TYPE_ACTION) ;
    add(selectTypeBox_);

    inputText_ = new UIStringInput("keyword", "");		
    add(inputText_) ;

    referenceBox_ = new UICheckBox("reference","").setSelect(true) ;
    add(referenceBox_) ;
    relationBox_ = new UICheckBox("relation","").setSelect(true) ;                    
    add(relationBox_) ;    

    addActionListener(SearchActionListener.class, SEARCH_ACTION);
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
    addActionListener(ChangeTypeActionListener.class,CHANGE_TYPE_ACTION );
  }

  public String getSearchAction() { return SEARCH_ACTION; }
  public boolean isDocumentSearchType() {return isDocumentType ;}

  public void updateNode(Node node) throws Exception {
    node_ = node;    
  }    

  public List<Node> searchByCategory(String keyword) throws Exception{
    List<Node> resultList = new ArrayList<Node>() ;
    UISearchController  controller = (UISearchController)getParent() ;
    UISearch uisearch = (UISearch)controller.getParent() ;
    UIContentExplorer explorer = (UIContentExplorer) uisearch.getAncestorOfType(UIContentExplorer.class);      
    Session session = explorer.getSession();      
    QueryManager queryManager = session.getWorkspace().getQueryManager();      		    
    duration_ = 0 ;
    String statement = StringUtils.replace(CATEGORY_QUERY, "$1", keyword);
    for(String type:CBUtils.CATEGORY_NODE_TYPES) {            
      String queryStatement = StringUtils.replace(statement, "$0",type);
      Query query = queryManager.createQuery(queryStatement, Query.SQL);
      long beforeTime = System.currentTimeMillis();
      QueryResult queryResult = query.execute();
      long searchTime = System.currentTimeMillis() - beforeTime;
      duration_ += searchTime ;
      for(NodeIterator iter = queryResult.getNodes();iter.hasNext();) {
        Node node = iter.nextNode() ;
        if(node.getPath().startsWith(node_.getPath())) {                    
          resultList.add(node) ; 
        }
      }           
    }    
    return resultList ;
  }	

  public List<Node> searchDocument(String keyword, boolean reference, boolean relation) throws Exception {		
    List<Node> resultList = new ArrayList<Node>() ;        
    String statement = StringUtils.replace(DOCUMENT_QUERY,"$1",keyword) ;
    statement = StringUtils.replace(statement,"$2",node_.getPath()) ;    
    UISearchController  controller = (UISearchController)getParent() ;
    UISearch uisearch = (UISearch)controller.getParent() ;
    UIContentExplorer explorer = (UIContentExplorer) uisearch.getAncestorOfType(UIContentExplorer.class);      
    Session session = explorer.getSession();    
    QueryManager queryManager = session.getWorkspace().getQueryManager();
    duration_ = 0 ; 
    List<String> documentNodeTypes = templateService_.getDocumentTemplates() ;
    documentNodeTypes.add("nt:resource") ;
    for(String ntDocument:documentNodeTypes) {            
      String queryStatement = StringUtils.replace(statement,"$0",ntDocument) ;      
      Query query = queryManager.createQuery(queryStatement, Query.SQL);    
      long beforeTime = System.currentTimeMillis() ;
      QueryResult queryResult = query.execute() ;
      long searchTime = System.currentTimeMillis()- beforeTime ;
      duration_ += searchTime ;
      for(NodeIterator resultNodes = queryResult.getNodes(); resultNodes.hasNext() ;) {        
        Node node = resultNodes.nextNode() ;
        if(ntDocument.equals("nt:resource")) {
          Node paNode = node.getParent() ;
          if(CBUtils.isDocument(paNode.getPrimaryNodeType())) {
            resultList.add(paNode) ;
          }
        }else {
          resultList.add(node) ; 
        }        
      }
    }        
    return resultList ;
  }

  static public class ChangeTypeActionListener extends ExoActionListener{
    public void execute(ExoActionEvent event) throws Exception {
      UISearchForm uiform = (UISearchForm) event.getSource() ;
      String searchType = uiform.selectTypeBox_.getValue() ;
      if(searchType.equals(DOCUMENT_SEARCH)) { 
        uiform.isDocumentType = true ;        
        uiform.referenceBox_.setSelect(true) ;
        uiform.relationBox_.setSelect(true) ;
      }
      else { 
        uiform.isDocumentType = false ;       
      }
    }   
  }

  static public class SearchActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UISearchForm uiForm = (UISearchForm) event.getSource();
      String keyword = uiForm.inputText_.getValue();
      String type = uiForm.selectTypeBox_.getValue() ;            
      boolean reference = uiForm.referenceBox_.getSelect() ;
      boolean relation = uiForm.relationBox_.getSelect() ;      
      Information iprovider = findInformationProvider(uiForm);
      List<Node> queryResult ;      
      UIResults results = (UIResults) uiForm.getSibling(UIResults.class);
      try {      	
        if(keyword == null||keyword.length() == 0) {          
          return ;
        }
        if(type.equals(CATEGORY_SEARCH)) {
          queryResult = uiForm.searchByCategory(keyword) ;
          results.setQueryResults(uiForm.duration_, queryResult,uiForm.node_,CATEGORY_SEARCH);
        }else {
          queryResult = uiForm.searchDocument(keyword,reference,relation) ;
          results.setQueryResults(uiForm.duration_, queryResult,uiForm.node_,DOCUMENT_SEARCH);
        }
      } catch (Exception e) {		        
        JCRExceptionManager.process(iprovider, e);        
      }                  
    }
  } 

  static public class CancelActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UISearchForm uiForm = (UISearchForm) event.getComponent() ;
      UISearchController controller = (UISearchController)uiForm.getParent() ;
      UISearch uisearch = (UISearch)controller.getParent() ;
      UIContentExplorer uiExplorer = (UIContentExplorer) uisearch.getAncestorOfType(UIContentExplorer.class);
      Node node = uiExplorer.getSelectNode();
      uiExplorer.broadcastOnChange(node);  
      uisearch.setRenderedSibling(UIContentList.class) ;
    }
  }
}