package org.exoplatform.portlets.content.drives.component;

import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.jcr.Session;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.cms.component.JcrBrowser;
import org.exoplatform.faces.core.Selector;
import org.exoplatform.faces.core.SelectorHandler;
import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.faces.core.validator.PermissionValidator;
import org.exoplatform.faces.user.component.UIExoPermissionBrowser;
import org.exoplatform.faces.user.component.UIExoPermissionSelector;
import org.exoplatform.portal.Information;
import org.exoplatform.services.cms.drives.DriveData;
import org.exoplatform.services.cms.drives.ManageDriveService;
import org.exoplatform.services.cms.views.ManageViewService;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.core.ManageableRepository;

public class UIDriveForm extends UISimpleForm implements SelectorHandler {

  final private static String     SELECT_VIEWS_ACTION = "selectView".intern();
  final private static String     VIEW_PREFERENCES    = "viewPreferences".intern();
  final private static String     VIEW_NON_DOCUMENT   = "viewNonDocument".intern();
  final private static String     VIEW_EXPLORER       = "viewExplorer".intern();
  final private static String     VIEW_CLIPBOARD      = "viewClipboard".intern();
  final private static String     NAME                = "name".intern();
  final private static String     PATH                = "path".intern();
  final private static String     VIEWS               = "views".intern();
  final private static String     WORKSPACE           = "workspace".intern();
  final private static String     ICON                = "icon".intern();
  final private static String     EMPTY               = "".intern();
  final public static String      SELECT_PATH_ACTION  = "selectPath".intern();
  final public static String      SELECT_ICON_ACTION  = "selectIcon".intern();
  final public static String      SEARCH_GROUP        = "searchGroup";
  private UIExoPermissionSelector userOrGroup_;
  private UIStringInput           path_;
  private UIStringInput           icon_;
  private UISelectBox             workspace_;
  private ManageDriveService      dservice_;
  private ManageableRepository    repository_;
  private ManageViewService       vservice_;
  private String                  selectAction_       = SELECT_PATH_ACTION;
  private boolean                 isAddNew_           = false;

  public UIDriveForm(ManageDriveService dservice, RepositoryService repositoryService,
      ManageViewService vservice) throws Exception {
    super("uiDriveForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");
    dservice_ = dservice;
    vservice_ = vservice;
    repository_ = repositoryService.getRepository();
    UIStringInput name = new UIStringInput(NAME, EMPTY);
    path_ = new UIStringInput(PATH, EMPTY);
    List<SelectItem> workspaceItems = new ArrayList<SelectItem>();
    String[] wsNames = repository_.getWorkspaceNames();
    for (int i = 0; i < wsNames.length; i++) {
      workspaceItems.add(new SelectItem(wsNames[i].toString(), wsNames[i].toString()));
    }
    workspace_ = new UISelectBox(WORKSPACE, EMPTY, workspaceItems);
    icon_ = new UIStringInput(ICON, EMPTY);
    UIStringInput views = new UIStringInput(VIEWS, EMPTY);
    userOrGroup_ = new UIExoPermissionSelector("userOrGroup", null);
    //userOrGroup_.addValidator(PermissionValidator.class);
    UICheckBox viewReferences = new UICheckBox(VIEW_PREFERENCES, EMPTY, false);
    UICheckBox viewNonDocument = new UICheckBox(VIEW_NON_DOCUMENT, EMPTY, false);
    UICheckBox viewExplorer = new UICheckBox(VIEW_EXPLORER, EMPTY, false);
    UICheckBox viewClipboard = new UICheckBox(VIEW_CLIPBOARD, EMPTY, false);
    add(name);
    add(path_);
    add(workspace_);
    add(icon_);
    add(views);
    add(userOrGroup_);
    add(viewReferences);
    add(viewNonDocument);
    add(viewExplorer);
    add(viewClipboard);

    addActionListener(SelectViewsActionListener.class, SELECT_VIEWS_ACTION);
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
    addActionListener(SaveActionListener.class, SAVE_ACTION);
    addActionListener(SearchGroupActionListener.class, SEARCH_GROUP);
    addActionListener(SelectPathActionListener.class, SELECT_PATH_ACTION);
    addActionListener(SelectIconActionListener.class, SELECT_ICON_ACTION);
  }

  public String getSearchGroupAction() {
    return SEARCH_GROUP;
  }

  public String getSelectViewsAction() {
    return SELECT_VIEWS_ACTION;
  }

  public String getSelectPathParams() {
    return SELECT_PATH_ACTION;
  }

  public String getSelectIconParams() {
    return SELECT_ICON_ACTION;
  }

  public void refresh(String driveName) throws Exception {
    if (driveName == null) {
      getUIStringInput(NAME).setValue(EMPTY);
      getUIStringInput(NAME).setEditable(true);
      getUIStringInput(VIEWS).setValue(EMPTY);
      getUIStringInput(PATH).setValue(EMPTY);
      getUIStringInput(ICON).setValue(EMPTY);
      userOrGroup_.setValue(EMPTY);
      getUICheckBox(VIEW_PREFERENCES).setSelect(false);
      getUICheckBox(VIEW_NON_DOCUMENT).setSelect(false);
      getUICheckBox(VIEW_EXPLORER).setSelect(false);
      getUICheckBox(VIEW_CLIPBOARD).setSelect(false);
      isAddNew_ = true;
    } else {
      DriveData drive = (DriveData) dservice_.getDriveByName(driveName);
      if (drive != null) {
        getUIStringInput(NAME).setValue(drive.getName());
        getUIStringInput(NAME).setEditable(false);
        getUIStringInput(VIEWS).setValue(drive.getViews());
        getUISelectBox(WORKSPACE).setValue(drive.getWorkspace());
        getUIStringInput(PATH).setValue(drive.getHomePath());
        getUIStringInput(ICON).setValue(drive.getIcon());
        getUICheckBox(VIEW_PREFERENCES).setSelect(drive.getViewPreferences());
        getUICheckBox(VIEW_NON_DOCUMENT).setSelect(drive.getViewNonDocument());
        getUICheckBox(VIEW_EXPLORER).setSelect(drive.getViewExplorer());
        getUICheckBox(VIEW_CLIPBOARD).setSelect(drive.getViewClipboard());
        userOrGroup_.setValue(drive.getPermissions());
        isAddNew_ = false;
      }
    }
  }

  public void cancel(Selector selector) {
    selector.setRenderedSibling(UIDriveForm.class);
  }

  public void select(Selector selector) throws Exception {
    if (selectAction_.equals(SELECT_PATH_ACTION)) {
      path_.setValue(selector.getInputValue());
    } else if (selectAction_.equals(SEARCH_GROUP)) {
      UIExoPermissionBrowser browser = (UIExoPermissionBrowser)selector ;
      if(browser.isGroup()) {
        userOrGroup_.setPermission(selector.getInputValue());
      }else {
        userOrGroup_.setValue(browser.getSelectedUser()) ;
      }      
    } else {
      icon_.setValue(selector.getInputValue());
    }
    selector.setRenderedSibling(UIDriveForm.class);
  }

  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      try {
        UIDriveForm uiForm = (UIDriveForm) event.getSource();
        ResourceBundle res = getApplicationResourceBundle();
        Information iprovider = findInformationProvider(uiForm);

        String name = uiForm.getUIStringInput(NAME).getValue();
        if (name == null || name.length() < 1) {
          iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN,
              res.getString("UIDriveForm.msg.drive-name-invalid")));
          return;
        }
        String workspace = uiForm.getUISelectBox(WORKSPACE).getValue();
        if (workspace == null || workspace.length() < 1) {
          iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN,
              res.getString("UIDriveForm.msg.drive-workspace-invalid")));
          
          return;
        }
        String path = uiForm.getUIStringInput(PATH).getValue();
        if (path == null || path.length() == 0)
          path = "/";
        try {
          Session session = uiForm.repository_.login(workspace);
          session.getItem(path);
        } catch (Exception e) {
          iprovider.addMessage(new ExoFacesMessage(res
              .getString("UIDriveForm.msg.workspace-path-invalid")));
          return;
        }

        String views = uiForm.getUIStringInput(VIEWS).getValue();
        if (views == null || views.length() < 1) {
          iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN,
              res.getString("UIDriveForm.msg.drive-views-invalid")));
          return;
        }
        String[] array = views.split(",");
        for (int i = 0; i < array.length; i++) {
          if (!uiForm.vservice_.hasView(array[i].trim())) {
            Object[] arg = { array[i].trim() };
            iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN,
                res.getString("UIDriveForm.msg.view-not-exist"), arg));
            return;
          }
        }
        String icon = uiForm.getUIStringInput(ICON).getValue();
        String permissions = uiForm.userOrGroup_.getValue();
        if (permissions == null || permissions.length() < 1) {
          iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN ,
              res.getString("UIDriveForm.msg.permissions-not-empty")));
          return;
        }

        boolean viewReferences = uiForm.getUICheckBox(VIEW_PREFERENCES).getSelect();
        boolean viewExplorer = uiForm.getUICheckBox(VIEW_EXPLORER).getSelect();
        boolean viewNonDocument = uiForm.getUICheckBox(VIEW_NON_DOCUMENT).getSelect();
        boolean viewClipboard = uiForm.getUICheckBox(VIEW_CLIPBOARD).getSelect();
        if (uiForm.isAddNew_) {
          if (uiForm.dservice_.getDriveByName(name) == null)
            uiForm.dservice_.addDrive(name, workspace, permissions, path, views, icon,
                viewReferences, viewNonDocument, viewExplorer, viewClipboard);
          else {
            iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN ,
                    res.getString("UIDriveForm.msg.drive-exists")));
            return;
          }
        } else {
          uiForm.dservice_.addDrive(name, workspace, permissions, path, views, icon,
              viewReferences, viewNonDocument, viewExplorer, viewClipboard);
        }
        UIDriveList driveList = (UIDriveList) uiForm.getSibling(UIDriveList.class);
        driveList.refresh();
        uiForm.setRenderedSibling(UIDriveList.class);
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
  }

  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDriveForm viewForm = (UIDriveForm) event.getSource();
      viewForm.setRenderedSibling(UIDriveList.class);
    }
  }

  static public class SearchGroupActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDriveForm uiForm = (UIDriveForm) event.getSource();
      uiForm.selectAction_ = SEARCH_GROUP;
      UIExoPermissionBrowser uiGroupBrowser = (UIExoPermissionBrowser) uiForm
          .getSibling(UIExoPermissionBrowser.class);
      uiGroupBrowser.setHandler(uiForm);
      uiForm.setRenderedSibling(UIExoPermissionBrowser.class);
    }
  }

  static public class SelectPathActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDriveForm uicomp = (UIDriveForm) event.getComponent();
      uicomp.selectAction_ = SELECT_PATH_ACTION;
      JcrBrowser jcrBrowser = (JcrBrowser) uicomp.getSibling(JcrBrowser.class);
      jcrBrowser.setFilter(null);
      jcrBrowser.setHandler(uicomp);
      jcrBrowser.setParameters(new String[] { uicomp.workspace_.getValue() , "/" });
      uicomp.setRenderedSibling(JcrBrowser.class);
    }
  }

  static public class SelectIconActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDriveForm uicomp = (UIDriveForm) event.getComponent();
      uicomp.selectAction_ = SELECT_ICON_ACTION ;
      JcrBrowser jcrBrowser = (JcrBrowser) uicomp.getSibling(JcrBrowser.class);
      jcrBrowser.setHandler(uicomp);
      List<String> filter = new ArrayList<String>() ;
      filter.add("nt:file") ;
      jcrBrowser.setFilter(filter);
      jcrBrowser.setParameters(new String[] { "digital-assets", "/" });
      uicomp.setRenderedSibling(JcrBrowser.class);
    }
  }

  static public class SelectViewsActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      try {
        UIDriveForm uiForm = (UIDriveForm) event.getSource();
        String views = uiForm.getUIStringInput(VIEWS).getValue();
        List<String> types = new ArrayList<String>();
        if (views != null && views.length() > 0) {
          if (views.indexOf(",") > -1) {
            String[] array = views.split(",");
            for (int i = 0; i < array.length; i++) {
              types.add(array[i].trim());
            }
          } else {
            types.add(views.trim());
          }
        }
        UIViewList viewList = (UIViewList) uiForm.getSibling(UIViewList.class);
        viewList.update(types);
        uiForm.setRenderedSibling(UIViewList.class);
      } catch (Exception e) {
        e.printStackTrace();
      }
    }
  }
}
