/***************************************************************************
 * Copyright 2001-2007 The eXo Platform SAS         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.jcr.component;

import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.jcr.Node;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UICommandNode;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UIWYSIWYG;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.jcr.component.UIViewController.CancelActionListener;
import org.exoplatform.services.cms.comments.CommentsService;

/**
 * Created by The eXo Platform SAS
 * Author : Pham Xuan Hoa
 *          hoa.pham@exoplatform.com
 * Jan 17, 2007  
 */
public class UICommentsForm extends UISimpleForm implements ChangeNodeListener{
  
  private Node commentsDocument ;    
  
  public UICommentsForm() {
    super("CommentsForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");    
    
    add(new UIStringInput("commentor","")) ;
    add(new UIStringInput("email","")) ;
    add( new UIStringInput("site","")) ;   
    add(new UIWYSIWYG("comments","")) ;
    
    addActionListener(SaveActionListener.class,SAVE_ACTION) ;
    addActionListener(CancelActionListener.class,CANCEL_ACTION) ;    
  }
    
  public Node getDocument() { return commentsDocument ; }  
  public void setDocument2Comments(Node document) {
    this.commentsDocument = document ;
  }
            
  public void onChange(UIJCRExplorer uiExplorer, Node node) throws Exception {
    this.setRendered(false) ;
  }
  
  public static class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommentsForm uicomp = (UICommentsForm)event.getComponent() ;
      CommentsService commentsService = 
        (CommentsService)PortalContainer.getComponent(CommentsService.class) ;
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uicomp.getAncestorOfType(UIJCRExplorer.class);
      UICommandNode commandNode = (UICommandNode)uicomp.getParent() ;
      UIDocumentInfo documentInfo = 
        (UIDocumentInfo)commandNode.getChildComponentOfType(UIDocumentInfo.class) ;      
      String language = documentInfo.getLanguage() ;
      Node document = uicomp.getDocument() ;
      if(UIDocumentInfo.DEFAULT_LANGUAGE.equals(language)) {      
        if(!document.hasProperty("exo:language")) {
          return ;
        }
        language = document.getProperty("exo:language").getString() ;
      }
      String comment = uicomp.getUIInput("comments").getValue() ;
      String commentor = uicomp.getUIInput("commentor").getValue() ;
      String email = uicomp.getUIInput("email").getValue() ;
      String site = uicomp.getUIInput("site").getValue() ;
      Information iprovider = findInformationProvider(uicomp) ;
      ResourceBundle res = getApplicationResourceBundle() ;
      if(comment == null || comment.length() == 0) {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN,
            res.getString("UICommentForm.msg.null-comment"))) ;
        return ;
      }            
      try{
        commentsService.addComment(document,commentor,email,site,comment,language) ;
        iprovider.addMessage(
            new ExoFacesMessage(res.getString("UICommentForm.msg.add-successfull"))) ;
      }catch (Exception e) {        
        JCRExceptionManager.process(iprovider,e) ;
        e.printStackTrace() ;
      }                  
      uiExplorer.broadcastOnChange(uiExplorer.getSelectNode()) ;
      uicomp.reset() ;
    }    
  }
  
}
