package org.exoplatform.portlets.content.jcr.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.TreeMap;

import javax.faces.application.FacesMessage;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.jcr.AccessDeniedException;
import javax.jcr.ItemNotFoundException;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.PathNotFoundException;
import javax.jcr.Property;
import javax.jcr.PropertyIterator;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.Value;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.PropertyDefinition;
import javax.portlet.PortletRequest;

import org.exoplatform.container.SessionContainer;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.cms.CMSDelegate;
import org.exoplatform.faces.cms.CMSUtils;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.model.PageListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.ContentPageList;
import org.exoplatform.portlets.content.JCRUtil;
import org.exoplatform.portlets.content.contentbrowser.component.CBUtils;
import org.exoplatform.portlets.content.jcr.AlphaNodeComparator;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.jcr.PropertiesComparator;
import org.exoplatform.portlets.content.jcr.TypeNodeComparator;
import org.exoplatform.portlets.content.jcr.component.model.ClipboardCommand;
import org.exoplatform.portlets.content.jcr.component.model.Preference;
import org.exoplatform.services.cms.actions.ActionServiceContainer;
import org.exoplatform.services.cms.templates.TemplateService;
import org.exoplatform.services.grammar.wiki.WikiContext;
import org.exoplatform.services.grammar.wiki.WikiEngineService;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.core.ExtendedNode;
import org.exoplatform.services.security.SecurityService;

public class UIDocumentInfo extends UIExoCommand implements ChangeNodeListener{
  
  public static final String[] NON_EDITABLE_NODETYPES = 
  { "nt:unstructured", "nt:folder", "nt:resource" };
  public static final String[] PASTE_TYPES = { "nt:unstructured", "nt:folder" };
  public static final String DEFAULT_LANGUAGE = "default".intern() ;
  
  final static public String ACTION_NAME = "actionName".intern();  
  final static public String EDIT_DOCUMENT = "editDocument".intern();
  final static public String RENAME_ACTION = "rename".intern();
  public static final String COPY_ACTION = "copy".intern();
  public static final String CUT_ACTION = "cut".intern();
  final static public String SAVE_NODE = "saveNode".intern();
  final static public String REMOVE_NODE_ACTION = "removeNode".intern();
  final static public String CHECKIN_NODE_ACTION = "checkin".intern();
  final static public String CHECKOUT_NODE_ACTION = "checkout".intern();
  final static public String LOCK_NODE_ACTION = "lock".intern();
  final static public String UNLOCK_NODE_ACTION = "unlock".intern();  
  final static public String CUSTOM_ACTION = "custom".intern();
  final static public String HIDE_ACTION = "hide".intern() ;
  final static public String SORT_ACTION = "sort".intern() ;
  final static public String VIEW_NODE_ACTION = "viewNode".intern() ;
  final static public String CHANGE_LANGUAGE_ACTION = "changeLanguage" ;
  
  public static Parameter[] CHANGE_NODE_PARAMS = { new Parameter(ACTION, UIJCRExplorer.CHANGE_NODE_ACTION) };
  public static Parameter[] VIEW_NODE_PARAMS = { new Parameter(ACTION, VIEW_NODE_ACTION) };
  public static Parameter[] EDIT_DOCUMENT_PARAMS = { new Parameter(ACTION, EDIT_DOCUMENT) };
  public static Parameter[] RENAME_NODE_PARAMS        = { new Parameter(ACTION, RENAME_ACTION) };
  public static Parameter[] COPY_NODE_PARAMS     = { new Parameter(ACTION,COPY_ACTION) };
  public static Parameter[] CUT_NODE_PARAMS      = { new Parameter(ACTION,CUT_ACTION) };
  public static Parameter[] SAVE_NODE_PARAMS     = { new Parameter(ACTION,SAVE_NODE) };
  public static Parameter[] REMOVE_NODE          = { new Parameter(ACTION,REMOVE_NODE_ACTION) };
  public static Parameter[] CHECKIN_NODE_PARAMS  = { new Parameter(ACTION, CHECKIN_NODE_ACTION) };
  public static Parameter[] CHECKOUT_NODE_PARAMS = { new Parameter(ACTION,CHECKOUT_NODE_ACTION) };
  public static Parameter[] LOCK_NODE_PARAMS     = { new Parameter(ACTION,LOCK_NODE_ACTION) };
  public static Parameter[] UNLOCK_NODE_PARAMS   = { new Parameter(ACTION, UNLOCK_NODE_ACTION) };    
  public static Parameter[] HIDE_ACTION_PARAMS   = { new Parameter(ACTION,HIDE_ACTION) } ;
  public static Parameter[] SORT_ACTION_PARAMS   = { new Parameter(ACTION,SORT_ACTION) } ;
  public static Parameter[] CHANGE_LANGUAGE_ACTION_PARAMS   = { new Parameter(ACTION,CHANGE_LANGUAGE_ACTION) } ;
  
  public static final int NB_PER_ROW = 4;
  public static final int MAC_VIEW_NB_COLUMNS = 3;
  
  protected Node node_;
  private CMSDelegate delegate;
  protected ActionServiceContainer actionService_;
  private SecurityService securityService_;
  private TemplateService templateService_;
  private UIJCRExplorer uiExplorer_ ;
  protected boolean showRelation ;
  private boolean hasHiddenNode_ = false ;
  private UIPageListIterator uiPageIterator_ ;
  private RepositoryService repositoryService_ ;
  private String language_ = DEFAULT_LANGUAGE ;
  
  public UIDocumentInfo(TemplateService templateService, SecurityService securityService, 
      ActionServiceContainer actionService, RepositoryService repositoryService) {
    setId("UIDocumentInfo");
    setRendererType("VelocityRenderer");
    
    repositoryService_ = repositoryService ;
    templateService_ = templateService;
    securityService_ = securityService;
    actionService_ = actionService;
    uiPageIterator_ = new UIPageListIterator(new PageListDataHandler()) ;
    getChildren().add(uiPageIterator_) ;
    uiExplorer_ = (UIJCRExplorer)getAncestorOfType(UIJCRExplorer.class); 
    
    showRelation = false ;
    addActionListener(ChangeNodeActionListener.class,UIJCRExplorer.CHANGE_NODE_ACTION);
    addActionListener(ViewNodeActionListener.class, VIEW_NODE_ACTION);
    addActionListener(EditDocumentActionListener.class, EDIT_DOCUMENT);    
    addActionListener(CopyActionListener.class, COPY_ACTION);
    addActionListener(CutActionListener.class, CUT_ACTION);
    addActionListener(RenameActionListener.class,RENAME_ACTION) ;
    addActionListener(SaveNodeActionListener.class, SAVE_NODE);
    addActionListener(RemoveNodeActionListener.class, REMOVE_NODE_ACTION);
    addActionListener(CheckinNodeActionListener.class, CHECKIN_NODE_ACTION);
    addActionListener(CheckoutNodeActionListener.class, CHECKOUT_NODE_ACTION);
    addActionListener(LockActionListener.class, LOCK_NODE_ACTION);
    addActionListener(UnlockActionListener.class, UNLOCK_NODE_ACTION);
    addActionListener(CustomActionListener.class, CUSTOM_ACTION);
    addActionListener(HideRelationPanelActionListener.class,HIDE_ACTION) ;
    addActionListener(SortActionListener.class,SORT_ACTION) ;    
    addActionListener(ChangeLanguageActionListener.class,CHANGE_LANGUAGE_ACTION) ;
  }
  
  public Node getNodeByUUID(String uuid) throws Exception{
    Session session = uiExplorer_.getSystemSession();
    return session.getNodeByUUID(uuid);
  }
  
  public void onChange(UIJCRExplorer uiExplorer, Node node) throws Exception {
    node_ = node;
    uiExplorer_ = uiExplorer ;
    if (delegate == null)
      delegate = new CMSDelegate(node);
    else
      delegate.setNode(node);
    //List childrenList = getChildrenList() ;
    //uiPageIterator_.setPageList(new ContentPageList(childrenList)) ;
    setRendered(true);    
  }
  
  public boolean hasPropertyContent(Node node, String property){
    return delegate.hasPropertyContent(node, property) ;
  }
  
 
  
  public List getParents() throws Exception{
    List parents = new ArrayList() ;
    Node rootNode = uiExplorer_.getSession().getRootNode() ;
    String[] arrayPath = node_.getPath().split("/") ;
    for(int i = 0; i < arrayPath.length; i ++) {
      if(arrayPath[i] != null && arrayPath[i].trim().length() > 0) {
        parents.add(rootNode.getNode(arrayPath[i].trim())) ;
        rootNode = rootNode.getNode(arrayPath[i].trim()) ;        
      }
    }
    return parents ;
  }
  
  public boolean isPreferenceNode(Node node) throws RepositoryException{
    if(node_.hasNode(node.getName())){
      Node childNode = node_.getNode((node.getName())) ;
      if(childNode.getPath().equals(node.getPath())) return false;      
    }
    
    return true; 
  }
  
  public boolean isPreferenceNodeMac(Node node) throws RepositoryException{
    if(node_.hasNode(node.getName())) 
      return false;
    Node parent ;
    try{
      parent = node_.getParent() ;
    }catch(ItemNotFoundException e) {
      return true ;
    }    
    if(parent.hasNode(node.getName())) 
      return false ;
    return true; 
  }
  
  private boolean isReferenceableNode(Node node) throws Exception {
    NodeType[] nodeTypes = node.getMixinNodeTypes() ;
    for(NodeType type:nodeTypes) { 
      if(type.getName().equals(CBUtils.MIX_REFERENCEABLE)) return true ;
    }
    return false ;
  }
   
  public boolean isSameNameSibling(Node node) throws Exception{
    if(node.getPath().endsWith("]")) return true ;
    return false ;
  }
  
  public boolean viewRelationEnable() throws Exception{
    if(!isNodeTypeSupported()) return false ;
    if(!node_.hasProperty("exo:relation")) return false ;
    return showRelation ;
  }
  
  public boolean isJcrViewEnable() {    
    return uiExplorer_.getPreference().isJcrEnable();
  }
  
  public boolean isReadAuthorized(ExtendedNode node) throws RepositoryException {
    UIJCRExplorer explorer = (UIJCRExplorer) getAncestorOfType(UIJCRExplorer.class);
    return explorer.isReadAuthorized(node);
  }
  
  public boolean isReadAuthorized(String name) throws RepositoryException {
    UIJCRExplorer explorer = (UIJCRExplorer) getAncestorOfType(UIJCRExplorer.class);
    name = name.substring(name.lastIndexOf("/") + 1) ;
    ExtendedNode node = (ExtendedNode) node_.getNode(name);
    return explorer.isReadAuthorized(node);
  }
  
  public String getNodeOwner() {
    String str = null;
    try {
      str = ((ExtendedNode) node_).getACL().getOwner();    //return doc. 
    }catch (Exception e) {
      e.printStackTrace();
    }
    return str;
  }
  
  public boolean hasAdminRole() {
    FacesContext context  = FacesContext.getCurrentInstance() ;
    ExternalContext econtext = context.getExternalContext()  ;
    return econtext.isUserInRole("admin") ;
  }
  
  public boolean hasChildren() throws Exception{
   return hasChildren(null);        
  }
  
  public boolean isShowedHidden() {
    Preference preferences = uiExplorer_.getPreference() ;
    if(preferences.isShowNonDocumentType() && preferences.isShowPreferenceDocuments()) 
      return true ;
    return false ;
  }
  public boolean hasChildren(Node pNode) {
    UIJCRExplorer explorer = (UIJCRExplorer) getAncestorOfType(UIJCRExplorer.class);
    Preference pref = explorer.getPreference();
    hasHiddenNode_ = false ;
    NodeIterator childrenIterator ;
    List<Node> temp  = new ArrayList<Node>() ;
    try {      
      if(!pref.isShowNonDocumentType()){
        if (pNode == null)
          childrenIterator = node_.getNodes();
        else
          childrenIterator = pNode.getNodes();
        
        List documentTypes = templateService_.getDocumentTemplates() ;      
        while (childrenIterator.hasNext()){
          Node child = childrenIterator.nextNode() ;
          NodeType type = child.getPrimaryNodeType() ;
          if("nt:unstructured".equals(type.getName()) || "nt:folder".equals(type.getName()))
            temp.add(child) ;
          else
            if(documentTypes.contains(type.getName())) 
              temp.add(child) ;   
            else
              hasHiddenNode_ = true ;
        }        
        if (temp.size() > 0)
          return true ;        
        return false ;
      } 
      if (pNode == null)
        return node_.hasNodes();      
      return pNode.hasNodes(); 
      
    } catch (Exception e) {
      return false;
    }
  }
  
  public Parameter[] getChangeNodeParams() { return CHANGE_NODE_PARAMS; }	
  public Parameter[] getViewNodeParams() { return VIEW_NODE_PARAMS; }
  public Parameter[] getHideRelationParams() { return HIDE_ACTION_PARAMS ;}  
  public Parameter[] getEditDocumentParams() { return EDIT_DOCUMENT_PARAMS;	}	
  public Parameter[] getRenameParams() { return RENAME_NODE_PARAMS;}
  public Parameter[] getCopyParams() { return COPY_NODE_PARAMS;	}	
  public Parameter[] getCutParams() { return CUT_NODE_PARAMS; }	
  public Parameter[] getSaveParams() { return SAVE_NODE_PARAMS; }
  public Parameter[] getDeleteParams() { return REMOVE_NODE; }  
  public Parameter[] getCheckinParams() { return CHECKIN_NODE_PARAMS; }	
  public Parameter[] getCheckoutParams() { return CHECKOUT_NODE_PARAMS;	}  
  public Parameter[] getLockParams() { return LOCK_NODE_PARAMS; }	
  public Parameter[] getUnlockParams() {return UNLOCK_NODE_PARAMS;  }    
  public Parameter[] getSortParams() {return SORT_ACTION_PARAMS;  }
  public Parameter[] getChangeLanguageParams() {return CHANGE_LANGUAGE_ACTION_PARAMS;  }
  
  public Parameter[] getCustomActionParams(String actionName) {
    Parameter[] CUSTOM_ACTION_PARAMS = { new Parameter(ACTION,
        CUSTOM_ACTION),  new Parameter(ACTION_NAME, actionName)};      
    return CUSTOM_ACTION_PARAMS;
  }    
  
  public String getViewToParse() throws Exception {   		
    try {
      getNodeType();     
    } catch (Exception e) {
      //e.printStackTrace() ;
      return "";
    }
    UIJCRExplorer explorer = (UIJCRExplorer) getAncestorOfType(UIJCRExplorer.class);
    UIViewController viewController = (UIViewController)explorer.getChildComponentOfType(UIViewController.class) ;
    if (isJcrViewEnable()) {
      return "jcr:" + viewController.getTemplate();			
    } 
    if (isNodeTypeSupported()) {      
      return "jcr:" + getTemplatePath();
    }     
    return "jcr:" + viewController.getTemplate();		
  }
  
  public Collection getNodeIteratorList() throws Exception {
    UIJCRExplorer uiExplorer = (UIJCRExplorer) getAncestorOfType(UIJCRExplorer.class);
    uiExplorer.getSession().refresh(false) ;
    Node currentNode = uiExplorer.getSelectNode();
    Node selectedNode;
    int nbColumns = 1;
    int start;
    int columnMax;
    boolean rootFirst = false;
    List<Iterator> iteratorList;
    String currentPath = currentNode.getPath();
    
    iteratorList = new ArrayList<Iterator>(nbColumns);
    columnMax = nbColumns;
    
    if (hasChildren(currentNode)) {
      columnMax--;
    }
    selectedNode = currentNode;
    for (int i = 1; i < columnMax; i++) {
      try {
        selectedNode = selectedNode.getParent();
      } catch (Exception e) {
        rootFirst = true;
        break;
      }
    }
    if ("/".equals(selectedNode.getPath())) {
      rootFirst = true;
    }
    if (rootFirst) {
      Map<String,Node> m = new TreeMap();
      m.put("/", selectedNode);
      iteratorList.add(m.values().iterator());
      start = 2;
    } else {
      start = 1;
      try {
        Node parentNode = selectedNode.getParent();
        Map<String,Node> m = new TreeMap();
        m.put(parentNode.getPath(), parentNode);
        iteratorList.add(m.values().iterator());
        if(hasChildren(parentNode)) {
          Map nodeMap = getChildrenMap(parentNode);
          iteratorList.add(nodeMap.values().iterator());
        }
      } catch (Exception e) {
        return null;
      }
    }
    if ("/".equals(currentNode.getPath()))
      nbColumns = 2;
    for (int i = start; i <= nbColumns; i++) {
      if (hasChildren(selectedNode)) {
        Map nodeMap = getChildrenMap(selectedNode);
        iteratorList.add(nodeMap.values().iterator());
        if (i < nbColumns) {
          Iterator iter = nodeMap.values().iterator();
          while (iter.hasNext()) {
            Node node = (Node) iter.next();
            String nodePath = node.getPath();
            if (currentPath.indexOf(nodePath) > -1) {
              selectedNode = node;
              break;
            }
          }
        }
      } else
        break;
    }
    return iteratorList;
  }
  
  public Map getChildrenMap(Node pNode) throws Exception {
    UIJCRExplorer explorer = (UIJCRExplorer) getAncestorOfType(UIJCRExplorer.class);
    Preference pref = explorer.getPreference();
    String order = pref.getOrder();
    String sort = pref.getSort();
    String property = pref.getProperty();
    Iterator childrenIterator ;
    List<Node> temp  = new ArrayList<Node>() ;
    Node tempNode ;
    if (pNode == null){
      childrenIterator = node_.getNodes();
      tempNode = node_ ;      
    }else{
      childrenIterator = pNode.getNodes();
      tempNode = pNode ;      
    }
    
    if(isReferenceableNode(tempNode) && pref.isShowPreferenceDocuments()) {
      String[] workspaces = repositoryService_.getRepository().getWorkspaceNames() ;
      for(String workspace:workspaces) {
        Session session = repositoryService_.getRepository().getSystemSession(workspace) ;
        Node taxonomyNode = session.getNodeByUUID(tempNode.getUUID()) ;
        for(PropertyIterator categoriesIter = taxonomyNode.getReferences(); categoriesIter.hasNext();) {
          Property exoCategoryProp = categoriesIter.nextProperty();
          Node refNode = exoCategoryProp.getParent() ;
          temp.add(refNode) ;
        }
      }
    }
    if(!pref.isShowNonDocumentType()){
      List documentTypes = templateService_.getDocumentTemplates() ;      
      while (childrenIterator.hasNext()){
        Node node = (Node)childrenIterator.next() ;
        NodeType type = node.getPrimaryNodeType() ;
        if("nt:unstructured".equals(type.getName()) || "nt:folder".equals(type.getName()))
          temp.add(node) ;
        else
          if(documentTypes.contains(type.getName())) 
            temp.add(node) ;
      }
    }
    if(temp.size() > 0) {      
      while(childrenIterator.hasNext()) {
        temp.add((Node)childrenIterator.next()) ;
      }
      childrenIterator = temp.iterator() ;
    }
    
    Map<String,Node> nodesMap;
    int j = 0;
    String nodeType = "";
    if (Preference.ALPHABETICAL_SORT.equals(sort)) {
      nodesMap = new TreeMap(new AlphaNodeComparator(order));
      while (childrenIterator.hasNext()) {
        Node childNode = (Node)childrenIterator.next();
        try {
          nodeType = childNode.getPrimaryNodeType().getName();
          // unstructured, taxonomy node considered as folder node for sorting
          if ("nt:unstructured".equals(nodeType))
            nodeType = "nt:folder";
          if("exo:taxonomy".equals(nodeType))
            nodeType = "nt:folder" ;
        } catch (Exception ex) {
        }
        if ("nt:folder".equals(nodeType))
          nodeType = "1";
        else
          nodeType = "2";
        String key = new StringBuffer(nodeType).append(childNode.getName())
        .append(j).toString().toLowerCase();
        nodesMap.put(key, childNode);
        j++;
      }
    }else if(Preference.PROPERTY_SORT.equals(sort)) {
      nodesMap = new TreeMap(new PropertiesComparator(order));
      String indexList = "" ;
      String remainList = "" ;
      if(order.equals("Ascending")) {
        indexList = "1" ;
        remainList = "2" ;
      }else {
        indexList = "2" ;
        remainList = "1" ;
      }
      List remainNodes = new ArrayList() ;
      while (childrenIterator.hasNext()) {
        Node childNode = (Node)childrenIterator.next();
        if(childNode.hasProperty(property)) {
          NodeType nt = childNode.getPrimaryNodeType() ;
          PropertyDefinition[] propertiesDef = nt.getPropertyDefinitions() ;
          try {
            for(int i = 0; i < propertiesDef.length; i ++) {
              if(propertiesDef[i].getName().equals(property)) {
                int type = propertiesDef[i].getRequiredType() ;
                if(type == 1){ //String
                  String key = new StringBuffer(indexList).append(childNode.getProperty(property).getString()).append(j).toString().toLowerCase();
                  nodesMap.put(key, childNode);
                }else if( type == 3){ //Long
                  String value = String.valueOf(childNode.getProperty(property).getLong()) ;
                  String key = new StringBuffer(indexList).append(value).append(j).toString().toLowerCase();
                  nodesMap.put(key, childNode);
                }else if(type == 5){ //Date
                  String value = String.valueOf(childNode.getProperty(property).getDate().getTimeInMillis()) ;
                  String key = new StringBuffer(indexList).append(value).append(j).toString().toLowerCase();
                  nodesMap.put(key, childNode);
                }else if( type == 6){ //Boolean
                  String value = String.valueOf(childNode.getProperty(property).getBoolean()) ;
                  String key = new StringBuffer(indexList).append(value).append(j).toString().toLowerCase();
                  nodesMap.put(key, childNode);
                }else {
                  remainNodes.add(childNode) ;
                  break ;
                }
              }
            }
          } catch (Exception ex) {
            remainNodes.add(childNode) ;
            break ;
          }
        }else {
          remainNodes.add(childNode) ;
        }
        j++;
      }
      for(int i = 0; i < remainNodes.size(); i ++){
        Node node = (Node)remainNodes.get(i) ;
        String key = new StringBuffer(remainList).append(node.getName()).append(j).toString().toLowerCase();
        nodesMap.put(key, node);
        j++ ;
      }
    }else {
      // Sort by node type
      nodesMap = new TreeMap(new TypeNodeComparator(order));
      String mimeType = "";
      Node jcrContentNode;
      StringTokenizer strTk;
      while (childrenIterator.hasNext()) {
        Node childNode = (Node)childrenIterator.next();
        jcrContentNode = null;
        nodeType = "default";
        mimeType = "default";
        try {
          nodeType = childNode.getPrimaryNodeType().getName();
          // unstructured, taxonomy node considered as folder node for sorting
          if ("nt:unstructured".equals(nodeType))
            nodeType = "nt:folder" ;
          if("exo:taxonomy".equals(nodeType))
            nodeType = "nt:folder" ;
          if ("nt:file".equals(nodeType))
            jcrContentNode = childNode.getNode("jcr:content");
          else if ("nt:resource".equals(nodeType))
            jcrContentNode = childNode;
          if (jcrContentNode != null) {
            if (jcrContentNode.hasProperty("jcr:mimeType"))
              mimeType = jcrContentNode.getProperty("jcr:mimeType").getString();
          }
        } catch (Exception ex) {
        }
        // 2 node types for sorting : folder or file
        if ("nt:folder".equals(nodeType))
          nodeType = "folder";
        else
          nodeType = "file";
        // mime type if available. Ex : pdf for application/pdf
        if (!"default".equals(mimeType)) {
          strTk = new StringTokenizer(mimeType, "/");
          if (strTk.countTokens() == 2) {
            strTk.nextToken();
            mimeType = strTk.nextToken();
          }
        }
        String key = new StringBuffer(nodeType).append("//").append(mimeType)
        .append("//").append(childNode.getName()).append(j).toString()
        .toLowerCase();
        nodesMap.put(key, childNode);
        j++;
      }
    }
    return nodesMap;
  }
  
  public boolean hasHiddenNode() { return hasHiddenNode_ ; } 
  
  public boolean isRssLink() { return delegate.isRssLink() ; }
  public String getRssLink() { return delegate.getRssLink() ; }
  
  public List<Node> getRelations() throws Exception {
    return delegate.getRelations();
  }
  
  public List<Node> getAttachments() throws Exception {
    return delegate.getAttachments();
  }
  
  public String getAttachmentSize(Node node){
  	return delegate.getAttachmentSize(node);
  }
  
  public List<Value> getMultiValues(Node node, String name) throws Exception {    
    List<Value> list = new ArrayList<Value>() ;   
      if(!node.hasProperty(name)) return list ;
      if(!node.getProperty(name).getDefinition().isMultiple()){
        list.add(node.getProperty(name).getValue()) ;
        return list ;
      }    
      Value[] values =node.getProperty(name).getValues() ;    
      for( Value value:values) { list.add(value) ;}            
    return list ;
  }
  
  public Iterator getChildrenIterator() throws Exception {
    return getChildrenIterator(null);
  }
  
  public List getDatas() throws Exception{  
    //uiExplorer_.getSession().refresh(false) ;
    List childrenList = getChildrenList() ;
    uiPageIterator_.setPageList(new ContentPageList(childrenList)) ;
    return uiPageIterator_.getPageListDataHandler().getDatas() ;
  }
  private List getChildrenList() throws Exception {
    List<Node> childrenList = new ArrayList<Node>() ;
    Iterator itr = getChildrenIterator(null);
    while(itr.hasNext()){
      childrenList.add((Node)itr.next()) ;
    }
    return childrenList ;
  }
  
  public Iterator getChildrenIterator(Node pNode) throws Exception {		
    Map children = getChildrenMap(pNode) ;
    return children.values().iterator();
  }
  
  public Collection getChildrenPerRow() throws Exception {    
    return Utils.getChildrenPerRow(NB_PER_ROW, getDatas().iterator());
  }
  
  public String getThumbnailName(Node node) throws RepositoryException {    
    String icon = getLargeNodeTypeIcon(node);
    return icon + "<br clear=all/>" + node.getName();
  }
  
  public String getNodeFlag(Node node) {
    return JCRUtil.getNodeFlag(node);
  }
  
  public String getSmallNodeTypeIcon(Node node) {
	try{    
    return CMSUtils.getSmallNodeTypeIcon(node) ;
	}catch(Exception e) {
		e.printStackTrace() ;
	}  
    return null ;
  }
  
  public void isReferencedNode() {
    CMSUtils.setReference(true) ;    
  }
  
  public String getLargeNodeTypeIcon(Node node) {
    return CMSUtils.getLargeNodeTypeIcon(node);
  }
  
  public boolean isEditPossible(String nodePath) throws Exception {
    boolean isEdit = true;
    Node childNode ;    
    if(nodePath.indexOf(";") > -1){
      String[] array = nodePath.split(";") ;
      if(uiExplorer_.getSession().getWorkspace().getName().equals(array[0].trim()))
        childNode = (Node)uiExplorer_.getSession().getItem(array[1].trim()) ; 
      else{
        Session session = repositoryService_.getRepository().login(array[0].trim()) ;
        childNode = (Node)session.getItem(array[1]) ;
      }      
    }else{
      Node currentNode =  uiExplorer_.getSelectNode() ;
      String name = nodePath.substring(nodePath.lastIndexOf("/") + 1) ;
      //childNode = (Node)uiExplorer_.getSession().getItem(nodePath) ;
      childNode = currentNode.getNode(name) ;
    }      
    String nodeType = childNode.getPrimaryNodeType().getName();
    for (int i = 0; i < NON_EDITABLE_NODETYPES.length; i++) {
      String nonEditableType = NON_EDITABLE_NODETYPES[i];
      if (nonEditableType.equals(nodeType))
        return false;
    }    
    return isEdit;
  }
  
  public boolean hasEditPermissions(Node editNode){
    try {
      editNode.getSession().checkPermission(editNode.getPath(), "add_node,set_property");
    }catch(Exception e) {
      return false ;
    } 
    return true;
  }
  
  public boolean hasRemovePermissions(Node curNode){
    try {
      curNode.getSession().checkPermission(curNode.getPath(), "remove");
    }catch(Exception e) {
      return false ;
    } 
    return true;
  }
  
  public boolean isEditPossible(Node node) throws Exception {
    boolean isEdit = true;
    String nodeType = node.getPrimaryNodeType().getName();
    for (int i = 0; i < NON_EDITABLE_NODETYPES.length; i++) {
      String nonEditableType = NON_EDITABLE_NODETYPES[i];
      if (nonEditableType.equals(nodeType))
        return false;
    }
    return isEdit;
  }
  
  public Node getNode() throws Exception {
    Node node = delegate.getNode() ;     
    if(node.hasProperty("exo:language")) {
      String defaultLang = node.getProperty("exo:language").getString() ;
      if(!language_.equals("default") && !language_.equals(defaultLang)) {
        Node curNode = node.getNode("languages/" + language_) ;
        return curNode ;
      } 
    }    
    return node;    
  }
  
  public List getSupportedLocalise() throws Exception {
    List<String> localeNames = new ArrayList<String> () ;    
    Node currentNode = delegate.getNode() ;    
    String defaultLocale = currentNode.getProperty("exo:language").getString() ;
    if(! currentNode.hasNode("languages")) return localeNames ;    
    Node languages = currentNode.getNode("languages") ;      
    for(NodeIterator iter = languages.getNodes(); iter.hasNext();) {
      localeNames.add(iter.nextNode().getName()) ;
    }    
    if(Collections.frequency(localeNames,defaultLocale) == 0) {
      localeNames.add(defaultLocale) ;
    }   
    return localeNames ;
  }
  
  public void setNode(Node node) {
    delegate.setNode(node);
  }
  public String getWebDAVServerPrefix() throws Exception{    
    PortletRequest request = (PortletRequest)FacesContext.getCurrentInstance().getExternalContext().getRequest() ;
    return request.getScheme() + "://" + request.getServerName() + ":" +
    String.format("%s", request.getServerPort()) ;
  }  
  public String getPortalName() {
    return delegate.getPortalName();
  }
  
  public String getWorkspaceName() throws Exception {        
    return delegate.getWorkspaceName();
  }
  
  public boolean isNodeTypeSupported() {
    return delegate.isNodeTypeSupported();
  }
  
  public Object getComponentInstanceOfType(String className) throws Exception {
    return delegate.getComponentInstanceOfType(className);
  }
  
  public WikiContext getWikiContext() {
    return delegate.getWikiContext();
  }
  
  public WikiEngineService getWikiService() {
    return delegate.getWikiService();
  }
  
  public String getNodeType() throws Exception {
    return delegate.getNodeType();
  }
  
  public String getNodeUri(Node node) throws Exception {
    return node.getPath();
  }
  
  public String getTemplatePath() throws Exception {
    return delegate.getTemplatePath();
  }  
  
  public boolean isVersionableOrAncestor(Node node) throws RepositoryException {
    if (isVersionable(node) || isAncestorVersionable(node))
      return true;    
    return false;
  }
  
  public boolean isVersionable(Node node) throws RepositoryException {
    return node.isNodeType("mix:versionable") && !node.isNodeType("nt:frozenNode");
  }
  
  public boolean isAncestorVersionable(Node node) throws RepositoryException {
    int depth = node.getDepth();
    int i = depth - 1;    
    while (i != 0) {
      Node parent = (Node) node.getAncestor(i);
      if(parent == null ) return false ;
      if (parent.isNodeType("mix:versionable"))
        return true;
      i--;
    }
    return false;
  }
  
  public String getVersionNumber(Node node) throws RepositoryException {
    if(!isVersionable(node))
      return "-";
    return node.getBaseVersion().getName();
  }
  
  public List<Node> getCustomActions(Node node) throws Exception {
    List<Node> safeActions = new ArrayList<Node>();    
    try {
      String user = SessionContainer.getInstance().getRemoteUser();
      List<Node> unsafeActions = actionService_.getActions(node, ActionServiceContainer.READ_PHASE);
      for (Iterator iter = unsafeActions.iterator(); iter.hasNext();) {
        Node actionNode = (Node) iter.next();
        Value[] roles = actionNode.getProperty("exo:roles").getValues();
        for (int i = 0; i < roles.length; i++) {
          String role = roles[i].getString();
          if(securityService_.hasMembershipInGroup(user, role))
            safeActions.add(actionNode);
        }
      }      
    }
    catch(AccessDeniedException ignore) {
    }
    
    return safeActions;
  }
  
  public String toHTML(String str) {
    return CMSUtils.toHTML(str) ;
  }
  
  public String getLanguage() { return language_; }
  public void setLanguage(String language) { this.language_ = language; }
  
  static public class ChangeNodeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {      
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();      
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uicomp.getAncestorOfType(UIJCRExplorer.class);      
      String uri = event.getParameter(OBJECTID);
      if(uri.indexOf(":/") > -1) {
        uiExplorer.setSelectNode(uri) ;       
        return ;
      }
      Information iprovider = findInformationProvider(uicomp);
      String prefPath = uiExplorer.getPreferencesPath() ;
      uicomp.showRelation = false ;
      if((!prefPath.equals(""))&&
          (uiExplorer.getCurrentWorkspace().equals(uiExplorer.getPreferencesWorkspace()))) {
        if(!uri.contains(prefPath)) {      		
          JCRExceptionManager.process(iprovider,new PathNotFoundException());
          return ;
        }
        try {
          if ((".." + prefPath).equals(uri)) {
            if (prefPath.equals(uiExplorer.currentNode_.getPath())) {
              uiExplorer.setSelectNode(uiExplorer.getSelectNode().getParent());
            }
          } else {
            uiExplorer.setSelectNode(uri);
          }
        } catch (Exception e) {
          JCRExceptionManager.process(iprovider, e);
        }
      }else {
        try {
          if ("../".equals(uri)) {
            if (!"/".equals(uiExplorer.currentNode_.getPath())) {
              uiExplorer.setSelectNode(uiExplorer.getSelectNode().getParent());
            }
          } else {
            uiExplorer.setSelectNode(uri);
          }
        } catch (Exception e) {
          JCRExceptionManager.process(iprovider, e);
        }
      }			
    }
  }
  
  static public class ViewNodeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uicomp.getAncestorOfType(UIJCRExplorer.class);      
      String uri = event.getParameter(OBJECTID);
      uiExplorer.setSelectNode(uri) ;
    }
  }
  
  static public class EditDocumentActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uicomp.getAncestorOfType(UIJCRExplorer.class);
      Information iprovider = findInformationProvider(uicomp);
      String name = event.getParameter(OBJECTID);
      boolean isReferenced = false ;
      Session session = null ;
      try {        
        Node selectedNode ;
        if(name.indexOf(";") > -1) {
          isReferenced = true ;
          String[] array = name.split(";") ;
          if(!uicomp.uiExplorer_.getSession().getWorkspace().getName().equals(array[0].trim()))
            session = uicomp.repositoryService_.getRepository().login(array[0].trim()) ;            
          else
            session = uicomp.uiExplorer_.getSession() ;
          selectedNode = (Node)session.getItem(array[1].trim()) ;          
        }else {
          selectedNode = uiExplorer.getSelectNode();
          name = name.substring(name.lastIndexOf("/") + 1) ;
          selectedNode = selectedNode.getNode(name);          
        }
        String nodeType = selectedNode.getPrimaryNodeType().getName();
//        if ("nt:file".equals(nodeType)) {
//          Node contentNode = selectedNode.getNode("jcr:content");
//          String contentNodeType = contentNode.getPrimaryNodeType().getName();
//          String mimeType = contentNode.getProperty("jcr:mimeType").getString() ;
//          if ("nt:resource".equals(contentNodeType) && !mimeType.startsWith("text")) {
//            UIUploadForm form = (UIUploadForm) uicomp.getSibling(UIUploadForm.class);
//            form.update(selectedNode.getName()) ;
//            uicomp.setRenderedSibling(UIUploadForm.class);
//            return;
//          }
//        }
        UIDocumentFormController uiNewNodeController = 
          (UIDocumentFormController) uicomp.getSibling(UIDocumentFormController.class);
        uiNewNodeController.showEditDocumentForm(selectedNode, isReferenced);
        uicomp.setRenderedSibling(UIDocumentFormController.class);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }
  
  static public class RenameActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {      
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();
      String renameNodePath = event.getParameter(OBJECTID);
      Information iprovider = findInformationProvider(uicomp);
      Node renameNode ;
      Session session ;
      boolean isReferencedNode = false ;
      try {
        if(renameNodePath.indexOf(";") > -1) {
          isReferencedNode = true ;
          String[] array = renameNodePath.split(";") ;
          if(!uicomp.uiExplorer_.getSession().getWorkspace().getName().equals(array[0].trim())){
            session = uicomp.repositoryService_.getRepository().login(array[0].trim()) ;
            renameNode = (Node)session.getItem(array[1].trim()) ;
          }else
            renameNode = (Node)uicomp.uiExplorer_.getSession().getItem(array[1].trim()) ;          
        }else {
          renameNode = (Node)uicomp.uiExplorer_.getSession().getItem(renameNodePath) ;
        }
        UIRenameForm renameForm = (UIRenameForm)uicomp.getSibling(UIRenameForm.class) ;        
        renameForm.update(renameNode, isReferencedNode) ;
        uicomp.setRenderedSibling(UIRenameForm.class) ;
      } catch (Exception e) {
        e.printStackTrace() ;
        JCRExceptionManager.process(iprovider, e);
      }
    }    
  }
  static public class SaveNodeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      ResourceBundle res = getApplicationResourceBundle() ;
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uicomp.getAncestorOfType(UIJCRExplorer.class);
      String nodePath = event.getParameter(OBJECTID);
      Node node = null ;
      Information iprovider = findInformationProvider(uicomp);
      try {
        if(nodePath.indexOf(";") > -1) {
          String[] array = nodePath.split(";") ;
          if(uiExplorer.getSession().getWorkspace().getName().equals(array[0].trim()))
            node = (Node)uiExplorer.getSession().getItem(array[1].trim()); 
          else {
            Session session = uicomp.repositoryService_.getRepository().login(array[0].trim()) ;
            node = (Node)session.getItem(array[1].trim()) ;            
          }
        }else
          node = (Node)uiExplorer.getSession().getItem(nodePath); 
        Object[] args = { nodePath };
        if(node.isNew()) {
          iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR ,
              res.getString("UIDocumentInfo.msg.unable-save-node"),args));
          return ;
        }
        node.save(); 
        if(!uiExplorer.getPreference().isJcrEnable()) uiExplorer.getSession().save() ;
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_INFO,
            res.getString("UIDocumentInfo.msg.save-node-success"), args));
      } catch (Exception e) {
        e.printStackTrace() ;
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }
  
  static public class CopyActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();
      ResourceBundle res = getApplicationResourceBundle() ;
      String srcPath = event.getParameter(OBJECTID);
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uicomp.getAncestorOfType(UIJCRExplorer.class);
      Information iprovider = findInformationProvider(uicomp);
      try {
        List<ClipboardCommand> clipboards = uiExplorer.getAllClipBoard() ;
        for(ClipboardCommand command:clipboards) {
          if(command.getSrcPath().equals(srcPath)) {
            clipboards.remove(command) ;
            break ;
          }
        }				
        ClipboardCommand clipboard = new ClipboardCommand() ;
        clipboard.setType(ClipboardCommand.COPY) ;
        clipboard.setSrcPath(srcPath) ;
        uiExplorer.getAllClipBoard().add(clipboard) ;				               
        Object[] args = { srcPath };
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_INFO,
            res.getString("UIDocumentInfo.msg.node-copied"), args));
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }
  
  static public class CutActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      ResourceBundle res = getApplicationResourceBundle() ;
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();
      String nodePath = event.getParameter(OBJECTID);
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uicomp.getAncestorOfType(UIJCRExplorer.class);
      Information iprovider = findInformationProvider(uicomp);
      try {
        List<ClipboardCommand> clipboards = uiExplorer.getAllClipBoard() ;
        for(ClipboardCommand command:clipboards) {
          if(command.getSrcPath().equals(nodePath)) {
            clipboards.remove(command) ;
            break ;
          }
        }				
        ClipboardCommand clipboard = new ClipboardCommand() ;
        clipboard.setType(ClipboardCommand.CUT) ;
        clipboard.setSrcPath(nodePath) ;
        uiExplorer.getAllClipBoard().add(clipboard) ;											
        Object[] args = { nodePath };
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_INFO,
            res.getString("UIDocumentInfo.msg.node-cut"), args));
      } catch (Exception e) {
        e.printStackTrace() ;
        JCRExceptionManager.process(iprovider, e);
      }
    }
    
  }
  
  static public class RemoveNodeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      ResourceBundle res = getApplicationResourceBundle() ;
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uicomp.getAncestorOfType(UIJCRExplorer.class);
      String nodePath = event.getParameter(OBJECTID);
      Information iprovider = findInformationProvider(uicomp);
      Node node ;
      try {        
        if ("/".equals(nodePath)) {
          iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR ,
              res.getString("UIDocumentInfo.msg.remove-root")));
          return;
        }
        if(nodePath.indexOf(";") > -1 ) {
          String[] array = nodePath.split(";") ;
          String workspaceName = array[0].trim() ;
          String path = array[1].trim() ;
          if(uiExplorer.getSession().getWorkspace().getName().equals(workspaceName)) {
            node = (Node)uiExplorer.getSession().getItem(path);            
          }else {
            Session session = uicomp.repositoryService_.getRepository().login(workspaceName) ;
            node = (Node)session.getItem(path) ;                
          }
          Node parentNode = node.getParent() ;
          node.remove();
          parentNode.getSession().save() ;
        }else{
          String name = nodePath.substring(nodePath.lastIndexOf("/") + 1) ;
          Node parentNode = uicomp.uiExplorer_.getSelectNode() ;
          node = parentNode.getNode(name);
          node.remove();
          parentNode.save() ;          
          uicomp.uiExplorer_.refreshSession() ;
        } 
        //uicomp.uiPageIterator_.setPageList(new ContentPageList(uicomp.getChildrenList())) ;
        Object[] args = { nodePath };
        iprovider.addMessage(new ExoFacesMessage(
            res.getString("UIDocumentInfo.msg.node-remove-success"), args));
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }
  
  static public class CheckinNodeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();
      String nodePath = event.getParameter(OBJECTID);
      Node node ;
      Information iprovider = findInformationProvider(uicomp);
      try {
        if(nodePath.indexOf(";") > -1) {
          String[] array = nodePath.split(";") ;          
          if(uicomp.uiExplorer_.getSession().getWorkspace().getName().equals(array[0].trim()))
            node = (Node)uicomp.uiExplorer_.getSession().getItem(array[1].trim()); 
          else {
            Session session = uicomp.repositoryService_.getRepository().login(array[0].trim()) ;
            node = (Node)session.getItem(array[1].trim()) ;            
          }
        }else
          node = (Node)uicomp.uiExplorer_.getSession().getItem(nodePath);
        node.checkin();
        //uicomp.uiPageIterator_.setPageList(new ContentPageList(uicomp.getChildrenList())) ;
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }
  
  static public class CheckoutNodeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();
      String nodePath = event.getParameter(OBJECTID);
      Node node ;
      Information iprovider = findInformationProvider(uicomp);
      try {
        if(nodePath.indexOf(";") > -1) {
          String[] array = nodePath.split(";") ;
          if(uicomp.uiExplorer_.getSession().getWorkspace().getName().equals(array[0].trim()))
            node = (Node)uicomp.uiExplorer_.getSession().getItem(array[1].trim()); 
          else {
            Session session = uicomp.repositoryService_.getRepository().login(array[0].trim()) ;
            node = (Node)session.getItem(array[1].trim()) ;            
          }
        }else
          node = (Node)uicomp.uiExplorer_.getSession().getItem(nodePath);
        node.checkout();
        //uicomp.uiPageIterator_.setPageList(new ContentPageList(uicomp.getChildrenList())) ;
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }
  
  static public class LockActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();
      Information iprovider = findInformationProvider(uicomp);
      Node node ;
      String name = event.getParameter(OBJECTID);
      try {
        if(name.indexOf(";") > -1) {
          String[] array = name.split(";") ;
          if(!uicomp.uiExplorer_.getSession().getWorkspace().getName().equals(array[0].trim())) {
            Session session = uicomp.repositoryService_.getRepository().getSystemSession(array[0].trim()) ;
            node = (Node)session.getItem(array[1].trim()) ;
          }else
            node = (Node)uicomp.uiExplorer_.getSession().getItem(array[1].trim());
        }else {
          name = name.substring(name.lastIndexOf("/") + 1) ;
          node = uicomp.node_.getNode(name);
        }      
        if(!node.isNodeType("mix:lockable")) {
          node.addMixin("mix:lockable");
          node.save();
        }
        node.lock(true, true);        
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }  
  
  static public class UnlockActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();
      Information iprovider = findInformationProvider(uicomp);
      String name = event.getParameter(OBJECTID);
      Node node ;
      try {
        if(name.indexOf(";") > -1) {
          String[] array = name.split(";") ;
          if(!uicomp.uiExplorer_.getSession().getWorkspace().getName().equals(array[0].trim())) {
            Session session = uicomp.repositoryService_.getRepository().getSystemSession(array[0].trim()) ;
            node = (Node)session.getItem(array[1].trim()) ;
          }else
            node = (Node)uicomp.uiExplorer_.getSession().getItem(array[1].trim());
        }else {
          name = name.substring(name.lastIndexOf("/") + 1) ;
          node = uicomp.node_.getNode(name);
        }
        node.unlock();        
      } catch (Exception e) {
        e.printStackTrace() ;
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }    
  
  static public class CustomActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      ResourceBundle res = getApplicationResourceBundle() ;
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();
      String nodeName = event.getParameter(OBJECTID);
      nodeName = nodeName.substring(nodeName.lastIndexOf("/") + 1) ;
      String actionName = event.getParameter(ACTION_NAME);
      Information iprovider = findInformationProvider(uicomp);
      try {
        Node node = uicomp.node_.getNode(nodeName);
        String userId = SessionContainer.getInstance().getRemoteUser();               
        
        uicomp.actionService_.executeAction(userId, node, actionName);
        Object[] args = { actionName };
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_INFO,
            res.getString("UIDocumentInfo.msg.custom-action-success"), args));
        uicomp.uiExplorer_.getSession().refresh(false) ;
        uicomp.uiExplorer_.broadcastOnChange(uicomp.uiExplorer_.getSelectNode()) ;
      } catch (Exception e) {
        //e.printStackTrace() ;
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }      
  
  static public class HideRelationPanelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();
      uicomp.showRelation = false ;
    }		
  }
  
  static public class SortActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();
      String sortParam = event.getParameter(OBJECTID);
      String[] array = sortParam.split(";") ;
      Preference pref = uicomp.uiExplorer_.getPreference() ;
      if(array.length == 2) {
        pref.setSort(array[0].trim()) ;
        pref.setOrder(array[1].trim()) ; 
      }else if(array.length == 3){
        pref.setSort(array[0].trim()) ;
        pref.setProperty(array[1].trim()) ;
        pref.setOrder(array[2].trim()) ;
      }else {
        return ;
      }       
      uicomp.uiExplorer_.broadcastOnChange(uicomp.node_) ;
    }   
  }
  
  static public class ChangeLanguageActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIDocumentInfo uicomp = (UIDocumentInfo) event.getComponent();
      uicomp.setLanguage(event.getParameter(OBJECTID));
    }   
  }  
}
