/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.jcr.component;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;

import javax.jcr.Item;
import javax.jcr.ItemNotFoundException;
import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.Session;
import javax.jcr.Value;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.NodeTypeManager;
import javax.jcr.nodetype.PropertyDefinition;

import org.exoplatform.faces.cms.CMSDialogComponent;
import org.exoplatform.faces.core.component.UICalendarInput;
import org.exoplatform.faces.core.component.UIInput;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleMulltipartForm;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.services.cms.CmsConfigurationService;
import org.exoplatform.services.cms.metadata.MetadataService;
import org.exoplatform.services.cms.templates.TemplateService;
import org.exoplatform.services.jcr.RepositoryService;

/**
 * @email: nguyenkequanghung@yahoo.com
 * @version: $Id: UIContentDisplayer.java,v 1.2 2006/04/04
 */
public class UIMetadataForm extends UISimpleMulltipartForm implements CMSDialogComponent,
    ChangeNodeListener {

  final static public String      CHANGE_METADATA_ACTION = "changeMetadata";

  final static public String      ADD_METADATA_ACTION    = "addMetadata";

  final static public String      REFRESH_ACTION         = "refresh";

  private Session                 session_;

  private CmsConfigurationService cmsConfigurationService_;

  private UISelectBox             uiSelect_;

  private String                  metadataName_;

  private boolean                 isView_                = true;

  protected Node                  node_;

  private List<SelectItem>        options_               = new ArrayList<SelectItem>();

  private MetadataService         mservice_;

  private TemplateService         tservice_;

  public UIMetadataForm(RepositoryService repositoryService, TemplateService tservice,
      CmsConfigurationService cmsConfigurationService, MetadataService mservice) throws Exception {

    super("uiMetadataForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");
    cmsConfigurationService_ = cmsConfigurationService;
    session_ = repositoryService.getRepository().getSystemSession(
        cmsConfigurationService_.getWorkspace());
    mservice_ = mservice;
    tservice_ = tservice;
    uiSelect_ = new UISelectBox("metadataType", "", null);
    uiSelect_.setUpdateOnChangeAction(CHANGE_METADATA_ACTION);
    add(uiSelect_);

    addActionListener(ChangeMetadataActionListener.class, CHANGE_METADATA_ACTION);
    addActionListener(AddMetadataActionListener.class, ADD_METADATA_ACTION);
    addActionListener(SaveMetadataActionListener.class, SAVE_ACTION);
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
    addActionListener(BackActionListener.class, BACK_ACTION);
  }

  public void onChange(UIJCRExplorer uiExplorer, Node node) throws Exception {
    node_ = node;
    setRendered(false);
  }

  public String getAddParameter() {
    return ADD_METADATA_ACTION;
  }

  public String getBackParameter() {
    return BACK_ACTION;
  }

  public Node getViewNode(String nodeType) throws Exception {
    try {
      Item primaryItem = node_.getPrimaryItem();
      if (primaryItem != null && primaryItem.isNode()) {
        Node primaryNode = (Node) node_.getPrimaryItem();
        if (primaryNode.isNodeType(nodeType))
          return primaryNode;
      }
    } catch (Exception e) {
    }

    return node_;
  }

  public Node getNode() throws Exception {
    try {
      Item primaryItem = node_.getPrimaryItem();
      if (primaryItem != null && primaryItem.isNode()) {
        Node primaryNode = (Node) node_.getPrimaryItem();
        if (primaryNode.isNodeType(metadataName_))
          return primaryNode;
      }
    } catch (Exception e) {
    }

    return node_;
  }

  public List getMultiValues(Node node, String name) throws Exception {
    List list = new ArrayList();
    try {
      if (!node.hasProperty(name))
        return list;
      if (!node.getProperty(name).getDefinition().isMultiple()) {
        list.add(node.getProperty(name).getString());
        return list;
      }
      Value[] values = node.getProperty(name).getValues();
      for (Value value : values) {
        list.add(value.getString());
      }
    } catch (Exception e) {
    }
    return list;
  }

  public void update(boolean isView) throws Exception {
    isView_ = isView;
    options_.clear();
    boolean isMeta = false;
    NodeType[] nodeTypes = node_.getMixinNodeTypes();
    for (NodeType nt : nodeTypes) {
      if (mservice_.hasMetadata(nt.getName())) {
        options_.add(new SelectItem(nt.getName(), nt.getName()));
        if (!isMeta) {
          metadataName_ = nt.getName();
          isMeta = true;
        }
      }
    }
    Item primaryItem = null;
    try {
      primaryItem = node_.getPrimaryItem();
    } catch (ItemNotFoundException e) {
    }
    if (primaryItem != null && primaryItem.isNode()) {
      Node primaryNode = (Node) node_.getPrimaryItem();
      NodeType[] primaryTypes = primaryNode.getMixinNodeTypes();
      for (NodeType nt : primaryTypes) {
        if (mservice_.hasMetadata(nt.getName())) {
          options_.add(new SelectItem(nt.getName(), nt.getName()));
          if (!isMeta) {
            metadataName_ = nt.getName();
            isMeta = true;
          }
        }
      }
    }
    for(NodeType nt : node_.getPrimaryNodeType().getSupertypes()) {
      if (mservice_.hasMetadata(nt.getName())) {
        options_.add(new SelectItem(nt.getName(), nt.getName()));
        if (!isMeta) {
          metadataName_ = nt.getName();
          isMeta = true;
        }
      }
    }
    uiSelect_.setOptions(options_);
    uiSelect_.setValue(metadataName_);
  }

  public boolean isView() {
    return isView_;
  }

  public List getMetadataTemplates() throws Exception {
    // Using a HashSet to avoid duplicated entries
    HashSet templates = new HashSet<String>();
    try {
      if (isView_) {
        // Search for added mixin types
        for(NodeType mixin : node_.getMixinNodeTypes()) {
          try {
            String path = mservice_.getMetadataPath(mixin.getName(), false);
            templates.add("jcr:" + path);
          }
          catch (Exception ignore) {
          }
        }
        // Take into account the primary child item (notably useful when
        // processing nt:file nodes, as metadata are contained by jcr:content).
        Item primaryItem = null;
        try {
          primaryItem = node_.getPrimaryItem();
        } catch (ItemNotFoundException e) {
        }
        if (primaryItem != null && primaryItem.isNode()) {
          Node primaryNode = (Node) node_.getPrimaryItem();
          NodeType[] primaryTypes = primaryNode.getMixinNodeTypes();
          for (NodeType nt : primaryTypes) {
            try {
              String path = mservice_.getMetadataPath(nt.getName(), false);
              templates.add("jcr:" + path);
            } catch (Exception ignore) {
            }
          }
        }
        // Search for inherited mixin types
        for(NodeType nt : node_.getPrimaryNodeType().getSupertypes()) {
          if(nt.isMixin()) {
            try {
              String path = mservice_.getMetadataPath(nt.getName(), false);
              templates.add("jcr:" + path);
            }
            catch(Exception ignore) {
            }
          }
        }
      } else {
        templates.add("jcr:" + mservice_.getMetadataPath(metadataName_, true));
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
    return new ArrayList(templates);
  }

  public boolean isCheckedOut() throws RepositoryException {
    UIJCRExplorer uiExplorer = (UIJCRExplorer) getAncestorOfType(UIJCRExplorer.class);
    Node node = uiExplorer.getSelectNode();
    return node.isCheckedOut();
  }

  static public class ChangeMetadataActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIMetadataForm uicomp = (UIMetadataForm) event.getSource();
      uicomp.metadataName_ = uicomp.uiSelect_.getValue();
      uicomp.clearInputs();
      uicomp.add(uicomp.uiSelect_);
    }
  }

  static public class AddMetadataActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIMetadataForm uicomp = (UIMetadataForm) event.getSource();
      uicomp.update(false);
    }
  }

  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIMetadataForm uicomp = (UIMetadataForm) event.getSource();
      uicomp.update(true);
    }
  }

  static public class BackActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIMetadataForm uiForm = (UIMetadataForm) event.getComponent();
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uiForm.getAncestorOfType(UIJCRExplorer.class);
      uiExplorer.broadcastOnChange(uiExplorer.getSelectNode());
    }
  }

  static public class SaveMetadataActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      try {
        UIMetadataForm uiForm = (UIMetadataForm) event.getSource();
        Node node = uiForm.getViewNode(uiForm.metadataName_);
        NodeTypeManager ntManager_ = uiForm.session_.getWorkspace().getNodeTypeManager();
        PropertyDefinition[] props = ntManager_.getNodeType(uiForm.metadataName_)
            .getPropertyDefinitions();
        for (int i = 0; i < props.length; i++) {
          PropertyDefinition prop = props[i];
          String name = prop.getName();
          String inputName = name.substring(name.indexOf(":") + 1);
          if (!prop.isProtected()) {
            int requiredType = prop.getRequiredType();
            if (prop.isMultiple()) {
              List<UIInput> values = uiForm.getMultipleValuesUIInput(inputName);
              Value[] multiValue = getMultiValue(values, requiredType, uiForm.session_);
              node.setProperty(name, multiValue);
            } else {
              String value = uiForm.getUIInput(inputName).getValue();
              if (value != null && value.length() > 0) {
                if (requiredType == 6) { // boolean
                  node.setProperty(name, Boolean.parseBoolean(value));
                } else if (requiredType == 5) { // date
                  UICalendarInput cal = (UICalendarInput) uiForm.getUIInput(inputName);
                  Value val = uiForm.session_.getValueFactory().createValue(cal.getCalendar());
                  node.setProperty(name, val);
                } else {
                  node.setProperty(name, value);
                }
              }
            }
          }
        }
        node.save();
        node.getSession().save();
        uiForm.update(true);
      } catch (Exception e) {
        e.printStackTrace();
      }
    }

    private Value[] getMultiValue(List<UIInput> values, int valueType, Session session)
        throws Exception {
      List<Value> valueList = new ArrayList<Value>();
      for (int i = 0; i < values.size(); i++) {
        String value = values.get(i).getValue();
        if (value != null && value.length() > 0) {
          /*
           * switch(valueType) { case 6: //boolean
           * valueList.add(session.getValueFactory().createValue(Boolean.parseBoolean(value)));
           * break ; case 5: //date UICalendarInput cal =
           * (UICalendarInput)values.get(i) ;
           * valueList.add(session.getValueFactory().createValue(cal.getCalendar()));
           * break ; default:
           * valueList.add(session.getValueFactory().createValue(value)); break; }
           */
          if (valueType == 6) { // boolean
            valueList.add(session.getValueFactory().createValue(Boolean.parseBoolean(value)));
          } else if (valueType == 5) {
            UICalendarInput cal = (UICalendarInput) values.get(i);
            valueList.add(session.getValueFactory().createValue(cal.getCalendar()));
          } else {
            valueList.add(session.getValueFactory().createValue(value));
          }
        }
      }
      return valueList.toArray(new Value[] {});
    }
  }

}