/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.jcr.component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Set;

import javax.faces.application.FacesMessage;
import javax.jcr.Node;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.jcr.component.UIViewController.CancelActionListener;
import org.exoplatform.services.jcr.access.AccessControlEntry;
import org.exoplatform.services.jcr.access.PermissionType;
import org.exoplatform.services.jcr.core.ExtendedNode;
/**
 * @email:   tuan08@users.sourceforge.net
 * @version: $Id: UIContentDisplayer.java,v 1.2 2004/08/07 18:11:26 tuan08 Exp $
 */
public class UIPermissionsInfo extends UIExoCommand implements ChangeNodeListener {
      
  private static String ADD_PERMISSION_ACTION = "addPermission" ;
  private static String REMOVE_PERMISSION_ACTION = "removePermission" ;
  
  static Parameter[] ADD_PERMISSION_PARAMS = { new Parameter(ACTION , ADD_PERMISSION_ACTION)} ;
  static Parameter[] REMOVE_PERMISSION_PARAMS = { new Parameter(ACTION , REMOVE_PERMISSION_ACTION)} ;
  static Parameter[] BACK_PARAMS = { new Parameter(ACTION , CANCEL_ACTION)} ;
  private ExtendedNode node_ ;
  
  public UIPermissionsInfo() {
    setRendererType("VelocityRenderer") ;    
    addActionListener(AddPermissionActionListener.class, ADD_PERMISSION_ACTION);
    addActionListener(RemovePermissionActionListener.class, REMOVE_PERMISSION_ACTION);
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
  }
  
  public void  onChange(UIJCRExplorer explorer, Node node)  {
    node_ = (ExtendedNode) node ;
    setRendered(false) ;
  }
  
  public Parameter[] getAddPermissionParams() { return ADD_PERMISSION_PARAMS; }
  public Parameter[] getRemovePermissionParams() { return REMOVE_PERMISSION_PARAMS; }
  public Parameter[] getBackParams() { return BACK_PARAMS; }
  
  public List<PermissionBean> getPermissions() throws Exception {
    List<PermissionBean> permBeans = new ArrayList<PermissionBean>(); 
    List globalPermissionsList = node_.getACL().getPermissionEntries();
    Map identitiesAndPermissionsMap = new HashMap();
    for (Iterator globalPermissionsListIterator = globalPermissionsList.iterator(); globalPermissionsListIterator.hasNext();) {
      AccessControlEntry accessControlEntry = (AccessControlEntry)globalPermissionsListIterator.next();
      String currentIdentity = accessControlEntry.getIdentity();
      String currentPermission = accessControlEntry.getPermission();
      List currentPermissionsList = (List)identitiesAndPermissionsMap.get(currentIdentity);
      if (identitiesAndPermissionsMap.containsKey(currentIdentity) == false) {
        identitiesAndPermissionsMap.put(currentIdentity, null);
      }
      if (currentPermissionsList == null){
        currentPermissionsList = new ArrayList();
      }  
      if (currentPermissionsList.contains(currentPermission) == false) {
        currentPermissionsList.add(currentPermission);
      }
      identitiesAndPermissionsMap.put(currentIdentity,currentPermissionsList);
    }
    //at this stage we have Map <name, (List)permissions>
    Set keys = identitiesAndPermissionsMap.keySet();  
    for (Iterator tmpIterator = keys.iterator(); tmpIterator.hasNext();) {
      String userOrGroup = (String) tmpIterator.next();
      List permissions = (List) identitiesAndPermissionsMap.get(userOrGroup);
      PermissionBean permBean = new PermissionBean();
      permBean.setUsersOrGroups(userOrGroup);
      for (Iterator iterator = permissions.iterator(); iterator.hasNext();) {
        String perm = (String)iterator.next(); 
        if(PermissionType.READ.equals(perm))
          permBean.setRead(true);
        else if(PermissionType.ADD_NODE.equals(perm))
          permBean.setAddNode(true);
        else if(PermissionType.SET_PROPERTY.equals(perm))
          permBean.setSetProperty(true);
        else if(PermissionType.REMOVE.equals(perm))
          permBean.setRemove(true);
      }
      permBeans.add(permBean);
    }
    return permBeans;
  }
  
  public boolean hasPermissions() throws Exception{
    List perms = node_.getACL().getPermissionEntries();
    boolean isPermission = false;
    for (Iterator iter = perms.iterator(); iter.hasNext();) {
      AccessControlEntry accessControlEntry = (AccessControlEntry)iter.next();
      String userOrGroup = accessControlEntry.getPermission();
      if (!userOrGroup.equals("any"))        
        isPermission =  true;
    }
    return isPermission;
  }
  
  static public class AddPermissionActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIPermissionsInfo permissionsInfo = (UIPermissionsInfo) event.getSource(); 
      UIPermissionForm permissionForm = (UIPermissionForm) permissionsInfo.getSibling(UIPermissionForm.class);
      permissionForm.refresh();
      permissionsInfo.setRenderedSibling(UIPermissionForm.class);
    }
  }      

  static public class RemovePermissionActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception  {
      UIPermissionsInfo permissionsInfo = (UIPermissionsInfo) event.getSource();
      String name = event.getParameter(OBJECTID) ;
      Information iprovider = findInformationProvider(permissionsInfo);
      /*if(name.equals("any")) {
        ResourceBundle res = getApplicationResourceBundle() ;
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR,
            res.getString("UIPermissionsInfo.msg.cannot-delete-permission")));
        return;
      }*/
      try {
        permissionsInfo.node_.removePermission(name);
        permissionsInfo.node_.getSession().save();
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  } 
  
  public class PermissionBean {    
    private String usersOrGroups;
    private boolean read;
    private boolean addNode;
    private boolean setProperty;
    private boolean remove;    
    
    public String getUsersOrGroups() {
      return usersOrGroups;
    }
    public void setUsersOrGroups(String usersOrGroups) {
      this.usersOrGroups = usersOrGroups;
    }
    
    public boolean isAddNode() {
      return addNode;
    }
    public void setAddNode(boolean addNode) {
      this.addNode = addNode;
    }
    public boolean isRead() {
      return read;
    }
    public void setRead(boolean read) {
      this.read = read;
    }
    public boolean isRemove() {
      return remove;
    }
    public void setRemove(boolean remove) {
      this.remove = remove;
    }
    public boolean isSetProperty() {
      return setProperty;
    }
    public void setSetProperty(boolean setProperty) {
      this.setProperty = setProperty;
    }
  }
  
}