package org.exoplatform.portlets.content.jcr.component;

import java.io.InputStream;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Map;

import javax.jcr.Node;

import org.exoplatform.commons.utils.IOUtil;
import org.exoplatform.commons.utils.MimeTypeResolver;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.core.component.UISimpleMulltipartForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UIUploadInputStream;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.jcr.component.UIViewController.CancelActionListener;
import org.exoplatform.services.cms.CmsService;
import org.exoplatform.services.cms.JcrInputProperty;

public class UIUploadForm extends UISimpleMulltipartForm implements
    ChangeNodeListener {
  private boolean isEdit_ = false ;
  private Node node_;
  private UIStringInput name_;
  static Parameter[] CANCEL_PARAMS = { new Parameter(ACTION , CANCEL_ACTION)} ;
  
  final public static String NAME = "name".intern() ;
  final public static String MIX_VERSION = "mix:versionable".intern() ; 
  final public static String NT_FILE = "nt:file".intern() ;
  final public static String NT_RESOURCE = "nt:resource".intern() ;
  final public static String JCR_CONTENT = "jcr:content".intern() ;
  final public static String JCR_DATA = "jcr:data".intern() ;
  final public static String JCR_MIMETYPE = "jcr:mimeType".intern() ;
  final public static String JCR_LASTMODIFIED = "jcr:lastModified".intern() ;
  
  public UIUploadForm() {
    super("uploadForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");
    name_ = new UIStringInput(NAME, "");
    add(name_);    
    add(new UIUploadInputStream("upload"));    
    addActionListener(SaveActionListener.class, SAVE_ACTION);
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
  }
  
  public Parameter[] getCancelParams() { return CANCEL_PARAMS ; }
  public void onChange(UIJCRExplorer explorer, Node node) {
    setRendered(false);
    node_ = node;
  }
  
  public Node getNode() { return node_ ; }
  
  public void update(String name) { 
    if(name != null){
      name_.setValue(name).setEditable(false) ;
      isEdit_ = true ;
    } else{
      name_.setValue("").setEditable(true) ;
    } 
  }

  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIUploadForm uiForm = (UIUploadForm) event.getSource();
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uiForm.getAncestorOfType(UIJCRExplorer.class);
      UIUploadInputStream input = (UIUploadInputStream) uiForm.getUIInput("upload");
      InputStream jcrDataStream = input.getInputStream();                  
      String fileName = input.getFileName().trim();
      String name = uiForm.getUIInput(NAME).getValue().trim() ;
      if ("".equals(name)) {
    	  name = fileName;
      }
      
      MimeTypeResolver resolver = new MimeTypeResolver();
      String mimeType = resolver.getMimeType(fileName);      
      Node selectedNode = uiExplorer.getSelectNode() ;
      
      if(uiForm.isEdit_ && selectedNode.getPrimaryNodeType().getName().endsWith(NT_FILE)){
        selectedNode = uiExplorer.getSelectNode().getParent();
      }
      Information iprovider = findInformationProvider(uiForm);
      boolean isExist = selectedNode.hasNode(name) ;
      
      try {
        selectedNode.getSession().checkPermission(
            selectedNode.getPath(), "add_node,set_property");
        if(!isExist){          
          Map<String,JcrInputProperty> inputProperties = new HashMap<String,JcrInputProperty>() ;
          
          JcrInputProperty nodeInput = new JcrInputProperty() ;
          nodeInput.setJcrPath("/node") ;
          nodeInput.setValue(name) ;
          nodeInput.setMixintype("mix:i18n,mix:votable,mix:commentable") ;
          nodeInput.setType(JcrInputProperty.NODE) ;
          inputProperties.put("/node",nodeInput) ;
          
          JcrInputProperty jcrContent = new JcrInputProperty() ;
          jcrContent.setJcrPath("/node/jcr:content") ;
          jcrContent.setValue("") ;
          jcrContent.setMixintype("dc:elementSet") ;
          jcrContent.setNodetype(NT_RESOURCE) ;
          jcrContent.setType(JcrInputProperty.NODE) ;
          inputProperties.put("/node/jcr:content",jcrContent) ;
          
          JcrInputProperty jcrData = new JcrInputProperty() ;
          jcrData.setJcrPath("/node/jcr:content/jcr:data") ;
          byte[] dataInput = IOUtil.getStreamContentAsBytes(jcrDataStream) ;
          jcrData.setValue(dataInput) ;          
          inputProperties.put("/node/jcr:content/jcr:data",jcrData) ; 
          
          JcrInputProperty jcrMimeType = new JcrInputProperty() ;
          jcrMimeType.setJcrPath("/node/jcr:content/jcr:mimeType") ;
          jcrMimeType.setValue(mimeType) ;          
          inputProperties.put("/node/jcr:content/jcr:mimeType",jcrMimeType) ;
          
          JcrInputProperty jcrLastModified = new JcrInputProperty() ;
          jcrLastModified.setJcrPath("/node/jcr:content/jcr:lastModified") ;
          jcrLastModified.setValue(new GregorianCalendar()) ;
          inputProperties.put("/node/jcr:content/jcr:lastModified",jcrLastModified) ;
          
          JcrInputProperty jcrEncoding = new JcrInputProperty() ;
          jcrEncoding.setJcrPath("/node/jcr:content/jcr:encoding") ;
          jcrEncoding.setValue("UTF-8") ;
          inputProperties.put("/node/jcr:content/jcr:encoding",jcrEncoding) ;          
          CmsService cmsService = (CmsService)PortalContainer.getComponent(CmsService.class) ;
          String path = cmsService.storeNode(NT_FILE,selectedNode,inputProperties,true) ;
          selectedNode.save() ;
          selectedNode.getSession().save() ;
          String newNodePath = path.substring(path.indexOf(":") + 1 ) ;
          Node newNode = (Node)uiExplorer.getSession().getItem(newNodePath) ;                             
          uiExplorer.broadcastOnChange(newNode.getParent()) ;
        }else {
          Node node = selectedNode.getNode(name) ;        
          if(!node.isNodeType(MIX_VERSION)) {
            node.addMixin(MIX_VERSION) ;            
            node.save() ;            
            node.checkin() ;
            node.checkout() ;
          }         
         Node contentNode = node.getNode(JCR_CONTENT);
         contentNode.setProperty(JCR_DATA, jcrDataStream);
         contentNode.setProperty(JCR_MIMETYPE, mimeType);
         contentNode.setProperty(JCR_LASTMODIFIED, new GregorianCalendar());
         node.save() ;       
         node.checkin() ;
         node.checkout() ;
        }
        uiExplorer.getSession().save() ;
        uiExplorer.broadcastOnChange(selectedNode);
      } catch (Exception e) {
        //e.printStackTrace() ;
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }
}
