/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.jcr.component;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Property;
import javax.jcr.PropertyIterator;
import javax.jcr.Session;
import javax.jcr.Value;
import javax.jcr.Workspace;
import javax.jcr.nodetype.NodeType;
import javax.portlet.PortletPreferences;
import javax.portlet.PortletRequest;

import org.apache.commons.lang.StringUtils;
import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.cms.component.JcrBrowser;
import org.exoplatform.faces.cms.component.JcrCategoryBrowser;
import org.exoplatform.faces.cms.component.JcrRelationBrowser;
import org.exoplatform.faces.core.component.UICommandNode;
import org.exoplatform.faces.core.component.UINode;
import org.exoplatform.faces.core.component.UIToolbar;
import org.exoplatform.faces.core.component.model.Button;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.faces.user.component.UIExoPermissionBrowser;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.actions.component.UIActionFormController;
import org.exoplatform.portlets.content.actions.component.UIActionList;
import org.exoplatform.portlets.content.categories.component.UICategoriesList;
import org.exoplatform.portlets.content.categories.component.UIReferencesList;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.jcr.component.model.ClipboardCommand;
import org.exoplatform.portlets.content.jcr.component.model.Preference;
import org.exoplatform.portlets.content.relations.component.UIRelationsList;
import org.exoplatform.portlets.content.search.component.UIAdvanceConstraints;
import org.exoplatform.portlets.content.search.component.UISearch;
import org.exoplatform.portlets.content.search.component.UISelectMetadataForm;
import org.exoplatform.portlets.content.signatures.component.UIAddSignature;
import org.exoplatform.portlets.content.versions.component.UIDiff;
import org.exoplatform.portlets.content.versions.component.UILabelForm;
import org.exoplatform.portlets.content.versions.component.UIRemoveLabelForm;
import org.exoplatform.portlets.content.versions.component.UIVersionInfo;
import org.exoplatform.portlets.content.versions.component.UIViewVersion;
import org.exoplatform.portlets.content.watch.component.UIWatchForm;
import org.exoplatform.services.cms.actions.ActionServiceContainer;
import org.exoplatform.services.cms.relations.RelationsService;
import org.exoplatform.services.cms.templates.TemplateService;
import org.exoplatform.services.cms.views.ManageViewService;
import org.exoplatform.services.jcr.RepositoryService;

/**
 * @email: tuan08@users.sourceforge.net
 * @version: $Id: UIExplorer.java,v 1.2 2004/07/31 15:07:15 tuan08 Exp $
 */
public class UIViewController extends UINode implements  ChangeNodeListener {
  final static public String SEARCH = "search".intern();	
  final static public String VIEW_PERMISSIONS = "viewPermissions".intern();
  final static public String VIEW_PROPERTIES = "viewProperties".intern();
  final static public String VIEW_META = "viewMetadatas".intern();
  final static public String VIEW_NODETYPE = "viewNodeType".intern();
  final static public String IMPORT_NODE = "importNode".intern();
  final static public String EXPORT_NODE = "exportNode".intern();
  final static public String ADD_DOCUMENT = "addDocument".intern();
  final static public String UPLOAD = "upload".intern();
  final static public String ADD_FOLDER = "addFolder".intern();  
  public static final String PASTE_ACTION = "paste".intern();
  public static final String MANAGE_ACTIONS_ACTION = "manageActions".intern();
  public static final String MANAGE_CATEGORIES_ACTION = "manageCategories".intern();
  public static final String MANAGE_RELATIONS_ACTION = "manageRelations".intern();
  public static final String VIEW_REFERENCES_ACTION = "viewReferences".intern();
  public static final String MANAGE_VERSIONS_ACTION = "manageVersions".intern();
  public static final String CREATE = "createTicket".intern();
  public static final String VIEW_RELATION_ACTION = "viewRelations".intern() ;
  public static final String EDIT_DOCUMENT_ACTION="editDocument".intern() ;
  public static final String SHOW_JCR_STRUCTURE_ACTION = "showJCRStructure".intern() ;
  public static final String ADD_SIGNATURE_ACTION = "addSignature".intern();
  public static final String WATCH_DOCUMENT_ACTION = "watchDocument".intern() ;
  public static final String TAGGING_DOCUMENT_ACTION = "taggingDocument".intern() ;
  public static final String MULTI_LANGUAGE_ACTION = "multiLanguage".intern() ;
  public static final String VOTING_DOCUMENT_ACTION = "voting".intern() ;
  public static final String COMMENTS_ACTION = "comment".intern() ;


  public static final String[] PASTE_TYPES = { "nt:unstructured", "nt:folder" };

  public static Parameter[] SEARCH_PARAMS = { new Parameter(ACTION, SEARCH) };
  public static Parameter[] VIEW_PERMISSIONS_PARAMS = { new Parameter(ACTION, VIEW_PERMISSIONS) };
  public static Parameter[] VIEW_PROPS_PARAMS = { new Parameter(ACTION, VIEW_PROPERTIES) };
  public static Parameter[] VIEW_META_PARAMS = { new Parameter(ACTION, VIEW_META) };
  public static Parameter[] VIEW_NODETYPE_PARAMS = { new Parameter(ACTION,VIEW_NODETYPE) };
  public static Parameter[] ADD_DOCUMENT_PARAMS = { new Parameter(ACTION, ADD_DOCUMENT) };
  public static Parameter[] UPLOAD_PARAMS = { new Parameter(ACTION, UPLOAD) };
  public static Parameter[] IMPORT_NODE_PARAMS = { new Parameter(ACTION, IMPORT_NODE) };
  public static Parameter[] EXPORT_NODE_PARAMS = { new Parameter(ACTION, EXPORT_NODE) };
  public static Parameter[] ADD_FOLDER_PARAMS = { new Parameter(ACTION, ADD_FOLDER) };  
  public static Parameter[] PASTE_NODE_PARAMS = { new Parameter(ACTION, PASTE_ACTION) };
  public static Parameter[] MANAGE_ACTIONS_PARAMS = { new Parameter(ACTION, MANAGE_ACTIONS_ACTION) };
  public static Parameter[] MANAGE_CATEGORIES_PARAMS = { new Parameter(ACTION, MANAGE_CATEGORIES_ACTION) };
  public static Parameter[] MANAGE_RELATIONS_PARAMS = { new Parameter(ACTION, MANAGE_RELATIONS_ACTION) };
  public static Parameter[] VIEW_REFERENCES_PARAMS = { new Parameter(ACTION, VIEW_REFERENCES_ACTION) };
  public static Parameter[] MANAGE_VERSIONS_PARAMS = { new Parameter(ACTION, MANAGE_VERSIONS_ACTION) };
  public static Parameter[] CREATE_TICKET = { new Parameter(ACTION,CREATE) };
  public static Parameter[] VIEW_RELATION_PARAMS = { new Parameter(ACTION, VIEW_RELATION_ACTION) };
  public static Parameter[] EDIT_DOCUMENT_PARAMS = {new Parameter(ACTION,EDIT_DOCUMENT_ACTION)} ;
  public static Parameter[] SHOW_JCR_STRUCTURE_PARAMS = { new Parameter(ACTION,SHOW_JCR_STRUCTURE_ACTION)} ;
  public static Parameter[] ADD_SIGNATURE_PARAMS = { new Parameter(ACTION, ADD_SIGNATURE_ACTION) };
  public static Parameter[] WATCH_DOCUMENT_PARAMS = {new Parameter(ACTION, WATCH_DOCUMENT_ACTION) } ;
  public static Parameter[] TAGGING_DOCUMENT_PARAMS = {new Parameter(ACTION, TAGGING_DOCUMENT_ACTION) } ;
  public static Parameter[] MULTI_LANGUAGE_PARAMS = {new Parameter(ACTION, MULTI_LANGUAGE_ACTION) } ;
  public static Parameter[] VOTING_DOCUMENT_PARAMS = {new Parameter(ACTION,VOTING_DOCUMENT_ACTION) } ;
  public static Parameter[] COMMENTS_PARAMS = { new Parameter(ACTION,COMMENTS_ACTION) } ;

  private UIToolbar uitoolbar;
  private Node node_;
  private ManageViewService vservice_ ;
  String template_ ;
  private HashMap<String, Parameter[]> parameters_ = new HashMap<String, Parameter[]>() ;  

  public UIViewController(ManageViewService vservice) throws Exception {
    setRendererType("VelocityNodeMenuRenderer") ;
    setTemplatePath("org/exoplatform/faces/core/renderer/html/UIFirstTabLevel.vm") ;
    vservice_ = vservice ;
    parameters_.put(SEARCH, SEARCH_PARAMS) ;
    parameters_.put(VIEW_PERMISSIONS, VIEW_PERMISSIONS_PARAMS) ;
    parameters_.put(VIEW_PROPERTIES, VIEW_PROPS_PARAMS) ;
    parameters_.put(VIEW_META, VIEW_META_PARAMS) ;
    parameters_.put(VIEW_NODETYPE, VIEW_NODETYPE_PARAMS) ;
    parameters_.put(IMPORT_NODE, IMPORT_NODE_PARAMS) ;
    parameters_.put(EXPORT_NODE, EXPORT_NODE_PARAMS) ;
    parameters_.put(ADD_DOCUMENT, ADD_DOCUMENT_PARAMS) ;
    parameters_.put(UPLOAD, UPLOAD_PARAMS) ;
    parameters_.put(ADD_FOLDER, ADD_FOLDER_PARAMS) ;
    parameters_.put(PASTE_ACTION, PASTE_NODE_PARAMS) ;
    parameters_.put(MANAGE_ACTIONS_ACTION, MANAGE_ACTIONS_PARAMS) ;
    parameters_.put(MANAGE_CATEGORIES_ACTION, MANAGE_CATEGORIES_PARAMS) ;
    parameters_.put(MANAGE_RELATIONS_ACTION, MANAGE_RELATIONS_PARAMS) ;
    parameters_.put(VIEW_REFERENCES_ACTION, VIEW_REFERENCES_PARAMS) ;
    parameters_.put(MANAGE_VERSIONS_ACTION, MANAGE_VERSIONS_PARAMS) ;
    parameters_.put(CREATE, CREATE_TICKET) ;
    parameters_.put(VIEW_RELATION_ACTION,VIEW_RELATION_PARAMS) ;
    parameters_.put(EDIT_DOCUMENT_ACTION,EDIT_DOCUMENT_PARAMS) ;
    parameters_.put(SHOW_JCR_STRUCTURE_ACTION,SHOW_JCR_STRUCTURE_PARAMS) ;
    parameters_.put(ADD_SIGNATURE_ACTION, ADD_SIGNATURE_PARAMS);
    parameters_.put(WATCH_DOCUMENT_ACTION,WATCH_DOCUMENT_PARAMS) ;
    parameters_.put(TAGGING_DOCUMENT_ACTION,TAGGING_DOCUMENT_PARAMS) ;
    parameters_.put(MULTI_LANGUAGE_ACTION,MULTI_LANGUAGE_PARAMS) ;
    parameters_.put(VOTING_DOCUMENT_ACTION,VOTING_DOCUMENT_PARAMS) ;
    parameters_.put(COMMENTS_ACTION,COMMENTS_PARAMS) ;
    ExternalContext eContext = FacesContext.getCurrentInstance().getExternalContext();
    PortletRequest request = (PortletRequest) eContext.getRequest();    
    PortletPreferences prefs_ = request.getPreferences();
    String views = prefs_.getValue("views", "") ;
    if(views != null && views.length() > 0 ){
      String[] array = views.split(",") ;
      renderTabs(array[0].trim()) ;
    }
  }

  public void renderTabs(String viewName) throws Exception{
    if(getChildCount() > 0)
      getChildren().clear() ;
    boolean isTabRendered = false ;    
    Node viewHome = vservice_.getViewHome() ;
    Node view = viewHome.getNode(viewName) ;
    NodeIterator tabs = view.getNodes() ;
    while(tabs.hasNext()){
      Node tabNode = tabs.nextNode() ;
      String tabName = tabNode.getName() ;
      UICommandNode tab = (UICommandNode)addChild(UICommandNode.class) ;
      tabName = tabName.replaceAll(" ", "_") ;
      tab.setId(tabName) ;
      uitoolbar = new UIToolbar("UIToolbar");
      String buttons = tabNode.getProperty("exo:buttons").getString() ;
      String[] buttonsInTab = StringUtils.split(buttons, ";") ;
      for(int j = 0 ; j < buttonsInTab.length ; j ++ ) {
        String buttonName = buttonsInTab[j].trim() ;
        if( buttonName != null && buttonName.length() > 0)
          uitoolbar.addLeftButton(new Button("#{UIViewController.button." + buttonName + "}", 
              parameters_.get(buttonName)));
      }
      tab.setRendererType("ChildrenRenderer") ;
      if(!isTabRendered){
        tab.setRendered(true) ;
        isTabRendered = true ;
      }else {
        tab.setRendered(false) ;
      }
      tab.getChildren().add(uitoolbar) ;
      addComponentAndListener(tab) ;
    }
    template_ = view.getProperty("exo:template").getString() ;		
  } 

  private void addComponentAndListener(UICommandNode tab) throws Exception{
    tab.addChild(UIDocumentInfo.class);    
    tab.addChild(UIActionList.class).setRendered(false);
    tab.addChild(UICategoriesList.class).setRendered(false);
    tab.addChild(UIRelationsList.class).setRendered(false);
    tab.addChild(UIExport.class).setRendered(false);
    tab.addChild(UIImport.class).setRendered(false);
    tab.addChild(UIPermissionForm.class).setRendered(false);
    tab.addChild(UIExoPermissionBrowser.class).setRendered(false);
    tab.addChild(UIPropertiesInfo.class).setRendered(false);
    tab.addChild(UIMetadataForm.class).setRendered(false);
    tab.addChild(UIPropertiesManager.class).setRendered(false);
    tab.addChild(UIPropertyForm.class).setRendered(false);
    tab.addChild(UIActionFormController.class).setRendered(false);
    tab.addChild(JcrBrowser.class).setRendered(false);
    tab.addChild(JcrRelationBrowser.class).setRendered(false);
    tab.addChild(JcrCategoryBrowser.class).setRendered(false);


    tab.addChild(UIVersionInfo.class).setRendered(false);
    tab.addChild(UIViewVersion.class).setRendered(false) ;
    tab.addChild(UILabelForm.class).setRendered(false);        
    tab.addChild(UIDiff.class).setRendered(false);
    tab.addChild(UIRemoveLabelForm.class).setRendered(false);
    tab.addChild(UIRenameForm.class).setRendered(false) ;
    tab.addChild(UIReferencesList.class).setRendered(false);
    tab.addChild(UINodeTypeInfo.class).setRendered(false);
    tab.addChild(UIPermissionsInfo.class).setRendered(false);
    tab.addChild(UISetPropertyForm.class).setRendered(false); 
    tab.addChild(UIDocumentFormController.class).setRendered(false);
    tab.addChild(UIFolderForm.class).setRendered(false);
    tab.addChild(UIUploadForm.class).setRendered(false);
    tab.addChild(UISearch.class).setRendered(false);
    tab.addChild(UISelectMetadataForm.class).setRendered(false);
    tab.addChild(UIAdvanceConstraints.class).setRendered(false);
    tab.addChild(UITicketForm.class).setRendered(false);
    tab.addChild(UIAddSignature.class).setRendered(false);
    tab.addChild(UIWatchForm.class).setRendered(false) ;
    tab.addChild(UITaggingController.class).setRendered(false) ;
    tab.addChild(UIMultiLanguageForm.class).setRendered(false) ;
    tab.addChild(UIAddLanguageForm.class).setRendered(false) ;
    tab.addChild(UIVotingForm.class).setRendered(false) ;
    tab.addChild(UICommentsForm.class).setRendered(false) ;

    tab.addActionListener(ManageVersionsActionListener.class, MANAGE_VERSIONS_ACTION);
    tab.addActionListener(ManageActionsActionListener.class, MANAGE_ACTIONS_ACTION);
    tab.addActionListener(ManageCategoriesActionListener.class, MANAGE_CATEGORIES_ACTION); 
    tab.addActionListener(ManageRelationsActionListener.class, MANAGE_RELATIONS_ACTION);
    tab.addActionListener(ExportNodeActionListener.class, EXPORT_NODE);
    tab.addActionListener(ImportNodeActionListener.class, IMPORT_NODE);
    tab.addActionListener(PasteActionListener.class, PASTE_ACTION);
    tab.addActionListener(AddDocumentActionListener.class, ADD_DOCUMENT) ;    
    tab.addActionListener(AddFolderActionListener.class, ADD_FOLDER);
    tab.addActionListener(UploadActionListener.class, UPLOAD);
    tab.addActionListener(SearchActionListener.class, SEARCH);
    tab.addActionListener(CreateTicketActionListener.class, CREATE); 
    tab.addActionListener(ViewReferencesActionListener.class, VIEW_REFERENCES_ACTION);
    tab.addActionListener(ViewNodeTypeActionListener.class, VIEW_NODETYPE);
    tab.addActionListener(ViewPermissionsActionListener.class, VIEW_PERMISSIONS);
    tab.addActionListener(ViewPropertiesActionListener.class, VIEW_PROPERTIES);
    tab.addActionListener(ViewMetadataActionListener.class, VIEW_META);
    tab.addActionListener(ViewRelationInfoActionListener.class,VIEW_RELATION_ACTION) ;
    tab.addActionListener(EditDocumentActionListener.class,EDIT_DOCUMENT_ACTION) ;
    tab.addActionListener(ShowJCRStructureActionListener.class,SHOW_JCR_STRUCTURE_ACTION) ;
    tab.addActionListener(AddSignatureActionListener.class, ADD_SIGNATURE_ACTION);
    tab.addActionListener(WatchDocumentActionListener.class,WATCH_DOCUMENT_ACTION) ;
    tab.addActionListener(TaggingDocumentActionListener.class,TAGGING_DOCUMENT_ACTION) ;
    tab.addActionListener(MultiLanguageActionListener.class,MULTI_LANGUAGE_ACTION) ;
    tab.addActionListener(VotingDocumentActionListener.class,VOTING_DOCUMENT_ACTION) ;
    tab.addActionListener(CommentsActionListener.class,COMMENTS_ACTION) ;
  }

  public void onChange(UIJCRExplorer uiExplorer, Node node) throws Exception {
    node_ = node;   
  }

  public String getTemplate() { return template_ ; }

  public boolean isPastePossible(Node node) throws Exception {
    boolean isPaste = false;
    UIJCRExplorer uiExplorer = (UIJCRExplorer) getAncestorOfType(UIJCRExplorer.class);
    if (uiExplorer.getAllClipBoard().isEmpty()) return false;
    String nodeType = node.getPrimaryNodeType().getName();
    for (int i = 0; i < PASTE_TYPES.length; i++) {
      String pasteType = PASTE_TYPES[i];
      if (pasteType.equals(nodeType)) return true;
    }
    return isPaste;
  }

  static public class AddFolderActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      ((UIFolderForm)uicomp.getChildComponentOfType(UIFolderForm.class)).reset() ;
      uicomp.setRenderedComponent(UIFolderForm.class);
    }
  }

  static public class AddDocumentActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      UIDocumentFormController uiNewDocumentController = 
        (UIDocumentFormController) uicomp.getChildComponentOfType(UIDocumentFormController.class);
      uiNewDocumentController.showNewDocumentForm();
      uicomp.setRenderedComponent(UIDocumentFormController.class);
    }
  }

  static public class UploadActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      UIUploadForm uploadForm = 
        (UIUploadForm) uicomp.getChildComponentOfType(UIUploadForm.class);
      uploadForm.update(null) ;
      uicomp.setRenderedComponent(UIUploadForm.class);
    }
  }

  static public class PasteActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      ResourceBundle res = getApplicationResourceBundle() ;
      UIViewController uicomp = (UIViewController) event.getComponent().getParent();
      Information iprovider = findInformationProvider(uicomp);
      UIJCRExplorer uiExplorer = 
        (UIJCRExplorer) uicomp.getAncestorOfType(UIJCRExplorer.class);
      Node currentNode = uiExplorer.getSelectNode();
      ClipboardCommand lastClipboard = null ;
      try {
        lastClipboard = uiExplorer.getAllClipBoard().getLast() ;
      }catch (Exception e) {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN ,
            res.getString("UIViewController.msg.no-node-selected")));
        return ;
      }            
      String type = lastClipboard .getType();
      String srcPath = lastClipboard .getSrcPath();
      String srcWorkspace = null ;
      if(srcPath.indexOf(";") > -1 ){
        String[] array = srcPath.split(";") ;
        srcWorkspace = array[0].trim() ;
        srcPath = array[1].trim() ;
      }
      String destPath = null ; 
      if(currentNode.getPath().equals("/")) {
        destPath = "/" + srcPath.substring(srcPath.lastIndexOf("/") + 1);
      }else {
        destPath = currentNode.getPath() + srcPath.substring(srcPath.lastIndexOf("/")); 
      }			      
      try {
        Session session = uiExplorer.getSession() ;
        Workspace workspace = session.getWorkspace();
        if (ClipboardCommand.COPY.equals(type)) {
          if(srcWorkspace != null)
            workspace.copy(srcWorkspace, srcPath, destPath);
          else  
            workspace.copy(srcPath, destPath);
          Node destNode = (Node) session.getItem(destPath) ;
          removeReferences(destNode, session) ;
        } else {
          if(srcWorkspace != null){
            workspace.copy(srcWorkspace, srcPath, destPath);
            Node destNode = (Node) session.getItem(destPath) ;
            removeReferences(destNode, session) ;
            RepositoryService repositoryService = (RepositoryService)
            PortalContainer.getInstance().getComponentInstanceOfType(RepositoryService.class) ;
            Session srcSession = repositoryService.getRepository().login(srcWorkspace) ;
            srcSession.getItem(srcPath).remove() ;
            srcSession.save() ;
          }else{
            RelationsService relationsService = (RelationsService)
            PortalContainer.getInstance().getComponentInstanceOfType(RelationsService.class) ;
            List<Node> refList = new ArrayList<Node>() ;
            boolean isReference = false ;
            PropertyIterator references = null;
            Node srcNode = (Node)uiExplorer.getSession().getItem(srcPath) ;
            try{
              references = srcNode.getReferences() ;
              isReference = true ;
            }catch(Exception e){
              isReference = false ;
            }  
            if(isReference && references != null){
              if(references.getSize() > 0 ){
                while(references.hasNext()){
                  Property pro = references.nextProperty() ;
                  Node refNode = pro.getParent() ;
                  relationsService.removeRelation(refNode, srcPath) ;
                  refNode.save() ;
                  refList.add(refNode) ;
                }
              }
            }            
            session.move(srcPath, destPath);
            session.save() ;
            for(int i = 0; i < refList.size(); i ++){
              Node addRef = refList.get(i) ;
              relationsService.addRelation(addRef, destPath) ;
              addRef.save() ;
            }
          }          
          uiExplorer.getAllClipBoard().removeLast() ;
        }

        ActionServiceContainer actionservice = (ActionServiceContainer)
        PortalContainer.getInstance().getComponentInstanceOfType(ActionServiceContainer.class) ;
        actionservice.initiateObservation(currentNode) ;
        Object[] args = { srcPath, destPath };
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_INFO,
            res.getString("UIViewController.msg.node-pasted"), args));
        uiExplorer.broadcastOnChange(currentNode);
      } catch (Exception e) {				
        e.printStackTrace() ;
        JCRExceptionManager.process(iprovider, e);
      }
    }
    private void removeReferences(Node destNode, Session session) throws Exception{
      NodeType[] mixinTypes = destNode.getMixinNodeTypes() ;
      for(int i = 0; i < mixinTypes.length; i ++) {
        if(mixinTypes[i].getName().equals("exo:categorized") 
            && destNode.hasProperty("exo:category")) {
          Node valueNode = null ;
          Value value2add = session.getValueFactory().createValue(valueNode);
          destNode.setProperty("exo:category", new Value[] {value2add}) ;            
        }            
      }
      destNode.save() ;
    }
  }

  static public class ViewPermissionsActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      uicomp.setRenderedComponent(UIPermissionsInfo.class);
    }
  }

  static public class ViewPropertiesActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();      
      uicomp.setRenderedComponent(UIPropertiesInfo.class);
    }
  }

  static public class ViewMetadataActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      UIMetadataForm metadata = 
        (UIMetadataForm)uicomp.getChildComponentOfType(UIMetadataForm.class) ;
      metadata.update(true) ;
      uicomp.setRenderedComponent(UIMetadataForm.class);
    }
  }

  static public class ViewNodeTypeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uiForm = (UICommandNode)event.getComponent() ;
      UIViewController uiViewControl = (UIViewController) uiForm.getParent();

      UINodeTypeInfo uiInfo = 
        (UINodeTypeInfo) uiForm.getChildComponentOfType(UINodeTypeInfo.class);
      Information iprovider = findInformationProvider(uiViewControl); 
      try {
        NodeType nodetype = uiViewControl.node_.getPrimaryNodeType();
        Collection types = new ArrayList();
        types.add(nodetype);
        NodeType[] mixins = uiViewControl.node_.getMixinNodeTypes();
        if (mixins != null) {
          List list = Arrays.asList(mixins);
          types.addAll(list);
        }
        uiInfo.setNodeTypes(types);
        uiForm.setRenderedComponent(UINodeTypeInfo.class);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }

  static public class ImportNodeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      uicomp.setRenderedComponent(UIImport.class);
    }
  }

  static public class ExportNodeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      uicomp.setRenderedComponent(UIExport.class);
    }
  }

  static public class SearchActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      uicomp.setRenderedComponent(UISearch.class);
    }
  }

  static public class ManageActionsActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      uicomp.setRenderedComponent(UIActionList.class);
    }
  }

  static public class ManageCategoriesActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      uicomp.setRenderedComponent(UICategoriesList.class);
    }
  }

  static public class ManageRelationsActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      uicomp.setRenderedComponent(UIRelationsList.class);
    }
  }

  static public class ViewReferencesActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      uicomp.setRenderedComponent(UIReferencesList.class);
    }
  }  

  static public class ManageVersionsActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      uicomp.setRenderedComponent(UIVersionInfo.class);
    }
  }

  static public class CancelActionListener extends ExoActionListener  {
    public void execute(ExoActionEvent event) throws Exception {
      UINode uiForm = (UINode) event.getComponent().getParent().getParent();      
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uiForm.getAncestorOfType(UIJCRExplorer.class);
      Node node = uiExplorer.getSelectNode();
      uiExplorer.broadcastOnChange(node);      
    }
  }

  static public class CreateTicketActionListener extends ExoActionListener{

    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      uicomp.setRenderedComponent(UITicketForm.class);
    }		
  }  

  static public class ViewRelationInfoActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {			
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      UIDocumentInfo uiInfo = (UIDocumentInfo)uicomp.getChildComponentOfType(UIDocumentInfo.class) ;
      ResourceBundle res = getApplicationResourceBundle() ;
      Information iprovider = findInformationProvider(uicomp) ;			
      if(!uiInfo.isNodeTypeSupported()) {
        iprovider.addMessage( new ExoFacesMessage(FacesMessage.SEVERITY_WARN ,
            res.getString("UIViewController.msg.unsupported-action")));
        return;
      }
      Node currentNode = uiInfo.getNode() ;
      if(!currentNode.hasProperty("exo:relation")) {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN ,
            res.getString("UIViewController.msg.not-relationable")));
        return;
      }
      uiInfo.showRelation = true ;			     
    }    
  }

  static public class EditDocumentActionListener extends ExoActionListener {    
    public void execute(ExoActionEvent event) throws Exception {      
      UICommandNode uicomp = (UICommandNode) event.getComponent();      
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uicomp.getAncestorOfType(UIJCRExplorer.class);
      Node selectedNode = uiExplorer.getSelectNode() ;
      Information iprovider = findInformationProvider(uicomp);
      ResourceBundle res = getApplicationResourceBundle() ;
      TemplateService templateService = 
        (TemplateService)PortalContainer.getComponent(TemplateService.class) ;
      try {
        selectedNode.getSession().checkPermission(selectedNode.getPath(), "add_node,set_property");
      }catch(Exception e) {
        iprovider.addMessage(
            new ExoFacesMessage(res.getString("UIViewController.msg.no-permissions-edit"))) ;
        return ;
      }
      try {                        
        String nodeType = selectedNode.getPrimaryNodeType().getName() ;
        if(!templateService.isManagedNodeType(nodeType)) {
          Object[] args = {selectedNode.getPath(),nodeType } ;
          iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN ,
              res.getString("UIViewController.msg.unable-edit"),args)) ;
          return ;
        }
//      if ("nt:file".equals(nodeType)) {
//      Node contentNode = selectedNode.getNode("jcr:content");
//      String contentNodeType = contentNode.getPrimaryNodeType().getName();
//      String mimeType = contentNode.getProperty("jcr:mimeType").getString();
//      if ("nt:resource".equals(contentNodeType) && !mimeType.startsWith("text")) {
//      UIUploadForm form = (UIUploadForm) uicomp.getChildComponentOfType(UIUploadForm.class);
//      form.update(selectedNode.getName()) ;
//      uicomp.setRenderedComponent(UIUploadForm.class);
//      return;
//      }
//      }

        UIDocumentFormController uiNewNodeController = 
          (UIDocumentFormController) uicomp.getChildComponentOfType(UIDocumentFormController.class);
        Node preNode = uiExplorer.rewind() ;
        if(selectedNode.getParent().equals(preNode.getPath()+ "/" + selectedNode.getName()))
          uiNewNodeController.showEditDocumentForm(selectedNode,false);
        else
          uiNewNodeController.showEditDocumentForm(selectedNode,true);
        uiExplorer.record(preNode) ;
        uicomp.setRenderedComponent(UIDocumentFormController.class);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }

    }    
  }

  static public class ShowJCRStructureActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {    
      UICommandNode uicomp = (UICommandNode) event.getComponent();      
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uicomp.getAncestorOfType(UIJCRExplorer.class);
      Preference pref = uiExplorer.getPreference() ;
      pref.setJcrEnable(!pref.isJcrEnable()) ;      
    }        
  }

  public static class AddSignatureActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uicomp.getAncestorOfType(UIJCRExplorer.class);
      Node curNode = uiExplorer.getSelectNode() ;
      if(curNode.isNodeType("exo:xmldsig") && curNode.hasProperty("exo:signature")){
        Information iprovider = findInformationProvider(uicomp);
        ResourceBundle res = getApplicationResourceBundle() ; 
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN ,
            res.getString("UIViewController.msg.signature-action-already")));
        return ;
      }        
      uicomp.setRenderedComponent(UIAddSignature.class);	    
    }
  }

  public static class WatchDocumentActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {      
      UICommandNode uicomp = (UICommandNode) event.getComponent();            
      TemplateService templateService = 
        (TemplateService)PortalContainer.getComponent(TemplateService.class) ;            
      Information iprovider = findInformationProvider(uicomp);
      ResourceBundle res = getApplicationResourceBundle() ;
      UIJCRExplorer uiExplorer = 
        (UIJCRExplorer) uicomp.getAncestorOfType(UIJCRExplorer.class);
      Node currentNode = uiExplorer.getSelectNode();      
      if(templateService.isManagedNodeType(currentNode.getPrimaryNodeType().getName())) {        
        uicomp.setRenderedComponent(UIWatchForm.class) ;
      } else {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN ,
            res.getString("UIViewController.msg.unsupported-watch"))) ;
        return ;
      }
    }    
  }

  static public class TaggingDocumentActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();            
      TemplateService templateService = 
        (TemplateService)PortalContainer.getComponent(TemplateService.class) ;            
      Information iprovider = findInformationProvider(uicomp);
      ResourceBundle res = getApplicationResourceBundle() ;
      UIJCRExplorer uiExplorer = 
        (UIJCRExplorer) uicomp.getAncestorOfType(UIJCRExplorer.class);
      Node currentNode = uiExplorer.getSelectNode();
      if(templateService.isManagedNodeType(currentNode.getPrimaryNodeType().getName())) {
        UITaggingController controller = 
          (UITaggingController)uicomp.getChildComponentOfType(UITaggingController.class) ;
        controller.setDatas(currentNode) ;
        uicomp.setRenderedComponent(UITaggingController.class) ;
      } else {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN ,
            res.getString("UIViewController.msg.unsupported-tagging"))) ;
        return ;
      }
    }    
  }

  public static class MultiLanguageActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();
      UIJCRExplorer uiExplorer = (UIJCRExplorer) uicomp.getAncestorOfType(UIJCRExplorer.class);
      Node curNode = uiExplorer.getSelectNode() ;
      TemplateService tservice = 
        (TemplateService)PortalContainer.getInstance().getComponentInstanceOfType(TemplateService.class) ;
      List documentNodetype = tservice.getDocumentTemplates() ;
      if(!documentNodetype.contains(curNode.getPrimaryNodeType().getName())){
        Information iprovider = findInformationProvider(uicomp);
        ResourceBundle res = getApplicationResourceBundle() ;
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN ,
            res.getString("UIViewController.msg.unsupported-multilanguage"))) ;
        return ;
      }
      UIMultiLanguageForm multiLangForm = 
        (UIMultiLanguageForm)uicomp.getChildComponentOfType(UIMultiLanguageForm.class) ; 
      multiLangForm.update(curNode) ;
      uicomp.setRenderedComponent(UIMultiLanguageForm.class);      
    }
  }

  public static class VotingDocumentActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();                             
      Information iprovider = findInformationProvider(uicomp);
      ResourceBundle res = getApplicationResourceBundle() ;
      UIDocumentInfo documentInfo = 
        (UIDocumentInfo)uicomp.getChildComponentOfType(UIDocumentInfo.class) ;
      Node currentNode = documentInfo.getNode();                 
      if(!currentNode.isNodeType("mix:votable")) {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN ,
            res.getString("UIViewController.msg.unsupported-voting"))) ;
        return ;
      }         
      UIVotingForm votingForm = (UIVotingForm)uicomp.getChildComponentOfType(UIVotingForm.class) ;
      votingForm.setDocument2Vote(currentNode) ;
      uicomp.setRenderedComponent(UIVotingForm.class) ;
    }    
  }

  public static class CommentsActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICommandNode uicomp = (UICommandNode) event.getComponent();                             
      Information iprovider = findInformationProvider(uicomp);
      ResourceBundle res = getApplicationResourceBundle() ;
      UIDocumentInfo documentInfo = 
        (UIDocumentInfo)uicomp.getChildComponentOfType(UIDocumentInfo.class) ;
      Node currentNode = documentInfo.getNode();           
      if(!currentNode.isNodeType("mix:commentable")) {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_WARN ,
            res.getString("UIViewController.msg.unsupported-comment"))) ;
        return ;
      }
      UICommentsForm commentsForm = 
        (UICommentsForm)uicomp.getChildComponentOfType(UICommentsForm.class) ;
      commentsForm.setDocument2Comments(currentNode) ;
      uicomp.setRenderedComponent(UICommentsForm.class) ;
    }    
  }
}

