package org.exoplatform.portlets.content.jcr.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.ResourceBundle;

import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.jcr.Node;
import javax.jcr.Session;
import javax.portlet.PortletPreferences;
import javax.portlet.PortletRequest;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.faces.context.PortletExternalContext;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.content.JCRUtil;
import org.exoplatform.portlets.content.jcr.component.model.Preference;
import org.exoplatform.services.cms.drives.DriveData;
import org.exoplatform.services.cms.drives.ManageDriveService;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.core.ManageableRepository;
import org.exoplatform.services.organization.Membership;
import org.exoplatform.services.organization.OrganizationService;

public class UIWorkspaceBrowser extends UIExoCommand{
  private static String SELECT_WORKSPACE_ACTION = "selectWorkspace".intern() ;
  static Parameter[] SELECT_WORKSPACE_PARAMS = { new Parameter(ACTION , SELECT_WORKSPACE_ACTION)} ;
  public static final int NB_PER_ROW = 4;
  private ManageableRepository repository;  
  private Collection memberships_ ;
  private String userName_ ;
  private ManageDriveService dservice_ ;
  public UIWorkspaceBrowser(RepositoryService repositoryService, OrganizationService oservice, 
                            ManageDriveService dservice) throws Exception {  	
    setRendererType("VelocityRenderer") ;
    setId("UIWorkspaceBrowser");
    repository = repositoryService.getRepository();   
    dservice_ = dservice ;
    FacesContext context  = FacesContext.getCurrentInstance() ;
    ExternalContext econtext = context.getExternalContext()  ;
    userName_ = econtext.getRemoteUser() ;
    memberships_ = oservice.getMembershipHandler().findMembershipsByUser(userName_) ;
    
    addActionListener(SelectWorkspaceActionListener.class, SELECT_WORKSPACE_ACTION);
  }
  
  public Parameter[] getSelectWorkspaceParams() { return SELECT_WORKSPACE_PARAMS ; }  
  
  public String[] getWorkspaces() { return repository.getWorkspaceNames(); }
  
  public List getDrives() {
    List driveList = new ArrayList() ;
    try {
      if(memberships_ != null && memberships_.size() > 0){
        Object[] objects = memberships_.toArray() ;
        for(int i = 0 ; i < objects.length ; i ++ ){
          Membership membership = (Membership)objects[i] ;
          String role = membership.getMembershipType() + ":" + membership.getGroupId() ;
          List wsByPermission = new ArrayList() ;
          wsByPermission = dservice_.getAllDriveByPermission(role) ;
          if(wsByPermission != null && wsByPermission.size() > 0) {
            for(int j = 0; j < wsByPermission.size(); j ++) {
              driveList.add(wsByPermission.get(j)) ;
            }
          }
        }      
      }
      List driveByName = dservice_.getAllDriveByPermission(userName_) ;
      for(int i = 0; i < driveByName.size(); i++) {
    	driveList.add(driveByName.get(i)) ;  
      }
    }catch(Exception e) {
      e.printStackTrace() ;
    }    
    return driveList ; 
  }
  
  public boolean hasWorkspaces(){
    String[] workspaces = repository.getWorkspaceNames();
    if(workspaces == null || workspaces.length == 0){
      return false;
    }
    return true;
  }
  
  public String getActionIcon(DriveData drive) throws Exception {
    String icon = null ;
    String path = drive.getIcon() ;
    if(path != null && path.length() > 0) {      
      try{
        Session session = repository.getSystemSession("digital-assets") ;
        Node file = (Node)session.getItem(path) ;
        if(file.hasNode("jcr:content")){
          String portalName = PortalContainer.getInstance().getPortalContainerInfo().getContainerName() ;
          icon = "<img src=\"/"+portalName+"/jcr?workspace=digital-assets&path=" + file.getPath() +
          "&relativePath=jcr:content\" width=\"64\" height=\"64\" />" ;
          return icon ;
        }
      }catch (Exception ex) {        
      }      
    }    
    ResourceBundle res = JCRUtil.getPortletBundle();
    icon = res.getString("UIWorkspaceBrowser.icon.workspace." + drive.getWorkspace());
    return icon;
  }
  
  static public class SelectWorkspaceActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event)  throws Exception {			
      UIWorkspaceBrowser uicomp = (UIWorkspaceBrowser) event.getComponent() ;			    						
      String driveName = event.getParameter(OBJECTID) ;
      DriveData drive = (DriveData)uicomp.dservice_.getDriveByName(driveName) ;      
      String workspaceName = drive.getWorkspace() ;
      String path = drive.getHomePath() ;
      String views = drive.getViews() ;      
      UIJCRExplorer explorer = (UIJCRExplorer) uicomp.getSibling(UIJCRExplorer.class);	
      ExternalContext eContext = FacesContext.getCurrentInstance().getExternalContext();
      PortletRequest request = (PortletRequest) eContext.getRequest();    
      PortletPreferences prefs_ = request.getPreferences(); 
      
      prefs_.setValue(Utils.WORKSPACE_NAME, workspaceName) ;
      prefs_.setValue(Utils.JCR_PATH, path) ;
      prefs_.setValue("views", views) ;
      prefs_.setValue("drive", driveName) ;
      prefs_.store() ;
      Session session = uicomp.repository.login(workspaceName) ; 
      Node node = (Node)session.getItem(path) ;
      explorer.getAllClipBoard().clear() ;
      explorer.setSession(session);				
      explorer.setSelectNode(node) ;
      Preference pref = explorer.getPreference() ;
      pref.setShowPreferenceDocuments(drive.getViewPreferences()) ;
      pref.setShowNonDocumentType(drive.getViewNonDocument()) ;
      pref.setLeftExplorer(drive.getViewExplorer()) ;
      pref.setClipBoardListState(drive.getViewClipboard()) ;
      pref.setEmpty(false) ;
      UIViewController viewController = (UIViewController)explorer.getChildComponentOfType(UIViewController.class) ;
      String[] array = views.split(",") ;
      viewController.renderTabs(array[0].trim()) ;
      explorer.broadcastOnChange(node) ;
      uicomp.setRenderedSibling(UIJCRExplorer.class) ;						   			
    }
  }  
}
