package org.exoplatform.portlets.content.metadata.component;

import java.util.ArrayList;
import java.util.List;

import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.NodeTypeIterator;
import javax.jcr.nodetype.NodeTypeManager;

import org.exoplatform.faces.core.Selector;
import org.exoplatform.faces.core.SelectorHandler;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UITextArea;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.faces.core.validator.PermissionValidator;
import org.exoplatform.faces.user.component.UIExoPermissionBrowser;
import org.exoplatform.faces.user.component.UIExoPermissionSelector;
import org.exoplatform.services.cms.metadata.MetadataService;

/**
 * @email:   nguyenkequanghung@yahoo.com
 * @version: $Id: UIMappingMetadataForm.java,v 1.2 2006/04/04
 */
public class UIMappingMetadataForm extends UISimpleForm implements SelectorHandler{
  
  static String  SEARCH_GROUP = "searchGroup" ;
  private MetadataService mservice_ ;
  private UISelectBox mixinTypes_ ;
  private UIExoPermissionSelector permSelector_;
  private boolean isAddNew_ = true ;
  List<String> mixinNodetypeName_ = new ArrayList<String>() ;
  
  public UIMappingMetadataForm(MetadataService mservice) throws Exception{    
    super("uiMappingMetadataForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");    
    mservice_ = mservice ;
        
    mixinTypes_ = new UISelectBox("mixinTypes", "", null) ;
    add(mixinTypes_) ;
    UITextArea dialog = new UITextArea("dialog", "") ;
    add(dialog) ;
    UITextArea view = new UITextArea("view", "") ;
    add(view) ;
    
    permSelector_ = new UIExoPermissionSelector("viewPermission", null) ;
    //permSelector_.addValidator(PermissionValidator.class) ;
    add(permSelector_) ;
    addActionListener(SearchGroupActionListener.class, SEARCH_GROUP) ;
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
    addActionListener(SaveActionListener.class, SAVE_ACTION);        
  }
  
  public String getSearchGroup() { return SEARCH_GROUP; }
  public void cancel(Selector selector) {
    selector.setRenderedSibling(UIMappingMetadataForm.class) ;
  }

  public void select(Selector selector) throws Exception {
    UIExoPermissionBrowser browser = (UIExoPermissionBrowser)selector ;
    if(browser.isGroup()) {
      permSelector_.setPermission(selector.getInputValue());
    }else {
      permSelector_.setValue(browser.getSelectedUser()) ;
    }
    selector.setRenderedSibling(UIMappingMetadataForm.class);    
  }
  
  public void update(NodeTypeManager ntManager)throws Exception{
    List<SelectItem> metadataOptions = new ArrayList<SelectItem>() ;
    NodeTypeIterator mixinTypes = ntManager.getMixinNodeTypes() ;
    List metadatas = mservice_.getMetadataList() ;
    while (mixinTypes.hasNext()) {
      NodeType nt = mixinTypes.nextNodeType() ;
      if(!metadatas.contains(nt.getName()))
        metadataOptions.add(new SelectItem(nt.getName(), nt.getName())) ;         
    }
    isAddNew_ = true ;
    permSelector_.setValue("") ;
    getUIInput("dialog").setValue("") ;    
    getUIInput("view").setValue("") ;
    mixinTypes_.setOptions(metadataOptions) ;
    mixinTypes_.setEditable(true) ;
  }
  
  public void update(String metadata)throws Exception{
    List<SelectItem> options = new ArrayList<SelectItem>() ;
    options.add(new SelectItem(metadata, metadata)) ;
    mixinTypes_.setOptions(options) ;
    mixinTypes_.setValue(metadata) ;
    String dialog = mservice_.getMetadataTemplate(metadata, true) ;
    String view = mservice_.getMetadataTemplate(metadata, false) ;
    String role = mservice_.getMetadataRoles(metadata, true) ;
    isAddNew_ = false ;
    mixinTypes_.setEditable(false) ;
    permSelector_.setValue(role) ;
    getUIInput("dialog").setValue(dialog) ;
    getUIInput("view").setValue(view) ;
  }
  
  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIMappingMetadataForm uiForm = (UIMappingMetadataForm) event.getSource();      
      String nt = uiForm.mixinTypes_.getValue() ;
      String roles = uiForm.permSelector_.getValue() ;
      String dialog = uiForm.getUIInput("dialog").getValue() ;
      String view = uiForm.getUIInput("view").getValue() ;
      uiForm.mservice_.addMetadata(nt, true, roles, dialog, uiForm.isAddNew_) ;
      uiForm.mservice_.addMetadata(nt, false, roles, view, uiForm.isAddNew_) ;
      UIMetadataManager metadataManager = 
        (UIMetadataManager)uiForm.getSibling(UIMetadataManager.class) ;
      metadataManager.refresh();
      uiForm.setRenderedSibling(UIMetadataManager.class) ;
    }  
  }
  
  static public class SearchGroupActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIMappingMetadataForm uiForm = (UIMappingMetadataForm) event.getSource();
      UIExoPermissionBrowser uiGroupBrowser = 
        (UIExoPermissionBrowser) uiForm.getSibling(UIExoPermissionBrowser.class);           
      uiGroupBrowser.setHandler(uiForm);
      uiForm.setRenderedSibling(UIExoPermissionBrowser.class);  
    }
  } 
  
  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIMappingMetadataForm uiForm = (UIMappingMetadataForm) event.getSource();
      uiForm.setRenderedSibling(UIMetadataManager.class) ;
    }
  }
}


