package org.exoplatform.portlets.content.nodetypes.component;

import java.util.ArrayList;
import java.util.List;
import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Session;
import javax.jcr.nodetype.NodeDefinition;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.NodeTypeManager;
import javax.jcr.nodetype.PropertyDefinition;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UISelectBox;
import org.exoplatform.faces.core.component.UISimpleForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.services.jcr.core.nodetype.ExtendedNodeTypeManager;
import org.exoplatform.services.jcr.core.nodetype.NodeDefinitionValue;
import org.exoplatform.services.jcr.core.nodetype.NodeTypeValue;
import org.exoplatform.services.jcr.core.nodetype.PropertyDefinitionValue;

public class UINodeTypeForm extends UISimpleForm {
  
  protected NodeType nodeType_ ;
  protected NodeTypeManager ntManager_ ;
  private PropertyDefinition[] propertyDefinitions_ ;
  private NodeDefinition[] childNodeDefinitions_ ;
  private static String VIEW_PROPERTY_ACTION = "viewProperty" ;
  private static String VIEW_CHILD_ACTION = "viewChild" ;
  private static String EDIT_PROPERTY_ACTION = "editProperty" ;
  private static String EDIT_CHILD_ACTION = "editChild" ;
  private static String ADD_PROPERTY_ACTION = "addProperty" ;
  private static String SELECT_SUPERTYPES_ACTION = "selectSuperTypes" ;
  private static String ADD_CHILD_ACTION = "addChild" ;
  private static String SAVE_DRAFT_ACTION = "saveDraft" ;
  private static String DELETE_CHILD_ACTION = "deleteChild" ;
  private static String DELETE_PROPERTY_ACTION = "deleteProperty" ;
  private static String MIXIN_ACTION = "mixin" ;
  public static Parameter[] DELETE_CHILD_PARAMS = { new Parameter(ACTION , DELETE_CHILD_ACTION)} ;
  public static Parameter[] DELETE_PROPERTY_PARAMS = { new Parameter(ACTION , DELETE_PROPERTY_ACTION)} ;
  public static Parameter[] VIEW_PROPERTY_PARAMS = { new Parameter(ACTION , VIEW_PROPERTY_ACTION)} ;
  public static Parameter[] VIEW_CHILD_PARAMS = { new Parameter(ACTION , VIEW_CHILD_ACTION)} ;
  public static Parameter[] EDIT_PROPERTY_PARAMS = { new Parameter(ACTION , EDIT_PROPERTY_ACTION)} ;
  public static Parameter[] EDIT_CHILD_PARAMS = { new Parameter(ACTION , EDIT_CHILD_ACTION)} ;
  public static Parameter[] ADD_PROPERTY_PARAMS = { new Parameter(ACTION , ADD_PROPERTY_ACTION)} ;
  public static Parameter[] ADD_CHILD_PARAMS = { new Parameter(ACTION , ADD_CHILD_ACTION)} ;
  public static Parameter[] SELECT_SUPERTYPES_PARAMS = { new Parameter(ACTION , SELECT_SUPERTYPES_ACTION)} ;
  public static Parameter[] SAVE_DRAFT_PARAMS = { new Parameter(ACTION , SAVE_DRAFT_ACTION)} ;
  
  private List<PropertyDefinitionValue> addedPropertyDef_ = new ArrayList<PropertyDefinitionValue>() ;
  private List<NodeDefinitionValue> addedChildDef_ = new ArrayList<NodeDefinitionValue>() ;
  private List<SelectItem> prefixes_ ;
  private boolean isView_ = false ;
  private boolean isDraft_ = false ;
  private boolean isMixin_ = false ;

  private Session session_ ;
  public UINodeTypeForm() {    
    super("uiNodeTypeForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");
    UISelectBox namespace = new UISelectBox("namespace","",new ArrayList()) ;
    UIStringInput nodeTypeName = new UIStringInput("nodeTypeName", "") ;
    UIStringInput primaryItemName = new UIStringInput("primaryItemName", "") ;
    List<SelectItem> booleanItem = new ArrayList<SelectItem>();
    booleanItem.add(new SelectItem("true", "true")) ;
    booleanItem.add(new SelectItem("false", "false")) ;
    UISelectBox mixinType = new UISelectBox("mixinType", "false", booleanItem) ;
    UISelectBox hasOrderableChildNodes = 
      new UISelectBox("hasOrderableChildNodes", "false", booleanItem) ;
    UIStringInput superTypes = new UIStringInput("superTypes","");
    add(namespace) ;
    add(nodeTypeName) ;
    add(primaryItemName) ;
    mixinType.setUpdateOnChangeAction(MIXIN_ACTION) ;
    add(mixinType) ;
    add(superTypes) ;
    add(hasOrderableChildNodes) ;
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
    addActionListener(SaveActionListener.class, SAVE_ACTION);
    addActionListener(ViewPropertiesActionListener.class, VIEW_PROPERTY_ACTION);
    addActionListener(ViewChildNodeActionListener.class, VIEW_CHILD_ACTION);
    addActionListener(AddPropertiesActionListener.class, ADD_PROPERTY_ACTION);
    addActionListener(AddChildNodeActionListener.class, ADD_CHILD_ACTION);
    addActionListener(SelectSuperTypesActionListener.class, SELECT_SUPERTYPES_ACTION);
    addActionListener(SaveDraftActionListener.class, SAVE_DRAFT_ACTION);
    addActionListener(DeleteChildActionListener.class, DELETE_CHILD_ACTION);
    addActionListener(DeletePropertyActionListener.class, DELETE_PROPERTY_ACTION);
    addActionListener(EditPropertiesActionListener.class, EDIT_PROPERTY_ACTION);
    addActionListener(EditChildActionListener.class, EDIT_CHILD_ACTION);
    addActionListener(ChangeMixinActionListener.class, MIXIN_ACTION);    
  }
  
  public Parameter[] getViewPropertyParams() { return VIEW_PROPERTY_PARAMS ;  }
  public Parameter[] getViewChildParams() { return VIEW_CHILD_PARAMS ;  }
  public Parameter[] getEditPropertyParams() { return EDIT_PROPERTY_PARAMS ;  }
  public Parameter[] getEditChildParams() { return EDIT_CHILD_PARAMS ;  }
  public Parameter[] getDeleteChildParams() { return DELETE_CHILD_PARAMS ;  }
  public Parameter[] getDeletePropertyParams() { return DELETE_PROPERTY_PARAMS ;  }
  public String getAddPropertyParams() { return ADD_PROPERTY_ACTION ;  }
  public String getAddChildParams() { return ADD_CHILD_ACTION ;  }  
  public String selectSuperTypesParams() { return SELECT_SUPERTYPES_ACTION ;  }
  public String saveDraftParams() { return SAVE_DRAFT_ACTION ;  }
  
  public void addNewPropertyDefinition(PropertyDefinitionValue property) {
    for(int i = 0; i < addedPropertyDef_.size(); i ++){
      if(addedPropertyDef_.get(i).getName().equals(property.getName())) {
        addedPropertyDef_.remove(i) ;        
        break;
      }
    }
    addedPropertyDef_.add(property) ;    
  }
  public void addNewChildNodeDefinition(NodeDefinitionValue nodeDef) {
    for(int i = 0; i < addedChildDef_.size(); i ++){
      if(addedChildDef_.get(i).getName().equals(nodeDef.getName())) {
        addedChildDef_.remove(i) ;
        break;
      }
    }
    addedChildDef_.add(nodeDef) ;
  }
  
  public boolean isView(){ return isView_ ; }
  
  public void update(Session session, NodeType nodeType, NodeTypeManager ntManager, String[] namespaces, boolean isView){    
    isView_ = isView ;
    isDraft_ = false;    
    session_ = session ;
    prefixes_ = new ArrayList<SelectItem>() ;
    for( int i = 0; i < namespaces.length; i ++){
      prefixes_.add(new SelectItem(namespaces[i], namespaces[i])) ;
    }
    getUISelectBox("namespace").setOptions(prefixes_) ;
    if(nodeType != null ){
      String name = nodeType.getName() ;
      String noteTypeName="", namespacesPrefix="";
      try {
        noteTypeName = name.substring(name.indexOf(":") + 1) ;
        namespacesPrefix = name.substring(0, name.indexOf(":")) ;
      } catch (Exception e) {
        noteTypeName = name ;
        namespacesPrefix = "" ;
      }      
      getUIStringInput("nodeTypeName").setValue(noteTypeName) ;
      getUISelectBox("namespace").setValue(namespacesPrefix) ;
      
      getUIStringInput("primaryItemName").setValue(nodeType.getPrimaryItemName()) ;
      isMixin_ = nodeType.isMixin() ;
      getUISelectBox("mixinType").setValue(String.valueOf(nodeType.isMixin())) ;
      getUISelectBox("hasOrderableChildNodes").setValue(
          String.valueOf(nodeType.hasOrderableChildNodes())) ;      
      NodeType[] superType = nodeType.getSupertypes() ;
      StringBuffer types = new StringBuffer(); 
      for(int i = 0; i < superType.length; i++){
        if(types.length() > 0) types.append(", ") ;
        types.append(superType[i].getName()) ;
      }
      if(isMixin_){
        getUIStringInput("superTypes").setValue("").setEditable(false) ;
      } else {
        getUIStringInput("superTypes").setValue(types.toString()).setEditable(true) ;
      }
      if(isView){
        getUISelectBox("namespace").setEditable(false) ;
        getUIStringInput("nodeTypeName").setEditable(false) ;
        getUIStringInput("primaryItemName").setEditable(false) ;
        getUISelectBox("mixinType").setEditable(false) ;
        getUISelectBox("hasOrderableChildNodes").setEditable(false);
        getUIStringInput("superTypes").setEditable(false) ;
      }
      ntManager_ = ntManager ; 
      nodeType_ = nodeType ;
      propertyDefinitions_ = nodeType.getPropertyDefinitions() ;
      childNodeDefinitions_ = nodeType.getChildNodeDefinitions() ;
    }else{
      ntManager_ = ntManager ;  
      propertyDefinitions_ = null ;
      childNodeDefinitions_ = null ;
      isMixin_ = false ;
      refresh() ;
    }
    addedPropertyDef_ = new ArrayList<PropertyDefinitionValue>() ;
    addedChildDef_ = new ArrayList<NodeDefinitionValue>() ;     
  }
  
  public boolean isMixin() { return isMixin_ ; }
  
  public void updateEdit(Node draftNodeType, NodeTypeManager ntManager, String[] namespaces, 
                          boolean isView, boolean isDraft) throws Exception{    
    isDraft_ = isDraft ; 
    isView_ = isView;    
    session_ = draftNodeType.getSession() ;
    prefixes_ = new ArrayList<SelectItem>() ;
    for( int i = 0; i < namespaces.length; i ++){
      prefixes_.add(new SelectItem(namespaces[i], namespaces[i])) ;
    }
    getUISelectBox("namespace").setOptions(prefixes_) ;
    String name = draftNodeType.getName() ;
    String noteTypeName="", namespacesPrefix="";
    try {
      noteTypeName = name.substring(name.indexOf(":") + 1) ;
      namespacesPrefix = name.substring(0, name.indexOf(":")) ;
    }catch (Exception e) {      
      noteTypeName = name ;
      namespacesPrefix = "" ;
    }    
    getUIStringInput("nodeTypeName").setValue(noteTypeName) ;
    getUISelectBox("namespace").setValue(namespacesPrefix) ;
    
    getUIStringInput("nodeTypeName").setEditable(false) ;    
    getUIStringInput("primaryItemName").setValue(draftNodeType.getProperty("primaryItemName").getString()) ;
    getUISelectBox("mixinType").setValue(draftNodeType.getProperty("isMixin").getString()) ;
    getUISelectBox("hasOrderableChildNodes").setValue(draftNodeType.getProperty("hasOrderableChildNodes").getString()) ;      
    getUISelectBox("namespace").setEditable(false) ;
    getUIStringInput("nodeTypeName").setEditable(false) ;
    getUIStringInput("primaryItemName").setEditable(true) ;
    getUISelectBox("mixinType").setEditable(true) ;
    getUISelectBox("hasOrderableChildNodes").setEditable(true) ;
    isMixin_ = draftNodeType.getProperty("isMixin").getBoolean() ;
    if(isMixin_) {
      getUIStringInput("superTypes").setValue("").setEditable(false) ;      
    }else {
      getUIStringInput("superTypes").setValue(draftNodeType.getProperty("superTypes").getString()).
      setEditable(true) ;
    }
    
    addedPropertyDef_ = new ArrayList<PropertyDefinitionValue>() ;
    if(draftNodeType.hasNode("propertyDefinitions")) {
      Node propertiesHome = draftNodeType.getNode("propertyDefinitions") ;
      NodeIterator proIter = propertiesHome.getNodes() ;
      while (proIter.hasNext()) {
        Node proDef = proIter.nextNode() ;
        PropertyDefinitionValue propertyInfo = new PropertyDefinitionValue() ;
        propertyInfo.setName(proDef.getName()) ;          
        int requiredType = Integer.parseInt(proDef.getProperty("requiredType").getString()) ;
        propertyInfo.setRequiredType(requiredType) ;
        propertyInfo.setMultiple(Boolean.parseBoolean(proDef.getProperty("multiple").getString())) ;          
        propertyInfo.setMandatory(Boolean.parseBoolean(proDef.getProperty("mandatory").getString())) ;          
        propertyInfo.setAutoCreate(Boolean.parseBoolean(proDef.getProperty("autoCreated").getString())) ;
        propertyInfo.setReadOnly(Boolean.parseBoolean(proDef.getProperty("protected").getString())) ;
        int onParent = Integer.parseInt(proDef.getProperty("onParentVersion").getString()) ;
        propertyInfo.setOnVersion(onParent) ; 
        String constraints = proDef.getProperty("valueConstraints").getString() ;
        List<String> valueConst = new ArrayList<String>() ;
        if(constraints != null && constraints.length() > 0) {
          if(constraints.indexOf(",") > -1) {
            String[] array = constraints.split(",") ;
            for(int i = 0; i < array.length; i ++) {
              valueConst.add(array[i].trim()) ;
            }
          }else{
            valueConst.add(constraints) ;
          }
        }
        propertyInfo.setValueConstraints(valueConst) ;
        addedPropertyDef_.add(propertyInfo) ;
      }
    }
    
    addedChildDef_ = new ArrayList<NodeDefinitionValue>() ;
    if(draftNodeType.hasNode("childNodeDefinitions")) {
      Node childNodeHome = draftNodeType.getNode("childNodeDefinitions") ; 
      NodeIterator childIter = childNodeHome.getNodes() ;
      while(childIter.hasNext()) {
        Node childDef = childIter.nextNode() ;
        NodeDefinitionValue nodeTypeValue = new NodeDefinitionValue();
        nodeTypeValue.setName(childDef.getName()) ;
        nodeTypeValue.setDefaultNodeTypeName(childDef.getProperty("defaultPrimaryType").getString()) ;
        nodeTypeValue.setSameNameSiblings(
            Boolean.parseBoolean(childDef.getProperty("sameNameSiblings").getString())) ;      
        nodeTypeValue.setMandatory(
            Boolean.parseBoolean(childDef.getProperty("mandatory").getString())) ;
        nodeTypeValue.setAutoCreate(
            Boolean.parseBoolean(childDef.getProperty("autoCreated").getString())) ;
        int parentVer = Integer.parseInt(childDef.getProperty("onParentVersion").getString()) ;
          nodeTypeValue.setOnVersion(parentVer) ;
        String requiredTypes = childDef.getProperty("requiredPrimaryType").getString() ;
        List<String> requiredList = new ArrayList<String>() ;
        if(requiredTypes != null && requiredTypes.length() > 0) {
          if(requiredTypes.indexOf(",") > -1){
            String[] array = requiredTypes.split(",") ;
            for(int i = 0; i < array.length; i ++) {
              requiredList.add(array[i].trim()) ;
            }
          }else {
            requiredList.add(requiredTypes) ;
          }              
        }
        nodeTypeValue.setRequiredNodeTypeNames(requiredList) ;
        addedChildDef_.add(nodeTypeValue) ;          
      }
    }
    ntManager_ = ntManager ;  
    propertyDefinitions_ = null ;
    childNodeDefinitions_ = null ;
  }
  
  private void refresh() {
    getUIStringInput("nodeTypeName").setValue("") ;
    getUIStringInput("primaryItemName").setValue("") ;
    getUISelectBox("mixinType").setValue("false") ;
    getUISelectBox("hasOrderableChildNodes").setValue("false") ;
    getUIStringInput("superTypes").setValue("") ;
    getUISelectBox("namespace").setEditable(true) ;
    getUIStringInput("nodeTypeName").setEditable(true) ;
    getUIStringInput("primaryItemName").setEditable(true) ;
    getUISelectBox("mixinType").setEditable(true) ;
    getUISelectBox("hasOrderableChildNodes").setEditable(true) ;
    getUIStringInput("superTypes").setEditable(true) ;
  }
  
  public boolean hasPropertyDefinitions() {
    if(propertyDefinitions_ != null && propertyDefinitions_.length > 0) return true ;
    return false ;
  }
  
  public List getPropertyDefinitions() {
    List<String> propertyDefName = new ArrayList<String>() ;
    for(int i = 0 ; i < propertyDefinitions_.length; i++){
      propertyDefName.add(propertyDefinitions_[i].getName()) ;
    }
    return propertyDefName;
  }
  
  public void setSuperTypes( List supertypes) {
    StringBuffer types = new StringBuffer("") ;
    for(int i = 0; i < supertypes.size(); i ++) {
      if(types.length() > 0) 
        types.append(", ") ;
      types.append(supertypes.get(i)) ;
    }
    getUIStringInput("superTypes").setValue(types.toString()) ;
  }
  
  public boolean hasAddedPropertyDefinitions() {
    if(addedPropertyDef_ != null && addedPropertyDef_.size() > 0) return true ;
    return false ;
  }
  
  public List getAddedPropertyDefinitions() {
    return addedPropertyDef_;
  }
  
  public boolean hasAddedChildNodeDefinitions() {
    if(addedChildDef_ != null && addedChildDef_.size() > 0) return true ;
    return false ;
  }
  
  public List getAddedChildNodeDefinitions() {
    return addedChildDef_ ;
  }
  
  public boolean hasChildNodeDefinitions() {
    if(childNodeDefinitions_ != null && childNodeDefinitions_.length > 0) return true ;
    return false ;
  }
  public List getChildNodeDefinitions(){
    List<String> childNodeDefName = new ArrayList<String>() ;
    for(int i = 0 ; i < childNodeDefinitions_.length; i++){
      childNodeDefName.add(childNodeDefinitions_[i].getName()) ;
    }
    return childNodeDefName;
  }
  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypeForm uiForm = (UINodeTypeForm) event.getSource();
      Information iprovider = findInformationProvider(uiForm);
      ResourceBundle res = getApplicationResourceBundle() ;
      String prefix = uiForm.getUISelectBox("namespace").getValue() ;
      String nodeTypeName ;
      String nodeTypeNameValue = uiForm.getUIStringInput("nodeTypeName").getValue() ;
      if(nodeTypeNameValue == null || nodeTypeNameValue.length() < 1) {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR ,
            res.getString("UINodeTypeForm.msg.name-is-madatory"))) ;
        return ;
      }
      if(prefix == null || prefix.length() == 0 ){
        nodeTypeName = nodeTypeNameValue;
      }else {
        nodeTypeName = prefix + ":" + nodeTypeNameValue ;
      }      
      String superTypes = uiForm.getUIStringInput("superTypes").getValue() ;
      if(!uiForm.isMixin_) {
        if(superTypes == null || superTypes.length() == 0 || "".equals(superTypes)) {
          iprovider.addMessage( new ExoFacesMessage(FacesMessage.SEVERITY_ERROR ,
              res.getString("UINodeTypeForm.msg.supertype-is-madatory"))) ;
          return ;
        }
      }            
      String[] arrayTypes = superTypes.split(",") ;
      List<String> listTypes = new ArrayList<String>() ;
      for(String type :arrayTypes ) {
        listTypes.add(type.trim()) ;
      }                  
      String primaryItemName = uiForm.getUIStringInput("primaryItemName").getValue() ;
      
      NodeTypeValue newNodeType = new NodeTypeValue() ;                             
      newNodeType.setName(nodeTypeName) ;            
      newNodeType.setPrimaryItemName(primaryItemName) ;      
      newNodeType.setMixin(Boolean.parseBoolean(uiForm.getUISelectBox("mixinType").getValue())) ;      
      newNodeType.setOrderableChild(
          Boolean.parseBoolean(uiForm.getUISelectBox("hasOrderableChildNodes").getValue())) ;      
      newNodeType.setDeclaredSupertypeNames(listTypes) ;      
      newNodeType.setDeclaredPropertyDefinitionValues(uiForm.addedPropertyDef_) ;
      newNodeType.setDeclaredChildNodeDefinitionValues(uiForm.addedChildDef_) ;
      ExtendedNodeTypeManager extNTManager = (ExtendedNodeTypeManager)uiForm.ntManager_ ;
      boolean registerStatus = true ;
      try{        
        extNTManager.registerNodeType(newNodeType, ExtendedNodeTypeManager.FAIL_IF_EXISTS) ;
      } catch (Exception e) {
        registerStatus = false;
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR,e.getMessage())) ;        
      }      
      UINodeTypeList nodeTypeList = (UINodeTypeList)uiForm.getSibling(UINodeTypeList.class) ;
      if(uiForm.isDraft_){
        uiForm.isDraft_ = false ;
        nodeTypeList.refresh(nodeTypeName) ;
      }else {
        nodeTypeList.refresh(null) ; 
      } 
      if(registerStatus) {
        Object[] args = {newNodeType.getName() } ; 
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_INFO,
                            res.getString("UINodeTypeForm.msg.register-successfully"),args)) ; 
      }      
      uiForm.setRenderedSibling(UINodeTypeList.class) ;      
    }           
  }
  
  static public class SaveDraftActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypeForm uiForm = (UINodeTypeForm) event.getSource();
      String prefix = uiForm.getUISelectBox("namespace").getValue() ;
      String nodeTypeNameValue = uiForm.getUIStringInput("nodeTypeName").getValue().trim() ;
      String nodeTypeName = "" ;
      if(prefix == null || prefix.length() == 0){
        nodeTypeName = nodeTypeNameValue ;        
      }else {        
        nodeTypeName = prefix + ":" + nodeTypeNameValue ;
      }  
      
      try{
        Node systemNode = (Node)uiForm.session_.getItem("/jcr:system") ;
        Node nodeTypeDraft ;
        if(systemNode.hasNode("jcr:nodetypesDraft")) {
          nodeTypeDraft = systemNode.getNode("jcr:nodetypesDraft") ;
        }else {
          nodeTypeDraft = systemNode.addNode("jcr:nodetypesDraft", "nt:unstructured") ;
        }
        systemNode.save() ;        
        
        Node newNode ;
        if(nodeTypeDraft.hasNode(nodeTypeName)) {
          newNode = nodeTypeDraft.getNode(nodeTypeName) ;
        }else  {
          newNode = nodeTypeDraft.addNode(nodeTypeName, "nt:unstructured") ;
        }
        nodeTypeDraft.save() ;
        String primaryItemName = uiForm.getUIStringInput("primaryItemName").getValue() ;
        newNode.setProperty("primaryItemName", primaryItemName) ;
        newNode.setProperty("isMixin", uiForm.getUISelectBox("mixinType").getValue()) ;
        newNode.setProperty("hasOrderableChildNodes", 
            uiForm.getUISelectBox("hasOrderableChildNodes").getValue()) ;      
        String superTypes = uiForm.getUIStringInput("superTypes").getValue() ;
        newNode.setProperty("superTypes", superTypes) ;
        newNode.save() ;
        nodeTypeDraft.save() ;
        if(uiForm.addedPropertyDef_ .size() > 0) {
          Node propertyDef ;
          if(newNode.hasNode("propertyDefinitions")) {
            propertyDef = newNode.getNode("propertyDefinitions") ;          
          }else {
            propertyDef = newNode.addNode("propertyDefinitions", "nt:unstructured") ;
          }
          newNode.save() ;
          propertyDef.save() ;
          for(int i = 0; i < uiForm.addedPropertyDef_.size(); i ++) {
            PropertyDefinitionValue property = uiForm.addedPropertyDef_.get(i) ;
            Node propertyNode ;
            if(propertyDef.hasNode(property.getName())) {
              propertyNode = propertyDef.getNode(property.getName()) ;            
            }else {
              propertyNode = propertyDef.addNode(property.getName(), "nt:unstructured") ;
              propertyNode.setProperty("name", property.getName()) ;
            }
            propertyNode.setProperty("requiredType", String.valueOf(property.getRequiredType())) ;
            propertyNode.setProperty("autoCreated", String.valueOf(property.isAutoCreate())) ;
            propertyNode.setProperty("mandatory", String.valueOf(property.isMandatory())) ;
            propertyNode.setProperty("onParentVersion", String.valueOf(property.getOnVersion())) ;
            propertyNode.setProperty("protected", String.valueOf(property.isReadOnly())) ;
            propertyNode.setProperty("multiple", String.valueOf(property.isMultiple())) ;
            if(property.getValueConstraints().size() > 0) {
              StringBuffer values = new StringBuffer("") ;
              for(int j = 0; j < property.getValueConstraints().size(); j ++) {
                if(values.length() > 0) 
                  values.append(", ") ;
                values.append(property.getValueConstraints().get(j).toString()) ;
              }
              propertyNode.setProperty("valueConstraints", values.toString()) ;
            }else {
              propertyNode.setProperty("valueConstraints", "") ;
            }
            propertyDef.save() ;
            propertyNode.save() ;
          }
        }
        
        if(uiForm.addedChildDef_.size() > 0) {
          Node childDef ; 
          if(newNode.hasNode("childNodeDefinitions")) {
            childDef = newNode.getNode("childNodeDefinitions") ;          
          }else {
            childDef = newNode.addNode("childNodeDefinitions", "nt:unstructured") ;
          }
          newNode.save() ;
          childDef.save() ;
          for(int i = 0; i < uiForm.addedChildDef_.size(); i ++ ){
            NodeDefinitionValue nodeDef =  uiForm.addedChildDef_.get(i) ;
            Node childNode ;
            if(childDef.hasNode(nodeDef.getName())) {
              childNode = childDef.getNode(nodeDef.getName()) ;
            }else {
              childNode = childDef.addNode(nodeDef.getName(), "nt:unstructured") ;
              childNode.setProperty("name", nodeDef.getName()) ;
            }
            childNode.setProperty("defaultPrimaryType", nodeDef.getDefaultNodeTypeName()) ;
            childNode.setProperty("autoCreated", String.valueOf(nodeDef.isAutoCreate())) ; 
            childNode.setProperty("mandatory", String.valueOf(nodeDef.isMandatory())) ;
            childNode.setProperty("onParentVersion", String.valueOf(nodeDef.getOnVersion())) ;
            childNode.setProperty("protected", String.valueOf(nodeDef.isReadOnly())) ;
            childNode.setProperty("sameNameSiblings", String.valueOf(nodeDef.isSameNameSiblings())) ;
            if(nodeDef.getRequiredNodeTypeNames().size() > 0 ) {
              StringBuffer requiredTypes = new StringBuffer("") ;
              for(int j = 0; j < nodeDef.getRequiredNodeTypeNames().size(); j ++) {
                if( requiredTypes.length() > 0)
                  requiredTypes.append(",") ;
                requiredTypes.append(nodeDef.getRequiredNodeTypeNames().get(j).toString()) ;
              }
              childNode.setProperty("requiredPrimaryType", requiredTypes.toString()) ;
            }else {
              childNode.setProperty("requiredPrimaryType", "") ;
            }
            childDef.save() ;
            childNode.save() ;
          }
        }
        uiForm.session_.save() ;
      }catch(Exception e){
      }      
      UINodeTypeList nodeTypeList = (UINodeTypeList)uiForm.getSibling(UINodeTypeList.class) ;
      nodeTypeList.refresh(null);
      uiForm.setRenderedSibling(UINodeTypeList.class) ;
    }
  }
  
  static public class DeleteChildActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypeForm uiForm = (UINodeTypeForm) event.getSource();
      String childName = event.getParameter(OBJECTID) ;
      for(int i = 0; i < uiForm.addedChildDef_.size(); i ++) {
        if(uiForm.addedChildDef_.get(i).getName().equals(childName)) {
          uiForm.addedChildDef_.remove(i) ;
          break;
        } 
      }
    }
  }
  
  static public class DeletePropertyActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypeForm uiForm = (UINodeTypeForm) event.getSource();
      String propertyName = event.getParameter(OBJECTID) ;
      for(int i = 0; i < uiForm.addedPropertyDef_.size(); i ++) {
        if(uiForm.addedPropertyDef_.get(i).getName().equals(propertyName)) {
          uiForm.addedPropertyDef_.remove(i) ;
          break;
        } 
      }
    }
  }
  
  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypeForm uiForm = (UINodeTypeForm) event.getSource();
      uiForm.setRenderedSibling(UINodeTypeList.class) ;
    }
  }
  
  static public class EditPropertiesActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypeForm uiForm = (UINodeTypeForm) event.getSource();
      UIPropertyDefinitionForm propertyDefForm = 
        (UIPropertyDefinitionForm)uiForm.getSibling(UIPropertyDefinitionForm.class) ;
      String defName = event.getParameter(OBJECTID) ;
      for(int i = 0; i < uiForm.addedPropertyDef_.size(); i ++) {
        if(uiForm.addedPropertyDef_.get(i).getName().equals(defName)) {
          propertyDefForm.updateEdit(uiForm.addedPropertyDef_.get(i), uiForm.prefixes_) ;
          break ;
        }
      }
      uiForm.setRenderedSibling(UIPropertyDefinitionForm.class) ;      
    }
  }
  
  static public class ViewPropertiesActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypeForm uiForm = (UINodeTypeForm) event.getSource();
      String defName = event.getParameter(OBJECTID) ;
      UIPropertyDefinitionForm propertyDefForm = 
        (UIPropertyDefinitionForm)uiForm.getSibling(UIPropertyDefinitionForm.class) ;
      propertyDefForm.update(uiForm.nodeType_, defName, uiForm.ntManager_, uiForm.prefixes_, uiForm.isView_) ;
      uiForm.setRenderedSibling(UIPropertyDefinitionForm.class) ;      
    }
  }
  
  static public class ViewChildNodeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypeForm uiForm = (UINodeTypeForm) event.getSource();
      String defName = event.getParameter(OBJECTID) ;
      UIChildNodeDefinitionForm childDefForm = 
        (UIChildNodeDefinitionForm)uiForm.getSibling(UIChildNodeDefinitionForm.class) ;
      childDefForm.update(uiForm.ntManager_, uiForm.nodeType_, defName, uiForm.prefixes_, uiForm.isView_) ;
      uiForm.setRenderedSibling(UIChildNodeDefinitionForm.class) ;
    }
  }
  
  static public class EditChildActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypeForm uiForm = (UINodeTypeForm) event.getSource();
      UIChildNodeDefinitionForm childDefForm = 
        (UIChildNodeDefinitionForm)uiForm.getSibling(UIChildNodeDefinitionForm.class) ;
      String defName = event.getParameter(OBJECTID) ;
      for(int i = 0; i < uiForm.addedChildDef_.size(); i ++) {
        if(uiForm.addedChildDef_.get(i).getName().equals(defName)) {
          childDefForm.updateEdit(uiForm.addedChildDef_.get(i), uiForm.prefixes_) ;
          break ;
        }
      }
      uiForm.setRenderedSibling(UIChildNodeDefinitionForm.class) ;      
    }
  }
  
  static public class AddPropertiesActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypeForm uiForm = (UINodeTypeForm) event.getSource();
      UIPropertyDefinitionForm propertyDefForm = 
        (UIPropertyDefinitionForm)uiForm.getSibling(UIPropertyDefinitionForm.class) ;
      propertyDefForm.update(uiForm.nodeType_, null, uiForm.ntManager_, uiForm.prefixes_, false) ;
      uiForm.setRenderedSibling(UIPropertyDefinitionForm.class) ;      
    }
  }
  static public class AddChildNodeActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypeForm uiForm = (UINodeTypeForm) event.getSource();
      UIChildNodeDefinitionForm childDefForm = 
        (UIChildNodeDefinitionForm)uiForm.getSibling(UIChildNodeDefinitionForm.class) ;
      childDefForm.update(uiForm.ntManager_, uiForm.nodeType_, null, uiForm.prefixes_,false) ;
      uiForm.setRenderedSibling(UIChildNodeDefinitionForm.class) ;
    }
  }
  
  static public class SelectSuperTypesActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypeForm uiForm = (UINodeTypeForm) event.getSource();
      String superTypes = uiForm.getUIStringInput("superTypes").getValue() ;
      List<String> types = new ArrayList<String>() ;
      if(superTypes != null && superTypes.length() > 0) {
        if(superTypes.indexOf(",") > -1) {
          String[] array = superTypes.split(",") ;
          for(int i = 0; i < array.length; i ++){
            types.add(array[i].trim()) ;
          }
        }else{
          types.add(superTypes.trim()) ;
        }
      }
      UIOptionList optionList = (UIOptionList)uiForm.getSibling(UIOptionList.class) ;
      optionList.update(uiForm.ntManager_, types, "supertypes") ;
      uiForm.setRenderedSibling(UIOptionList.class) ;
    }
  }
  static public class ChangeMixinActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypeForm uiForm = (UINodeTypeForm) event.getSource();
      if(uiForm.getUISelectBox("mixinType").getValue().equals("true")){
        uiForm.getUIStringInput("superTypes").setValue("").setEditable(false);
        uiForm.isMixin_ = true ;
      }else {
        uiForm.getUIStringInput("superTypes").setEditable(true);
        uiForm.isMixin_ = false ;
      }
    }
  }
}
