package org.exoplatform.portlets.content.nodetypes.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.Session;
import javax.jcr.nodetype.NodeType;
import javax.jcr.nodetype.NodeTypeIterator;
import javax.jcr.nodetype.NodeTypeManager;

import org.exoplatform.faces.cms.CMSUtils;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.model.PageListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.content.ContentPageList;
import org.exoplatform.portlets.content.admin.component.UIECMAdminSummary;
import org.exoplatform.portlets.content.jcr.component.ChangeNodeListener;
import org.exoplatform.portlets.content.jcr.component.UIJCRExplorer;
import org.exoplatform.services.cms.CmsConfigurationService;
import org.exoplatform.services.jcr.RepositoryService;
import org.exoplatform.services.jcr.core.ManageableRepository;

public class UINodeTypeList extends UIExoCommand implements ChangeNodeListener{
  
  final static public String ADD_ACTION = "add".intern();    
  final static public String EXPORT_ACTION = "export".intern();
  final static public String IMPORT_ACTION = "import".intern();
  
  public static Parameter[] VIEW_PARAMS = { new Parameter(ACTION , VIEW_ACTION)} ;
  public static Parameter[] EDIT_PARAMS = { new Parameter(ACTION , EDIT_ACTION)} ;
  public static Parameter[] DELETE_PARAMS = { new Parameter(ACTION , DELETE_ACTION)} ;
  public static Parameter[] IMPORT_PARAMS = { new Parameter(ACTION , IMPORT_ACTION)} ;
  public static Parameter[] ADD_PARAMS = { new Parameter(ACTION , ADD_ACTION)} ;
  public static Parameter[] EXPORT_PARAMS = { new Parameter(ACTION , EXPORT_ACTION)} ;
  public static Parameter[] BACK_PARAMS = { new Parameter(ACTION , BACK_ACTION)} ;
  
  private Session session_ ;
  protected NodeTypeManager ntManager_ ;
  private List nodeTypeList_ = new ArrayList();
  private ManageableRepository repository_ ;
  private CmsConfigurationService cmsConfigurationService_ ;
  public UIPageListIterator uiPageIterator_ ; 
  
  public UINodeTypeList(RepositoryService repositoryService,      
        CmsConfigurationService cmsConfigurationService) throws Exception{
    setId("UINodeTypeList") ;
    setRendererType("VelocityRenderer") ;
    cmsConfigurationService_ = cmsConfigurationService ;
    repository_ = repositoryService.getRepository() ;
    session_ = repository_.getSystemSession(cmsConfigurationService.getWorkspace()) ; 
    ntManager_ = session_.getWorkspace().getNodeTypeManager();     
    uiPageIterator_ = new UIPageListIterator(new PageListDataHandler()) ;  
    getChildren().add(uiPageIterator_);
    addActionListener(ViewActionListener.class, VIEW_ACTION) ;
    addActionListener(EditActionListener.class, EDIT_ACTION) ;
    addActionListener(DeleteActionListener.class, DELETE_ACTION) ;
    addActionListener(ImportActionListener.class, IMPORT_ACTION) ;    
    addActionListener(AddActionListener.class, ADD_ACTION) ;
    addActionListener(ExportActionListener.class, EXPORT_ACTION) ;
    addActionListener(BackActionListener.class, BACK_ACTION) ;   
    uiPageIterator_.setPageList(new ContentPageList(getAllNodeTypes()));       
  }  
  
  public Collection getPageListDataHandler() throws Exception{   
    return uiPageIterator_.getPageListDataHandler().getDatas();        
  }
  
  public Parameter[] getAddParams() { return ADD_PARAMS ; }
  public Parameter[] getExportParams() { return EXPORT_PARAMS ; }
  public Parameter[] getImportParams() { return IMPORT_PARAMS ; }
  public Parameter[] getEditParams() { return EDIT_PARAMS ; }
  public Parameter[] getViewParams() { return VIEW_PARAMS ; }
  public Parameter[] getDeleteParams() { return DELETE_PARAMS ; }
  public Parameter[] getBackParams() { return BACK_PARAMS ; }
  
  public void  onChange(UIJCRExplorer uiExplorer, Node node)  throws Exception {
    setRendered(false) ;
    session_ = uiExplorer.getSession() ;
  }
  
  public String getIcon(String nodeType) {
    return CMSUtils.getIconByNodeTypeName(nodeType) ;
  }
  public List getAllNodeTypes() throws Exception{
    List nodeList = new ArrayList<NodeType>(); 
    NodeTypeIterator nodeTypeIter = ntManager_.getAllNodeTypes() ;    
    while(nodeTypeIter.hasNext()){
      nodeList.add(nodeTypeIter.nextNodeType()) ;
    }    
    Collections.sort(nodeList, new NodeTypeNameComparator()) ;
    try{
      Node draftNode = session_.getRootNode().getNode("jcr:system/jcr:nodetypesDraft") ;    
      NodeIterator nodeIter = draftNode.getNodes() ;
      while(nodeIter.hasNext()){
        nodeList.add(nodeIter.nextNode()) ;
      }
    }catch (Exception e){
      
    }
    nodeTypeList_ = nodeList ;    
    return nodeList ;
  }
  
  public void refresh(String name) throws Exception{
    if(name != null) {
      Node draftNode = session_.getRootNode().getNode("jcr:system/jcr:nodetypesDraft") ;
      Node deleteNode = draftNode.getNode(name) ;
      deleteNode.remove() ;            
      draftNode.save() ;
      session_ = repository_.getSystemSession(cmsConfigurationService_.getWorkspace()) ; 
      ntManager_ = session_.getWorkspace().getNodeTypeManager(); 
      draftNode = session_.getRootNode().getNode("jcr:system/jcr:nodetypesDraft") ;
      if(!draftNode.hasNodes())        
        draftNode.remove() ;
      session_.save() ;
    }else {
      session_.refresh(true) ;
    }    
    uiPageIterator_.setPageList(new ContentPageList(getAllNodeTypes()));        
  }
  
  static public class NodeTypeNameComparator implements Comparator {
    public int compare(Object o1, Object o2) throws ClassCastException {
    	String name1 = ((NodeType) o1).getName() ;
    	String name2 = ((NodeType) o2).getName() ;
    	return name1.compareToIgnoreCase(name2) ;
    }
  }
  
  static public class AddActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypeList uicomp = (UINodeTypeList) event.getComponent() ;
      UINodeTypeForm nodeTypeForm = (UINodeTypeForm)uicomp.getSibling(UINodeTypeForm.class) ;
      String[] namespaces = uicomp.repository_.getNamespaceRegistry().getPrefixes() ;
      nodeTypeForm.update(uicomp.session_, null, uicomp.ntManager_, namespaces, false) ;
      uicomp.setRenderedSibling(UINodeTypeForm.class) ;
    }
  }
  
  static public class ExportActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypeList uicomp = (UINodeTypeList) event.getComponent() ;
      UINodeTypeExport nodeTypeExport = (UINodeTypeExport)uicomp.getSibling(UINodeTypeExport.class) ;
      nodeTypeExport.update(uicomp.ntManager_) ;
      uicomp.setRenderedSibling(UINodeTypeExport.class) ;
    }
  }
  
  static public class ImportActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypeList uicomp = (UINodeTypeList) event.getComponent() ;
      UINodeTypeUpload upload = (UINodeTypeUpload) uicomp.getSibling(UINodeTypeUpload.class) ;
      upload.update(uicomp.ntManager_) ;
      uicomp.setRenderedSibling(UINodeTypeUpload.class) ;      
    }
  }
  
  static public class ViewActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypeList uicomp = (UINodeTypeList) event.getComponent() ;
      String nodeName = event.getParameter(OBJECTID) ;
      NodeType nodeType = uicomp.ntManager_.getNodeType(nodeName) ;
      UINodeTypeForm nodeTypeForm = (UINodeTypeForm)uicomp.getSibling(UINodeTypeForm.class) ;      
      String[] namespaces = uicomp.repository_.getNamespaceRegistry().getPrefixes() ;      
      nodeTypeForm.update(uicomp.session_, nodeType, uicomp.ntManager_, namespaces, true) ;      
      uicomp.setRenderedSibling(UINodeTypeForm.class) ;
    }
  }
  
  static public class EditActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypeList uicomp = (UINodeTypeList) event.getComponent() ;
      String nodeName = event.getParameter(OBJECTID) ;
      Node draftNodeType = 
        uicomp.session_.getRootNode().getNode("jcr:system/jcr:nodetypesDraft/" + nodeName) ;
      UINodeTypeForm nodeTypeForm = (UINodeTypeForm)uicomp.getSibling(UINodeTypeForm.class) ;
      String[] namespaces = uicomp.repository_.getNamespaceRegistry().getPrefixes() ;
      nodeTypeForm.updateEdit(draftNodeType, uicomp.ntManager_, namespaces, false, true) ;
      uicomp.setRenderedSibling(UINodeTypeForm.class) ;
    }
  }
  
  static public class DeleteActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypeList uicomp = (UINodeTypeList) event.getComponent() ;
      String nodeName = event.getParameter(OBJECTID) ;  
      Node draftNode = uicomp.session_.getRootNode().getNode("jcr:system/jcr:nodetypesDraft") ;
      Node deleteNode = draftNode.getNode(nodeName) ;
      deleteNode.remove() ;
      draftNode.save() ;
      if(!draftNode.hasNodes())
        draftNode.remove() ;
      uicomp.session_.save() ;
      uicomp.refresh(null);
    }
  }
  
  static public class BackActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UINodeTypeList uicomp = (UINodeTypeList) event.getComponent() ;
      uicomp.setRenderedSibling(UIECMAdminSummary.class) ;     
    }
  }
}
