package org.exoplatform.portlets.content.relations.component;
import java.util.List;

import javax.jcr.Node;

import org.exoplatform.faces.cms.component.JcrRelationBrowser;
import org.exoplatform.faces.core.Selector;
import org.exoplatform.faces.core.SelectorHandler;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.jcr.component.ChangeNodeListener;
import org.exoplatform.portlets.content.jcr.component.UIJCRExplorer;
import org.exoplatform.portlets.content.jcr.component.UIViewController.CancelActionListener;
import org.exoplatform.services.cms.CmsConfigurationService;
import org.exoplatform.services.cms.relations.RelationsService;

/**
 * @author monica franceschini
 */

public class UIRelationsList extends UIExoCommand implements ChangeNodeListener, SelectorHandler{
  
  final static public String ADD_ACTION = "add".intern();  
  
  public static Parameter[] ADD_PARAMS = { new Parameter(ACTION , ADD_ACTION)} ;
  public static Parameter[] CANCEL_PARAMS = { new Parameter(ACTION , CANCEL_ACTION)} ;
  public static Parameter[] DELETE_PARAMS = { new Parameter(ACTION , DELETE_ACTION)} ;

  private RelationsService relationsService_;
  private Node node_;
  private CmsConfigurationService cmsConfig_;
  private UIJCRExplorer uiExplorer_ ;
  
  public UIRelationsList(RelationsService relationsService, CmsConfigurationService cmsConfig) {
    setId("UIRelationsList") ;
    setRendererType("VelocityRenderer") ;
    
    relationsService_ = relationsService;        
    cmsConfig_ = cmsConfig;
    
    addActionListener(AddActionListener.class, ADD_ACTION) ;    
    addActionListener(DeleteActionListener.class, DELETE_ACTION) ;
    addActionListener(CancelActionListener.class,   CANCEL_ACTION) ;
  }

  public void  onChange(UIJCRExplorer uiExplorer, Node node)  throws Exception {
    setRendered(false) ;
    node_ = node ;
    uiExplorer_ = uiExplorer ;
  }      
  
  public Parameter[] getAddParams() { return ADD_PARAMS ; }
  public Parameter[] getDeleteParams() { return DELETE_PARAMS ; }
   public Parameter[] getBackParams() { return CANCEL_PARAMS ; }
  
  public boolean hasRelations() throws Exception{
     return relationsService_.hasRelations(node_);
  }
  
  public List<Node> getRelations() throws Exception{
    return relationsService_.getRelations(node_);
  }
  
  public Node getNode() {
    return node_;
  }
  
  public void cancel(Selector selector) {
    selector.setRenderedSibling(UIRelationsList.class);
  }

  public void select(Selector selector) throws Exception {
    String selectedNodePath = selector.getInputValue();
    relationsService_.addRelation(node_, selectedNodePath);
    if(!uiExplorer_.getPreference().isJcrEnable()) uiExplorer_.getSession().save() ;
    selector.setRenderedSibling(UIRelationsList.class);
  }      

  static public class AddActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIRelationsList uicomp = (UIRelationsList) event.getComponent() ;
      JcrRelationBrowser jcrBrowser = (JcrRelationBrowser) uicomp.getSibling(JcrRelationBrowser.class);
      jcrBrowser.setHandler(uicomp) ;
      jcrBrowser.setSession(uicomp.uiExplorer_.getSession());
      String workspace =  uicomp.cmsConfig_.getWorkspace();

      //String relationsPath = uicomp.cmsConfig_.getJcrPath(BasePath.CMS_PATH);
      jcrBrowser.setParameters(new String[] {workspace, "/"});
      uicomp.setRenderedSibling(JcrRelationBrowser.class);   
    }
  }       
  
  static public class DeleteActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
    	UIRelationsList uicomp = (UIRelationsList) event.getComponent() ;
      String name = event.getParameter(OBJECTID) ;
      Information iprovider = findInformationProvider(uicomp);
      try {
        uicomp.relationsService_.removeRelation(uicomp.node_, name);
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }      
    }
  }
    
}

