package org.exoplatform.portlets.content.rules.component;

import java.util.ResourceBundle;

import javax.faces.application.FacesMessage;
import javax.jcr.Node;
import javax.jcr.RepositoryException;
import javax.jcr.version.VersionException;
import javax.jcr.version.VersionHistory;

import org.exoplatform.faces.application.ExoFacesMessage;
import org.exoplatform.faces.core.component.UICheckBox;
import org.exoplatform.faces.core.component.UISimpleMulltipartForm;
import org.exoplatform.faces.core.component.UIStringInput;
import org.exoplatform.faces.core.component.UITextArea;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.portlets.content.versions.component.VersionNode;
import org.exoplatform.services.cms.rules.RuleService;

public class UIRuleForm extends UISimpleMulltipartForm {

  static String RULE_NAME = "rule".intern(), CONTENT = "content".intern();
  static String RULE_ENABLE_VERSIONING = "enableVersioning";
  static String RESTORE_VERSION_ACTION = "restoreVersion".intern() ;
  
  protected RuleService ruleService_;
  protected UIStringInput ruleNameInput_;
  protected UITextArea ruleContentInput_;
  protected UICheckBox ruleEnableVersioning_;
  protected VersionNode rootVersion_;
  protected VersionNode selectedVersion_;
  protected Node node_;
  protected boolean newRule_;
  private Node frozenNode_ ;

  public UIRuleForm(RuleService ruleService) {
    super("RuleForm", "post", null);
    setRendererType("SimpleFormVelocityRenderer");
    ruleService_ = ruleService;
    ruleNameInput_ = new UIStringInput(RULE_NAME, "");
    ruleContentInput_ = new UITextArea(CONTENT, "");
    ruleContentInput_.setClazz("large");        
    add(ruleNameInput_);
    add(ruleContentInput_);    
    ruleEnableVersioning_ = new UICheckBox(RULE_ENABLE_VERSIONING, null);    
    ruleEnableVersioning_.setSelect(false) ;
    add(ruleEnableVersioning_) ;    
    addActionListener(CancelActionListener.class, CANCEL_ACTION);
    addActionListener(SaveActionListener.class, SAVE_ACTION);
    addActionListener(RestoreVersionActionListener.class, RESTORE_VERSION_ACTION);    
  }

  public String getRestoreVersionAction() { return RESTORE_VERSION_ACTION; }
      
  public boolean canEnableVersionning() throws Exception {
    return node_.canAddMixin("mix:versionable");
  }
  
  public boolean isNewRule() throws Exception {
    return newRule_;
  }
    
  public boolean isVersioned() throws RepositoryException {    
    if (!newRule_) {     
      return node_.isNodeType("mix:versionable");
    }    
    return false;
  }
  
  public boolean isEnableRestoring() throws Exception {    
    if (isVersioned()) {
      long versionSize = node_.getVersionHistory().getAllVersions().getSize();
      if (versionSize > 2) return true;     
    }
    return false;
  }
  
  public VersionNode getRootVersion() throws Exception{        
    if (rootVersion_ == null) {
      VersionHistory vH;
      try {        
        vH = node_.getVersionHistory();
        rootVersion_ = new VersionNode(vH.getRootVersion());        
      } catch (RepositoryException e) {    
        return null;
      }      
    }            
    return rootVersion_;
  }
  
  public void setRule(String ruleName, VersionNode selectedVersion) throws Exception{        
    rootVersion_ = null;          
    String rule = null;
    if (ruleName != null) {            
      rule = ruleService_.getRuleAsText(ruleName);
      ruleNameInput_.setText(ruleName).setEditable(false);
      ruleContentInput_.setText(rule);
      node_ = ruleService_.getRuleNode(ruleName);
      newRule_ = false;
      if (isVersioned()) {                        
        if (selectedVersion != null) {
          selectedVersion_ = selectedVersion;
          frozenNode_ = selectedVersion_.getVersion().getNode("jcr:frozenNode") ;          
          rule = frozenNode_.getProperty("jcr:data").getString();                   
          ruleNameInput_.setText(ruleName).setEditable(false);          
          ruleContentInput_.setText(rule);                                      
        }        
        ruleEnableVersioning_.setSelect(true);
        ruleEnableVersioning_.setEditable(false);        
      } else if (canEnableVersionning()){       
        ruleEnableVersioning_.setSelect(false);
        ruleEnableVersioning_.setEditable(true);        
      }
    }else {
      ruleNameInput_.setText("").setEditable(true);
      ruleContentInput_.setText("");  
      newRule_ = true;
    }
  }
  
  static public class RestoreVersionActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIRuleForm uiForm = (UIRuleForm) event.getSource();        
      ResourceBundle res = getApplicationResourceBundle() ;      
      Information iprovider = findInformationProvider(uiForm);
      try {                      
        if (uiForm.selectedVersion_ != null) {
          uiForm.node_.restore(uiForm.selectedVersion_.getVersion(), true);
          uiForm.node_.checkout();
          Object[] args = {uiForm.selectedVersion_.getName()};        
          iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_INFO,
                        res.getString("UIRuleForm.msg.version-restored"), args));
        }
        UIRuleFormController ruleFormController = (UIRuleFormController) uiForm.getParent();            
        UIRuleList ruleList = (UIRuleList) ruleFormController.getSibling(UIRuleList.class);                                
        ruleList.refresh();
        ruleList.setRenderedSibling(UIRuleList.class);      
      } catch (VersionException ex) {
        iprovider.addMessage(new ExoFacesMessage(FacesMessage.SEVERITY_ERROR,ex.getMessage()));
      } catch (Exception e) {
        JCRExceptionManager.process(iprovider, e);
      }
    }
  }
  
  static public class SaveActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {      
      UIRuleForm uiForm = (UIRuleForm) event.getSource();
      ResourceBundle res = getApplicationResourceBundle() ;
      String name = uiForm.ruleNameInput_.getText();
      String text = uiForm.ruleContentInput_.getText();
      Information iprovider = findInformationProvider(uiForm);
      if (name == null || name.length() < 1) {
        iprovider.addMessage(new ExoFacesMessage(res.getString("UIRuleForm.msg.view-name-invalid"))) ;
        return ;
      }
      try {
        uiForm.ruleService_.addRule(name, text);
        Node node = uiForm.ruleService_.getRuleNode(name);                    
        if (!uiForm.newRule_) {          
          boolean isRuleEnableVersioning = uiForm.ruleEnableVersioning_.getSelect();
          if (isRuleEnableVersioning) {
            if (uiForm.canEnableVersionning() && !uiForm.isVersioned()) {              
              node.addMixin("mix:versionable");
              node.save();                    
            }                      
            node.checkin();
            node.checkout();                
            node.save();
          }
        }                
        UIRuleFormController ruleFormController = (UIRuleFormController) uiForm.getParent();            
        UIRuleList ruleList = (UIRuleList) ruleFormController.getSibling(UIRuleList.class);                                
        ruleList.refresh();
        ruleList.setRenderedSibling(UIRuleList.class);        
      } catch (Exception e) {             
        JCRExceptionManager.process(iprovider, e);
      }      
    }
  }

  static public class CancelActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIRuleForm uiForm = (UIRuleForm) event.getSource();          
      UIRuleFormController ruleFormController = (UIRuleFormController) uiForm.getParent();            
      UIRuleList ruleList = (UIRuleList) ruleFormController.getSibling(UIRuleList.class);         
      ruleList.setRenderedSibling(UIRuleList.class);        
    }
  }

}
