package org.exoplatform.portlets.content.rules.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.NodeIterator;
import javax.jcr.RepositoryException;

import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.model.PageListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portlets.content.ContentPageList;
import org.exoplatform.portlets.content.admin.component.UIECMAdminSummary;
import org.exoplatform.services.cms.rules.RuleService;

public class UIRuleList extends UIExoCommand{  
  final static public String ADD_ACTION = "add".intern();    
  public static Parameter[] ADD_PARAMS = { new Parameter(ACTION , ADD_ACTION)} ;
  public static Parameter[] BACK_PARAMS = { new Parameter(ACTION , BACK_ACTION)} ;
  public static Parameter[] EDIT_PARAMS = { new Parameter(ACTION , EDIT_ACTION)} ;
  public static Parameter[] DELETE_PARAMS = { new Parameter(ACTION , DELETE_ACTION)} ;
  private RuleService ruleService_;
  public UIPageListIterator uiPageIterator_ ;  
  
  public UIRuleList(RuleService ruleService) throws Exception{
    setId("UIRuleList") ;
    setRendererType("VelocityRenderer") ;    
    ruleService_ = ruleService;    
    uiPageIterator_ = new UIPageListIterator(new PageListDataHandler()) ;  
    getChildren().add(uiPageIterator_);
    addActionListener(AddActionListener.class, ADD_ACTION) ;
    addActionListener(BackActionListener.class, BACK_ACTION) ;
    addActionListener(EditActionListener.class, EDIT_ACTION) ;
    addActionListener(DeleteActionListener.class, DELETE_ACTION) ;
    uiPageIterator_.setPageList(new ContentPageList(getRules()));  
  }
  
  public Collection getPageListDataHandler() throws Exception{   
    return uiPageIterator_.getPageListDataHandler().getDatas();        
  }
  
  public Parameter[] getAddParams() { return ADD_PARAMS ; }
  public Parameter[] getBackParams() { return BACK_PARAMS ; }
  public Parameter[] getEditParams() { return EDIT_PARAMS ; }
  public Parameter[] getDeleteParams() { return DELETE_PARAMS ; }  
  
  public boolean hasRules() throws Exception{
     return ruleService_.hasRules();
  } 
  
  public List getRules() throws Exception{
    List list = new ArrayList() ; 
    NodeIterator nodes = ruleService_.getRules();
    while (nodes.hasNext()) {
      Node node = nodes.nextNode();      
      list.add(node);
    }
    return list;
  }  

  public void refresh() throws Exception{        
    uiPageIterator_.setPageList(new ContentPageList(getRules()));        
  }
  
  public boolean isVersionable(Node node) throws RepositoryException {
    return node.isNodeType("mix:versionable")
    && !node.isNodeType("nt:frozenNode");
  }
     
  public String getBaseVersion(Node node) throws RepositoryException {
    if(!isVersionable(node))
      return "";
    return node.getBaseVersion().getName();
  }
  
  static public class AddActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIRuleList uicomp = (UIRuleList) event.getComponent() ;
      UIRuleFormController ruleFormController = (UIRuleFormController) uicomp.getSibling(
          UIRuleFormController.class);
      ruleFormController.showNewRuleForm();
      uicomp.setRenderedSibling(UIRuleFormController.class);
    }
  }    
  
  static public class BackActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIRuleList uicomp = (UIRuleList) event.getComponent() ;
      uicomp.setRenderedSibling(UIECMAdminSummary.class);
    }
  }    

  static public class EditActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIRuleList uicomp = (UIRuleList) event.getComponent() ;
      String name = event.getParameter(OBJECTID) ;      
      UIRuleFormController ruleFormController = (UIRuleFormController) uicomp.getSibling(
          UIRuleFormController.class);
      ruleFormController.showEditRuleForm(name);      
      uicomp.setRenderedSibling(UIRuleFormController.class);
    }
  }        
  
  static public class DeleteActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIRuleList uicomp = (UIRuleList) event.getComponent() ;
      String name = event.getParameter(OBJECTID) ;
      uicomp.ruleService_.removeRule(name);   
      uicomp.refresh();
    }
  }        
}
