/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.script.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.RepositoryException;

import org.apache.commons.lang.StringUtils;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.model.PageListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.ContentPageList;
import org.exoplatform.portlets.content.admin.component.UIECMAdminSummary;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.services.cms.scripts.ScriptService;

/**
 * Created by The eXo Platform SARL
 * Author : HoaPham
 hoa.pham@exoplatform.com
 *          phamvuxuanhoa@yahoo.com
 * May 12, 2006
 */

public class UICBScriptList extends UIExoCommand {
  
  final static public String ADD_ACTION = "add".intern();
  final static public String CB_SCRIPT = "Content Browser script".intern() ;
  
  public static Parameter[] ADD_PARAMS = { new Parameter(ACTION , ADD_ACTION)} ;
  public static Parameter[] BACK_PARAMS = { new Parameter(ACTION , BACK_ACTION)} ;
  public static Parameter[] EDIT_PARAMS = { new Parameter(ACTION , EDIT_ACTION)} ;
  public static Parameter[] DELETE_PARAMS = { new Parameter(ACTION , DELETE_ACTION)} ;
  
  protected ScriptService scriptService_;
  private UIScriptFilterForm cbScriptFilter_ ;
  public UIPageListIterator uiPageIterator_ ;  
  
  public UICBScriptList(ScriptService scriptService) throws Exception {
    setId("CBScriptList") ;
    setRendererType("VelocityRenderer") ;
    scriptService_ = scriptService ;    
    cbScriptFilter_ = (UIScriptFilterForm)addChild(UIScriptFilterForm.class) ;        
    cbScriptFilter_.updateForm(CB_SCRIPT,getCBCategoryOptions()) ;
    uiPageIterator_ = new UIPageListIterator(new PageListDataHandler()) ;  
    getChildren().add(uiPageIterator_);
    addActionListener(AddActionListener.class, ADD_ACTION) ;
    addActionListener(BackActionListener.class, BACK_ACTION) ;
    addActionListener(EditActionListener.class, EDIT_ACTION) ;
    addActionListener(DeleteActionListener.class, DELETE_ACTION) ; 
    uiPageIterator_.setPageList(new ContentPageList(getScriptList()));     
  }
  
  public Collection getPageListDataHandler() throws Exception{   
    return uiPageIterator_.getPageListDataHandler().getDatas();        
  }
  
  public Parameter[] getBackParams() {return BACK_PARAMS ;}
  public Parameter[] getAddParams() {return ADD_PARAMS ;}
  public Parameter[] getEditParams() { return EDIT_PARAMS ; }
  public Parameter[] getDeleteParams() { return DELETE_PARAMS ; }  
    
  public String getScriptNamePrefix() {
    return cbScriptFilter_.getSelectedScriptPrefix() ;
  }    
  
  public boolean hasScripts() throws Exception{
    return scriptService_.hasCBScript();
  } 
  
  public List<SelectItem> getCBCategoryOptions() throws Exception {
    String cbScripPath = scriptService_.getCBScriptHome().getPath() ;
    String baseScriptPaht = scriptService_.getBaseScriptPath() ;
    String correlativeCBPath = StringUtils.substringAfter(cbScripPath,baseScriptPaht + "/")  ;
    List<SelectItem> cbOptions = new ArrayList<SelectItem>() ;
    cbOptions.add(new SelectItem(CB_SCRIPT,correlativeCBPath)) ;
    return cbOptions ;
  }
  
  public List getScriptList() throws Exception{
    return scriptService_.getCBScripts();    
  }
  
  public void refresh() throws Exception{        
    uiPageIterator_.setPageList(new ContentPageList(getScriptList()));         
  }
  
  public boolean isVersionable(Node node) throws RepositoryException {
    return node.isNodeType("mix:versionable")
    && !node.isNodeType("nt:frozenNode");
  }
     
  public String getBaseVersion(Node node) throws RepositoryException {
    if(!isVersionable(node))
      return "";
    return node.getBaseVersion().getName();
  }
  
  static public class AddActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICBScriptList uicomp = (UICBScriptList) event.getComponent() ;      
      List<SelectItem> options = uicomp.getCBCategoryOptions();
      String selectedNamePrefix = uicomp.getScriptNamePrefix() ;     
      UIScriptFormController scriptFormController = (UIScriptFormController) uicomp.getSibling(
          UIScriptFormController.class);
      scriptFormController.showNewScriptForm(false, options, selectedNamePrefix);
      uicomp.setRenderedSibling(UIScriptFormController.class);
    }    
  }    
  
  static public class BackActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {      
      UICBScriptList uicomp = (UICBScriptList) event.getComponent() ;
      UIScriptsManager uimanager = (UIScriptsManager)uicomp.getAncestorOfType(UIScriptsManager.class) ;
      uimanager.setRenderedSibling(UIECMAdminSummary.class);
    }
  }    
  
  static public class EditActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {    
      UICBScriptList uicomp = (UICBScriptList) event.getComponent() ;
      List<SelectItem> options = uicomp.getCBCategoryOptions();
      String scriptName = event.getParameter(OBJECTID) ;      
      String selectedNamePrefix = uicomp.getScriptNamePrefix() ;      
      UIScriptFormController scriptFormController = (UIScriptFormController) uicomp.getSibling(
          UIScriptFormController.class);
      scriptFormController.showEditScriptForm(false,scriptName, options, selectedNamePrefix);      
      uicomp.setRenderedSibling(UIScriptFormController.class); 
    }
  }        
  
  static public class DeleteActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UICBScriptList uicomp = (UICBScriptList) event.getComponent() ;
      String scriptName = event.getParameter(OBJECTID) ;
      String selectedNamePrefix = uicomp.getScriptNamePrefix() ;       
      try {             
        String name2remove = selectedNamePrefix + "/" + scriptName ;
        uicomp.scriptService_.removeScript(name2remove) ;
      }catch (Exception e) {
        e.printStackTrace() ;
        Information iprovider = findInformationProvider(uicomp);
        JCRExceptionManager.process(iprovider,e) ;
      }
      uicomp.refresh();
    }
  }
}
