/***************************************************************************
 * Copyright 2001-2003 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/
package org.exoplatform.portlets.content.script.component;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.jcr.Node;
import javax.jcr.RepositoryException;

import org.apache.commons.lang.StringUtils;
import org.exoplatform.faces.core.component.UIExoCommand;
import org.exoplatform.faces.core.component.UIPageListIterator;
import org.exoplatform.faces.core.component.model.PageListDataHandler;
import org.exoplatform.faces.core.component.model.Parameter;
import org.exoplatform.faces.core.component.model.SelectItem;
import org.exoplatform.faces.core.event.ExoActionEvent;
import org.exoplatform.faces.core.event.ExoActionListener;
import org.exoplatform.portal.Information;
import org.exoplatform.portlets.content.ContentPageList;
import org.exoplatform.portlets.content.admin.component.UIECMAdminSummary;
import org.exoplatform.portlets.content.jcr.JCRExceptionManager;
import org.exoplatform.services.cms.scripts.ScriptService;

/**
 * Created by The eXo Platform SARL
 * Author : HoaPham
 hoa.pham@exoplatform.com
 *          phamvuxuanhoa@yahoo.com
 * May 12, 2006
 */

public class UIECMScriptList extends UIExoCommand {
  
  final static public String ADD_ACTION = "add".intern();
  final static public String INTERCEPTOR_SCRIPT = "ECM Interceptor".intern() ;
  final static public String WIDGET_SCRIPT = "ECM Widget".intern() ;
  final static public String ACTION_SCRIPT = "ECM Action".intern() ;
  
  public static Parameter[] ADD_PARAMS = { new Parameter(ACTION , ADD_ACTION)} ;
  public static Parameter[] BACK_PARAMS = { new Parameter(ACTION , BACK_ACTION)} ;
  public static Parameter[] EDIT_PARAMS = { new Parameter(ACTION , EDIT_ACTION)} ;
  public static Parameter[] DELETE_PARAMS = { new Parameter(ACTION , DELETE_ACTION)} ;
  
  protected ScriptService scriptService_;
  private UIScriptFilterForm ecmScriptFilter_ ;  
  public UIPageListIterator uiPageIterator_ ;  
  
  public UIECMScriptList(ScriptService scriptService) throws Exception {
    setId("ECMScriptList") ;
    setRendererType("VelocityRenderer") ;
    scriptService_ = scriptService ;   
    ecmScriptFilter_ = (UIScriptFilterForm)addChild(UIScriptFilterForm.class) ;   
    ecmScriptFilter_.updateForm(ACTION_SCRIPT,getECMOptions()) ;
    uiPageIterator_ = new UIPageListIterator(new PageListDataHandler()) ;  
    getChildren().add(uiPageIterator_);
    addActionListener(AddActionListener.class, ADD_ACTION) ;
    addActionListener(BackActionListener.class, BACK_ACTION) ;
    addActionListener(EditActionListener.class, EDIT_ACTION) ;
    addActionListener(DeleteActionListener.class, DELETE_ACTION) ;   
    uiPageIterator_.setPageList(new ContentPageList(getScriptList()));   
  }
  
  public Collection getPageListDataHandler() throws Exception{   
    return uiPageIterator_.getPageListDataHandler().getDatas();        
  }
  
  public Parameter[] getBackParams() {return BACK_PARAMS ;}
  public Parameter[] getAddParams() {return ADD_PARAMS ;}
  public Parameter[] getEditParams() { return EDIT_PARAMS ; }
  public Parameter[] getDeleteParams() { return DELETE_PARAMS ; }  
  
  public String getScriptNamePrefix() {
    return ecmScriptFilter_.getSelectedScriptPrefix() ;
  }
  
  public boolean hasScripts() throws Exception {
    String selectedType = ecmScriptFilter_.getSelectedScriptType() ;
    if(selectedType.equals(ACTION_SCRIPT)) {
      return scriptService_.getECMActionScriptHome().hasNodes() ;
    }else if(selectedType.equals(WIDGET_SCRIPT)) {
      return scriptService_.getECMWidgetScriptHome().hasNodes() ;
    }else if(selectedType.equals(INTERCEPTOR_SCRIPT)) {
      return scriptService_.getECMWidgetScriptHome().hasNodes() ;
    }
    return false ;
  } 
  
  public List getScriptList() throws Exception {
    String selectedType = ecmScriptFilter_.getSelectedScriptType() ;
    if(selectedType.equals(ACTION_SCRIPT)){      
      return scriptService_.getECMActionScripts() ;
    }else if(selectedType.equals(INTERCEPTOR_SCRIPT)) {      
      return scriptService_.getECMInterceptorScripts() ;
    }else if(selectedType.equals(WIDGET_SCRIPT)) {      
      return scriptService_.getECMWidgetScripts() ;
    }
    return new ArrayList<Node>() ;
  }
  
  public void refresh() throws Exception{          
    uiPageIterator_.setPageList(new ContentPageList(getScriptList()));         
  }
  
  public boolean isVersionable(Node node) throws RepositoryException {
    return node.isNodeType("mix:versionable")
    && !node.isNodeType("nt:frozenNode");
  }
     
  public String getBaseVersion(Node node) throws RepositoryException {
    if(!isVersionable(node))
      return "";
    return node.getBaseVersion().getName();
  }
  
  public List<SelectItem> getECMOptions() throws Exception {
    String baseScripPath = scriptService_.getBaseScriptPath() ;
    String actionPath = scriptService_.getECMActionScriptHome().getPath() ;
    String interceptorPath = scriptService_.getECMInterceptorScriptHome().getPath() ;
    String widgetPath = scriptService_.getECMWidgetScriptHome().getPath() ;
    String actionPrefix = StringUtils.substringAfter(actionPath,baseScripPath + "/") ;
    String interceptorPrefix = StringUtils.substringAfter(interceptorPath,baseScripPath + "/") ;
    String widgetPrefix = StringUtils.substringAfter(widgetPath,baseScripPath + "/") ;
    List<SelectItem> ecmOptions = new ArrayList<SelectItem>() ;
    ecmOptions.add(new SelectItem(ACTION_SCRIPT,actionPrefix));
    ecmOptions.add(new SelectItem(INTERCEPTOR_SCRIPT,interceptorPrefix));
    ecmOptions.add(new SelectItem(WIDGET_SCRIPT,widgetPrefix));
    return ecmOptions ; 
  }
  
  static public class AddActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIECMScriptList uicomp = (UIECMScriptList) event.getComponent() ; 
      List<SelectItem> options = uicomp.getECMOptions();
      String selectedNamePrefix = uicomp.getScriptNamePrefix() ;
      UIScriptFormController scriptFormController = (UIScriptFormController) uicomp.getSibling(
          UIScriptFormController.class);
      scriptFormController.showNewScriptForm(true, options, selectedNamePrefix);
      uicomp.setRenderedSibling(UIScriptFormController.class);
    }
  }    
  
  static public class BackActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIECMScriptList uicomp = (UIECMScriptList) event.getComponent() ;
      UIScriptsManager uimanager = (UIScriptsManager)uicomp.getAncestorOfType(UIScriptsManager.class) ;
      uimanager.setRenderedSibling(UIECMAdminSummary.class);
    }
  }    
  
  static public class EditActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIECMScriptList uicomp = (UIECMScriptList) event.getComponent() ;
      List<SelectItem> options = uicomp.getECMOptions();
      String scriptName = event.getParameter(OBJECTID) ;
      String selectedNamePrefix = uicomp.getScriptNamePrefix() ;                  
      UIScriptFormController scriptFormController = (UIScriptFormController) uicomp.getSibling(
          UIScriptFormController.class);
      scriptFormController.showEditScriptForm(true,scriptName, options, selectedNamePrefix);      
      uicomp.setRenderedSibling(UIScriptFormController.class); 
    }
  }        
  
  static public class DeleteActionListener extends ExoActionListener {
    public void execute(ExoActionEvent event) throws Exception {
      UIECMScriptList uicomp = (UIECMScriptList) event.getComponent() ;
      String scriptName = event.getParameter(OBJECTID) ;
      String scriptTypeValue = uicomp.getScriptNamePrefix() ;            
      try {                
        String name2remove = scriptTypeValue + "/" + scriptName ;        
        uicomp.scriptService_.removeScript(name2remove) ;
      }catch (Exception e) {
        e.printStackTrace() ;
        Information iprovider = findInformationProvider(uicomp);
        JCRExceptionManager.process(iprovider,e) ;
      }
      uicomp.refresh();
    }
  }
  
}
